/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#include "DialogOptions.h"
#include <iostream>

DialogOptions::DialogOptions() : Gtk::Window() {

	row = -1;
	Gtk::Box* tableItem = new Gtk::Box(Gtk::Orientation::HORIZONTAL);
	Gtk::Label* lbltableName = new Gtk::Label(lang->getText("txtDialog_Structure"));
	Gtk::Label* lblColName = new Gtk::Label(lang->getText("txtDialog_Atribut"));

	btnAddColumn->set_label(lang->getText("txtDialog_btnAddCol"));

	tableItem->append(*lbltableName);
	tableItem->append(*cmbTemplate);
	tableItem->append(*lblColName);
	tableItem->append(*cmbCol1);
	tableItem->append(*cmbCol2);
	tableItem->append(*cmbCol3);

	cmbTemplate->setOnChange(&DialogOptions::cmbTemplateOnChange,this);
	optItems->append(*tableItem);
	btnAdd->set_label(lang->getText("txtDialog_Add"));
	btnAdd->signal_clicked().connect(sigc::mem_fun(*this, &DialogOptions::btnAddOnClick));


	btnSave->set_label(lang->getText("txtDialog_btnSave"));
	btnSave->signal_clicked().connect(sigc::mem_fun(*this, &DialogOptions::btnSaveOnClick));

	addSaveBox->append(*btnAdd);
	addSaveBox->append(*btnSave);
	mainBox->append(*addSaveBox);

	this->set_child(*mainBox);

}

DialogOptions::~DialogOptions() {

}

void DialogOptions::setCmbBoxColumns(std::string &templ,rhea::ComboBox<StringColumns>* cmbCol) {

	int i = templ.find(global->gridTemplateColDelimiter);

	int col = 0;
	if (i <= templ.size()) {
		col = std::stoi(templ.substr(0,i));
	} else if (templ.size()>0 && global->isNumber(templ)){
		col = std::stoi(templ);
	} else {
		col=0;
	}

	for (auto row: columnList->children()) {
		if(col ==row[stringColumns.col_id]){
			cmbCol->set_active(row.get_iter());
		}
	}

}

void DialogOptions::fill(std::vector<std::string*> templateOptions, std::vector<std::string>* values, Gtk::Button* btn) {

	returnBtn = btn;
	myTemplates = templateOptions;
	options = values;
	templateList->clear();

	auto rowObject = *templateList->append();
	rowObject[stringColumns.col_id] = 0;
	rowObject[stringColumns.col_name] = "";

	for (std::string* val : myTemplates) {

		std::string templ = val->substr(global->gridTemplateDelimiter.size());
		size_t i = templ.find(global->gridTemplateDelimiter);
		std::string name = templ.substr(0,i);
		templ = templ.substr(i+global->gridTemplateDelimiter.size());
		i = templ.find(global->gridTemplateDelimiter);
		templ = templ.substr(0,i);
		int myrow = std::stoi(templ);
		rowObject = *templateList->append();
		rowObject[stringColumns.col_id] = myrow;
		rowObject[stringColumns.col_name] = name;


	}
	cmbTemplate->addList(&templateList);

	cmbCol1->addList(&columnList);
	cmbCol2->addList(&columnList);
	cmbCol3->addList(&columnList);

	for (std::string val : *values) {
		if (val.starts_with(global->gridColumnDelimiter)){

			val = val.substr(global->gridColumnDelimiter.size());
			std::vector<std::string> cols = {};
			global->parseOptions(val,global->gridColumnDelimiter,cols);
			for (auto& str :cols) {
				Gtk::Box* colItem = new Gtk::Box(Gtk::Orientation::HORIZONTAL);
				Gtk::TextView* tv = new Gtk::TextView();
				tv->set_size_request(250, 35);
				tv->set_editable(true);
				tv->set_focusable(true);
				tv->set_focus_on_click(true);
				tv->get_buffer()->set_text(str);
				gridColumns.push_back(tv);

				Gtk::Button* btn = new Gtk::Button();
				btn->set_label(lang->getText("txtDialog_btnRemove"));
				btn->signal_clicked().connect(sigc::bind(sigc::mem_fun(*this, &DialogOptions::btnRemoveOnClick),btn,tv,row));
				colItem->append(*tv);
				colItem->append(*btn);
				colItem->insert_before(*columnsBox,*btnAddColumn);
			}

		} else if (val.starts_with(global->gridTemplateDelimiter)){
			std::string templ = val.substr(global->gridTemplateDelimiter.size());
			size_t i = templ.find(global->gridTemplateDelimiter);
			std::string pojo = templ.substr(0,i);

			int myPojo = std::stoi(pojo);
			fillColumnList(myPojo);

			for (auto row: templateList->children()) {
				if(myPojo ==row[stringColumns.col_id]){
					cmbTemplate->set_active(row.get_iter());
					break;
				}
			}

			if (i+global->gridTemplateDelimiter.size() >= templ.size()) {continue;}
			templ =templ.substr(i+global->gridTemplateDelimiter.size());
			setCmbBoxColumns(templ,cmbCol1);
			i = templ.find(global->gridTemplateColDelimiter);

			if (i+global->gridTemplateColDelimiter.size()>=templ.size()) {continue;}
			templ =templ.substr(i+global->gridTemplateColDelimiter.size());
			setCmbBoxColumns(templ,cmbCol2);
			i = templ.find(global->gridTemplateColDelimiter);

			if (i+global->gridTemplateColDelimiter.size()>=templ.size()) {continue;}
			templ =templ.substr(i+global->gridTemplateColDelimiter.size());
			setCmbBoxColumns(templ,cmbCol3);

		} else {
			row++;
			Gtk::Box* optItem = new Gtk::Box(Gtk::Orientation::HORIZONTAL);
			Gtk::TextView* tv = new Gtk::TextView();
			tv->set_size_request(250, 35);
			tv->set_editable(true);
			tv->set_focusable(true);
			tv->set_focus_on_click(true);
			tv->get_buffer()->set_text(val);
			items.push_back(tv);

			Gtk::Button* btn = new Gtk::Button();
			btn->set_label(lang->getText("txtDialog_btnRemove"));
			btn->signal_clicked().connect(sigc::bind(sigc::mem_fun(*this, &DialogOptions::btnRemoveOnClick),btn,tv,row));
			optItem->append(*tv);
			optItem->append(*btn);
			optItems->append(*optItem);
		}
	}
	mainBox->prepend(*optItems);
}

void DialogOptions::fillColumnList (int id){

	columnList->clear();
	auto rowObject = *columnList->append();
	rowObject[stringColumns.col_id] = 0;
	rowObject[stringColumns.col_name] = "";


	for (std::string* val : myTemplates){
		if ( !val->starts_with(global->gridTemplateDelimiter) ) {continue;}
		std::string templ = val->substr(global->gridTemplateDelimiter.size());
		size_t postempl = templ.find(global->gridTemplateDelimiter);
		std::string ime = templ.substr(0,postempl);
		templ = templ.substr(ime.size()+global->gridTemplateDelimiter.size());
		postempl = templ.find(global->gridTemplateDelimiter);
		long myId = std::stol(templ.substr(0,postempl));

		if (id != myId) {continue;}

		templ = templ.substr(postempl+global->gridTemplateDelimiter.size());
		size_t pos = 1;

		while (true ) {
			pos = templ.find(global->gridTemplateColDelimiter);
			std::string colId = templ.substr(0,pos);
			templ = templ.substr(colId.size()+global->gridTemplateColDelimiter.size());
			pos = templ.find(global->gridTemplateColDelimiter);
			std::string colName = templ.substr(0,pos);

			rowObject = *columnList->append();
			rowObject[stringColumns.col_id] = std::stoi(colId);
			rowObject[stringColumns.col_name] = colName;

			if(pos+global->gridTemplateColDelimiter.size()<=templ.size()){
				templ = templ.substr(pos+global->gridTemplateColDelimiter.size());
			}else{
				break;
			}
		}
	}

}

void DialogOptions::cmbTemplateOnChange( rhea::ComboBox<StringColumns>* cmb, Gtk::TreeRow oldChoice, Gtk::TreeRow newChoice) {

	if (newChoice){
		fillColumnList(newChoice[stringColumns.col_id]);
	}
}

void DialogOptions::btnRemoveOnClick(Gtk::Button *btn, Gtk::TextView *tv, int row){
	btn->get_parent()->hide();
}

void DialogOptions::btnAddOnClick(){
	row++;

	Gtk::Box* optItem = new Gtk::Box(Gtk::Orientation::HORIZONTAL);
	Gtk::TextView* tv = new Gtk::TextView();
	tv->set_size_request(250, 35);
	tv->set_editable(true);
	tv->set_focusable(true);
	tv->set_focus_on_click(true);
	items.push_back(tv);

	Gtk::Button* btn = new Gtk::Button();
	btn->set_label(lang->getText("txtDialog_btnRemove"));
	btn->signal_clicked().connect(sigc::bind(sigc::mem_fun(*this, &DialogOptions::btnRemoveOnClick),btn,tv,row));
	optItem->append(*tv);
	optItem->append(*btn);
	optItems->append(*optItem);

}

void DialogOptions::btnSaveOnClick(){

	std::string tmp = "";
	options->clear();
    Glib::ustring myTempl ="";

	std::string columns = "";
	for (Gtk::TextView* txt : gridColumns){
		if (txt->is_visible()){
			columns += global->gridColumnDelimiter + txt->get_buffer()->get_text();
		}
	}

	if (!columns.empty()){
		options->push_back(columns);
	}

	Gtk::TreeRow choice = *cmbTemplate->get_active();
	long id = choice[stringColumns.col_id];
	if (id >0){
		myTempl = choice[stringColumns.col_name];
		std::string columns = "";

		choice = *cmbCol1->get_active();
		int col = choice[stringColumns.col_id];
		if (col > 0) {
			columns += std::to_string(col) + global->gridTemplateColDelimiter;
		}

		choice = *cmbCol2->get_active();
		col = choice[stringColumns.col_id];
		if (col > 0) {
			columns += std::to_string(col) + global->gridTemplateColDelimiter;
		}

		choice = *cmbCol3->get_active();
		col = choice[stringColumns.col_id];
		if (col > 0) {
			columns += std::to_string(col) + global->gridTemplateColDelimiter;
		}

		if (!columns.empty()) {
			columns = columns.substr(0,columns.size()-global->gridTemplateColDelimiter.size());
		}

		const Gtk::TreeRow choice = *cmbTemplate->get_active();
		long id = choice[stringColumns.col_id];

		std::string selected = global->gridTemplateDelimiter
							 + std::to_string(id)
							 + global->gridTemplateDelimiter
							 + columns;

		options->push_back(selected);
		tmp = myTempl;
	}

	tmp += tmp.empty() ? "" : "\n";

	for (Gtk::TextView* txt : items){
		if (txt->is_visible()){
			options->push_back(txt->get_buffer()->get_text());
			tmp += txt->get_buffer()->get_text() + "\n";
		}
	}

	returnBtn->set_label(tmp);
	close();
}
