/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#include "FrameBase.h"
#include "iostream"

template <class T> FrameBase<T>::FrameBase(): Gtk::Frame() {

	std::string cssGrid =".rhea_gridHeaderLbl {background-color: #5391c1; color: #FFFFFF; border-color: black; border-style: solid solid solid solid;border-width: 0.5px;}";
	cssGrid += ".rhea_gridRowEven {background-color: #FFFFFF; color:black; border-color: black; border-style: none none  solid solid; border-width: 0.7px;}";
	cssGrid += ".rhea_gridRowEven .text {background-color: #FFFFFF; color:black; border-color: black; border-style: none none solid solid; border-width: 0.7px;}";
	cssGrid += ".rhea_gridRowOdd {background-color: #E0E0E0; color:black; border-color: black;border-style: none none solid solid;border-width: 0.7px;}";
	cssGrid += ".rhea_gridRowOdd .text {background-color: #E0E0E0; color:black; border-color: black; border-style: none none solid solid; border-width: 0.7px;}";
//	cssGrid += ".rhea_selectedRow, .rhea_selectedRow .text {background-color: #466176; color: #FFFFFF;}";
    cssGrid += ".rhea_selectedRow, .rhea_selectedRow .view, .rhea_selectedRow .view text, .rhea_selectedRow .btnGrid, {caret-color: white; background-image: none; background-color: #466176; color: #FFFFFF;} ";
	cssGrid += ".rhea_gridRowEven:hover, .rhea_gridRowOdd:hover {background-color: #D0D0EF;}";

	listingGrid->addCSS(cssGrid);
	addCSS(".formButton {border-radius:7px;} .listingBtn{ margin-top:10px; margin-left:7px;margin-bottom:5px; margin-right:7px;}");


	btnImport->add_css_class("formButton");
	btnExport->add_css_class("formButton");
	btnOneDoc->add_css_class("formButton");
	btnImport->add_css_class("listingBtn");
	btnExport->add_css_class("listingBtn");
    btnOneDoc->add_css_class("listingBtn");
	btnNew->add_css_class("formButton");
	btnDelete->add_css_class("formButton");
	btnSave->add_css_class("formButton");
    btnPrev->add_css_class("formButton");
    btnNext->add_css_class("formButton");

	btnImport->get_style_context()->add_provider_for_display(display,cssProvider,GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
	btnExport->get_style_context()->add_provider_for_display(display,cssProvider,GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
	btnOneDoc->get_style_context()->add_provider_for_display(display,cssProvider,GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
	btnNew->get_style_context()->add_provider_for_display(display,cssProvider,GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
	btnDelete->get_style_context()->add_provider_for_display(display,cssProvider,GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
	btnSave->get_style_context()->add_provider_for_display(display,cssProvider,GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
    btnPrev->get_style_context()->add_provider_for_display(display,cssProvider,GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
    btnNext->get_style_context()->add_provider_for_display(display,cssProvider,GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);

	btnImport->set_label("Import");
    btnExport->set_label("Export");
    btnOneDoc->set_label("1 Doc");

	btnImport->set_tooltip_text("Import");;
    btnExport->set_tooltip_text("Export");
    btnOneDoc->set_tooltip_text("Generate all in one doc.");

	btnImport->signal_clicked().connect(sigc::mem_fun(*this, &FrameBase::onImport));
    btnExport->signal_clicked().connect(sigc::mem_fun(*this, &FrameBase::onExport));
    btnOneDoc->signal_clicked().connect(sigc::mem_fun(*this, &FrameBase::onOneDoc));

    listingBottom->append(*btnImport);
    listingBottom->append(*btnExport);
    listingBottom->append(*btnOneDoc);
    
    listingBox->append(*listingTop);
    listingBox->append(*listingGrid);
	listingBox->append(*listingBottom);

    tvVersion->set_size_request(40,30);
    tvVersion->set_accepts_tab(false);
    tvVersion->set_editable(false);

    lblVersion->set_name(lblVersion_NAME);
    lblValid->set_name(lblValid_NAME);

    editorVersion->put(*btnNew,0,3);
    btnNew->set_size_request(90,40);
    
    editorVersion->put(*lblVersion,105,10);
    editorVersion->put(*btnPrev,175,5);
    editorVersion->put(*btnNext,260,5);
    editorVersion->put(*tvVersion,215,7);
    editorVersion->put(*lblValid,520,10);
    lblValid->hide();

    btnPrev->signal_clicked().connect(sigc::mem_fun(*this, &FrameBase::onPrevVersion));
    btnNext->signal_clicked().connect(sigc::mem_fun(*this, &FrameBase::onNextVersion));
    btnPrev->set_label("<");
    btnNext->set_label(">");
    btnPrev->set_image_from_icon_name("pan-start"); //"arrow-left"
    btnNext->set_image_from_icon_name("pan-end"); //"arrow-right"
    btnPrev->set_tooltip_text("Predhodna verzija");
    btnNext->set_tooltip_text("Naslednja verzija");

	btnPrev->set_sensitive(false);
    btnNext->set_sensitive(false);

    btnNew->set_name(btnEditorNew_NAME);
    btnNew->signal_clicked().connect(sigc::mem_fun(*this, &FrameBase::newPo));
    btnNew->set_tooltip_text("Novi element");
    btnNew->set_label("Novi");
//    btnNew->set_image_from_icon_name("document-new");
    btnDelete->set_name(btnEditorDelete_NAME);
    btnDelete->signal_clicked().connect(sigc::mem_fun(*this,&FrameBase::deletePo));
    btnDelete->set_label("Izbriši");
    btnDelete->set_tooltip_text("Izbriši element");
    btnSave->set_name(btnEditorSave_NAME);
    btnSave->signal_clicked().connect(sigc::mem_fun(*this,&FrameBase::onSave));
    btnSave->set_image_from_icon_name("document-save");
	btnSave->set_tooltip_text("Shrani element");
	btnSave->set_label("Shrani");
	btnSave->set_sensitive(false);

    editorBtns->put(*btnDelete,3,5);
    btnDelete->set_size_request(115,50);
    editorBtns->put(*btnSave,285,5);
    btnSave->set_size_request(115,50);

    editorBox->append(*editorVersion);
    editorBox->append(*editorData);
    editorBox->append(*editorBtns);
    editorBox->append(*editorGrids);

 //   editorBox->set_size_request(650,-1);

    mainBox->append(*listingBox);
    mainBox->append(*editorBox);
//    mainBox->append(*printBox);

//	auto fileCSS = Gtk::CssProvider::create();
//	std::string data =".gridHeaderCell {background-color: #0000FF;}";
//	fileCSS->load_from_data(data);
//  auto ctx = listingGrid->get_style_context();
//	ctx->add_class("filesHeader");
//	ctx->add_provider(fileCSS, GTK_STYLE_PROVIDER_PRIORITY_USER);
	

	
    set_child(*mainBox);

    cssProvider->load_from_data(cssData);
}

template <typename T> FrameBase<T>::~FrameBase() {

}

template <typename T> void FrameBase<T>::translateFrame(Gtk::Widget* frm) {


    auto child = frm->get_first_child();
    auto lastChild = frm->get_last_child();

    bool doExecute = child != nullptr;


    while (doExecute){
        std::string tmpStr = child->get_name();
        auto tmpObj = child->get_first_child();
        if(tmpObj) {
            translateFrame(child);
        }

        if (tmpStr.ends_with("_RHEA")){
            if (child == dynamic_cast<Gtk::Notebook*>(child)){
                auto tmpObj =  dynamic_cast<Gtk::Notebook*>(child);
                for (int i = 0 ;i<tmpObj->get_n_pages();i++) {
                    tmpObj->set_tab_label_text(*tmpObj->get_nth_page(i),
                                               langPack->getText(tmpObj->get_nth_page(i)->get_name()));
                }
            }

            if (child == dynamic_cast<Gtk::Label*>(child)){
                auto tmpObj =  dynamic_cast<Gtk::Label*>(child);
                tmpObj->set_label(langPack->getText(tmpObj->get_name()));
            }

            if (child == dynamic_cast<Gtk::Button*>(child)){
                auto tmpObj = dynamic_cast<Gtk::Button*>(child);
                tmpObj->set_label(langPack->getText(tmpObj->get_name()));
            }

        }

        if (child != lastChild) {
            doExecute = true;
            child = frm->get_next_sibling();
        }else{
            doExecute= false;
        }
    }

}

template <typename T> void FrameBase<T>::getVersions(T* obj) {
    poVersions.clear();
    populateVersions(obj, poVersions);
    version = poVersions.size();
    setVersionGui();
    populateGUI();
}

template <typename T> void FrameBase<T>::setVersionGui() {
    btnPrev->set_sensitive(!poVersions.empty() && version > 1);
    btnNext->set_sensitive(!poVersions.empty() && version < poVersions.size());
   	if (btnNext->get_sensitive()) {
       	lblValid->show();
   	}else{
       	lblValid->hide();
   	}
	lblValid->set_text("");
	tvVersion->get_buffer()->set_text("");
	
	if(!poVersions.empty()) {
    	po = dynamic_cast<T*>(poVersions.at(version - 1));
    	Po* tmp = dynamic_cast<Po*>(po);
	    lblValid->set_text(tmp->validTill->getAsString(langPack->getLanguage()));
    	tvVersion->get_buffer()->set_text(std::to_string(tmp->getPojoVersion()));
	}
	
    populateGUI();
}

template <typename T> void FrameBase<T>::onPrevVersion() {
    version = version == 1 ? 1 : version - 1;
    setVersionGui();
}

template <typename T> void FrameBase<T>::onNextVersion() {
    version = version < poVersions.size() ? version+1 : poVersions.size();
    setVersionGui();
}

template<class T> void FrameBase<T>::newPo() {
    if (po && (po->getPojoId()==-1)) { delete(po); }
    po = new T();
    onNew();
    getVersions(po);
    btnSave->set_sensitive(true);
}

template<class T> void FrameBase<T>::deletePo(){
  onDelete();
  listingGrid->setRows(mapOfPo.size());
  listingGrid->refresh();
  newPo();
}

template<class T> void FrameBase<T>::onImport() {
    fillImport();	
}

template<class T> void FrameBase<T>::onExport() {
	fillExport();
}

template<class T> void FrameBase<T>::onOneDoc() {

}

template<class T> void FrameBase<T>::addCSS(std::string css) {
	cssData += css;
}
