/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#ifndef GUI_FRAMEBASE_H
#define GUI_FRAMEBASE_H

#include "gtkmm.h"
#include "main/LanguagePack.h"
#include "main/main.h"
#include "po/Po.h"
#include "rhea/Grid/Grid.h"
#include "rhea/SqlLite/DbConn.h"
#include "rhea/ListedMap/ListedMap.h"

struct ObjectArea {
    const int X      = 0;
    const int Y      = 0;
    const int width  = 0;
    const int height = 0;
};

template <class T> class FrameBase : public Gtk::Frame {
protected:
    const int editorWidth = 500;
    std::string cssData ="";
    T* po = {nullptr};
    T* poFromListing = {nullptr};
    ObjectArea* txtSize = new ObjectArea {120, 120, 200, 30};
    ObjectArea* lblSize = new ObjectArea {5, 5, 110, 30};

    const std::string lblValid_NAME         = {"lblPoValid"};
    const std::string lblVersion_NAME       = {"lblVersion_RHEA"};
    const std::string btnPrevVer_NAME      = {"txtPrevVer_RHEA"};
    const std::string btnNextVer_NAME      = {"txtNextVer_RHEA"};
    const std::string btnEditorNew_NAME    = {"txtEditorNew_RHEA"};
    const std::string btnEditorSave_NAME   = {"txtEditorSave_RHEA"};
    const std::string btnEditorDelete_NAME = {"txtEditorDelete_RHEA"};
    const std::string lblComment_NAME       = {"lblComment_RHEA"};
    const std::string lblSuperior_NAME     = {"txtSuperior_RHEA"};
    const std::string lblSubordinate_NAME  = {"txtSubordinate_RHEA"};
    LanguagePack* langPack = LanguagePack::getInstance();
    rhea::DbConn* dbConn = rhea::DbConn::getInstance();
    Glib::RefPtr<Gdk::Display> display = Gdk::Display::get_default();
    Glib::RefPtr<Gtk::CssProvider> cssProvider = Gtk::CssProvider::create();

    void translateFrame(Gtk::Widget* frm);
    virtual std::string dataListing (int col,int row) = 0;

public:
    int version = {1};
    std::vector<T*> poVersions         = {};
    rhea::ListedMap<std::string, T*> mapOfPo  = {};
    Gtk::Box*      mainBox       = new Gtk::Box(Gtk::Orientation::HORIZONTAL);
    Gtk::Box*      listingBox    = new Gtk::Box(Gtk::Orientation::VERTICAL);
    Gtk::Box*      editorBox     = new Gtk::Box(Gtk::Orientation::VERTICAL);
    Gtk::Box*      printBox      = new Gtk::Box(Gtk::Orientation::VERTICAL);
    rhea::Grid*    listingGrid   = new rhea::Grid();
    Gtk::Box*      listingTop    = new Gtk::Box(Gtk::Orientation::HORIZONTAL);
    Gtk::Fixed*    editorVersion = new Gtk::Fixed();
    Gtk::Fixed*    editorData    = new Gtk::Fixed();
    Gtk::Fixed*    editorGrids   = new Gtk::Fixed();
    Gtk::Fixed*    editorBtns    = new Gtk::Fixed();
    Gtk::Fixed*    printPane     = new Gtk::Fixed();
    Gtk::Box*      comparePane   = new Gtk::Box(Gtk::Orientation::VERTICAL);
    Gtk::Label*    lblVersion    = new Gtk::Label();
    Gtk::Label*    lblValid      = new Gtk::Label();
    Gtk::Button*   btnPrev       = new Gtk::Button();
    Gtk::Button*   btnNext       = new Gtk::Button();
    Gtk::TextView* tvVersion     = new Gtk::TextView();
    Gtk::Button*   btnNew        = new Gtk::Button();
    Gtk::Button*   btnSave       = new Gtk::Button();
    Gtk::Button*   btnDelete     = new Gtk::Button();
    Gtk::Button*   btnImport	 = new Gtk::Button();
    Gtk::Button*   btnExport	 = new Gtk::Button();
    Gtk::Button*   btnOneDoc	 = new Gtk::Button();
    Gtk::Box*      listingBottom = new Gtk::Box(Gtk::Orientation::HORIZONTAL);

    FrameBase();
    ~FrameBase();
    virtual void onListingGridClick(rhea::GridCell* cell) = 0;
    void getVersions(T* po);
    void setVersionGui();
    void onPrevVersion();
    void onNextVersion();
    void newPo();
    void deletePo();
    void onImport();
    void onExport();
    void onOneDoc();
    virtual void getListing() = 0;
    virtual void populateGUI() = 0;
    virtual T* populatePojo(rhea::DbResultSet* res,long row = -1 ) = 0;
    virtual void setDataOnOldPojo(T po) = 0;
    virtual void onNew() = 0;
    virtual void onDelete() = 0;
    virtual long setId() = 0;
    virtual void onSave() = 0;
    virtual bool onClose() = 0;
    virtual void onActivate()= 0;
    virtual void populateVersions(T* obj, std::vector<T*> &list) = 0; //dodaj resultset in poljubni list za parameter funkcije
    virtual void populateTemplateFiles(T* myPo) = 0;
    virtual void fillImport() = 0;
    virtual void fillExport() = 0;
    void addCSS(std::string css);
};

#include "FrameBase.cpp"

#endif //GUI_FRAMEBASE_H
