/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#include "FrmSettings.h"
#include "main/Global.h"

FrmSettings::FrmSettings() : Gtk::Frame() {
	auto display = Gdk::Display::get_default();
	std::string cssData = ".labelLang {margin-bottom:20px;}";

	cssProvider->load_from_data(cssData);

	labelLang->set_valign(Gtk::Align::CENTER);
	labelLang->set_halign(Gtk::Align::CENTER);
	labelLang->add_css_class("labelLang");
	labelLang->get_style_context()->add_provider_for_display(display,cssProvider,GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
	mainBox->append(*labelLang);

	switch (lang->getLanguage()) {
		case Languages::si: {
			chkSi->set_active(true);
			break;
		} case Languages::hr: {
			chkHr->set_active(true);
			break;

		} case Languages::bih: {
			chkBih->set_active(true);
			break;

		} case Languages::en: {
			chkEn->set_active(true);
			break;

		} case Languages::de: {
			chkDe->set_active(true);
			break;

		} case Languages::fr: {
			chkFr->set_active(true);
			break;

		} case Languages::it: {
			chkIt->set_active(true);
			break;

		} default: {
			chkEn->set_active(true);
			break;
		}
	}

	chkSi->signal_toggled().connect(sigc::mem_fun(*this, &FrmSettings::onChkPressed));
	chkHr->signal_toggled().connect(sigc::mem_fun(*this, &FrmSettings::onChkPressed));
	chkBih->signal_toggled().connect(sigc::mem_fun(*this, &FrmSettings::onChkPressed));
	chkEn->signal_toggled().connect(sigc::mem_fun(*this, &FrmSettings::onChkPressed));
	chkDe->signal_toggled().connect(sigc::mem_fun(*this, &FrmSettings::onChkPressed));
	chkFr->signal_toggled().connect(sigc::mem_fun(*this, &FrmSettings::onChkPressed));
	chkIt->signal_toggled().connect(sigc::mem_fun(*this, &FrmSettings::onChkPressed));


	chkSi->set_label("Slovenščina");
	chkHr->set_label("Hrvatski");
	chkBih->set_label("Bosanski");
	chkEn->set_label("English");
	chkDe->set_label("Deutche");
	chkFr->set_label("French");
	chkIt->set_label("Italian");

	chkHr->set_group(*chkSi);
	chkBih->set_group(*chkSi);
	chkEn->set_group(*chkSi);
	chkDe->set_group(*chkSi);
	chkFr->set_group(*chkSi);
	chkIt->set_group(*chkSi);

	mainBox->append(*chkSi);
	mainBox->append(*chkHr);
	mainBox->append(*chkBih);
	mainBox->append(*chkEn);
	mainBox->append(*chkDe);
	mainBox->append(*chkFr);
	mainBox->append(*chkIt);


	mainBox->set_vexpand(true);
	mainBox->set_hexpand(true);
	mainBox->set_halign(Gtk::Align::CENTER);
	mainBox->set_valign(Gtk::Align::CENTER);

	set_child(*mainBox);

}

FrmSettings::~FrmSettings() {
	// TODO Auto-generated destructor stub
}

void FrmSettings::onChkPressed() {
	std::string langStr = "";
	if (chkSi->get_active()) {
		lang->setLanguage(Languages::si);
		langStr = "si";
	} else if (chkHr->get_active()) {
		lang->setLanguage(Languages::hr);
		langStr = "hr";
	} else if (chkBih->get_active()) {
		lang->setLanguage(Languages::bih);
		langStr = "bih";
	} else if (chkEn->get_active()) {
		lang->setLanguage(Languages::en);
		langStr = "en";
	} else if (chkDe->get_active()) {
		lang->setLanguage(Languages::de);
		langStr = "de";
	} else if (chkFr->get_active()) {
		lang->setLanguage(Languages::fr);
		langStr = "fr";
	} else if (chkIt->get_active()) {
		lang->setLanguage(Languages::it);
		langStr = "it";
	}

    std::string sql = "UPDATE Settings SET lang = '" + langStr + "';";
    dbConn->execQuery(sql);

	labelLang->set_label(lang->getText("txtSettingsLabelLang"));

	Global* global = Global::getInstance();
	global->appMenu->setLabel("file",lang->getText("txtMainMenu_FileMenu"));
	global->appMenu->setLabel("templates",lang->getText("txtMainMenu_FileStructures"));
	global->appMenu->setLabel("setings",lang->getText("txtMainMenu_FileSettings"));
	global->appMenu->setLabel("quit",lang->getText("txtMainMenu_FileQuit"));
	global->appMenu->setLabel("work",lang->getText("txtMainMenu_WorkMenu"));
	global->appMenu->setLabel("help",lang->getText("txtMainMenu_HelpMenu"));
	global->appMenu->setLabel("content",lang->getText("txtMainMenu_HelpInstr"));
	global->appMenu->setLabel("about",lang->getText("txtMainMenu_HelpAbout"));

}

void FrmSettings::onActivate() {
	labelLang->set_label(lang->getText("txtSettingsLabelLang"));
}
bool FrmSettings::onClose() {
	return true;
}
