/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#include "FrmTemplData.h"
#include "WndImportTD.h"
#include "main/Global.h"
#include "GridDataSection.h"

FrmTemplData::FrmTemplData(std::string templateId) : FrameBase() {

	rhea::DbParameters params = rhea::DbParameters();
	params.add(templateId);

	rhea::DbResultSet* res = dbConn->execQuery("SELECT * FROM Templates WHERE valid = 1 and templId=?;", params.get());
	if (res->rowCount()>0) {
		tmpl->setRowId(res->getField("rowId")->getAsLong());
//		tmpl->setMenuVisibility(res->getField("showInMenu")->getAsInteger()==1);
//		tmpl->menuLabel = res->getField("menuLabel")->getAsString();
		tmpl->setName(res->getField("name")->getAsString());
		tmpl->setPojoId(res->getField("templId")->getAsLong());
		tmpl->setPojoName(res->getField("rowId")->getAsString());


	    rhea::DbResultSet* resCol = dbConn->execQuery("SELECT * FROM TemplateColumns WHERE valid = 1 and "
	    											  "templRowId in (select rowId from Templates where templId = "
	    		    			  	  	  	  	  	  + res->getField("templId")->getAsString()
													  + ") ORDER BY orderWeight;");
	    if (resCol->rowCount()>0){
	    	while(!resCol->isEof()){
	    		ColumnW* col = new ColumnW();
		        col->setRowId(resCol->getField("rowId")->getAsLong());
		        col->setPojoId(resCol->getField("columnId")->getAsLong());
		        col->setUUID(resCol->getField("uuid")->getAsString());
		        col->order = resCol->getField("orderWeight")->getAsInteger();
		        col->label = resCol->getField("label")->getAsString();
		        col->libreCode = resCol->getField("libreCode")->getAsString();
		        std::string strObject = resCol->getField("object")->getAsString();
		        col->objectType = strObject == "TextView" ? ColumnObject::TextView :
		        		      strObject == "Combo" ? ColumnObject::Combo :
		        		      strObject == "ChkBox" ? ColumnObject::ChkBox :
		        		      strObject == "Options" ? ColumnObject::Options :
		        		      strObject == "PickupDt" ? ColumnObject::PckpDT :
		        		      strObject == "Grid" ? ColumnObject::Grid : ColumnObject::TextView;
		        std::string strType = resCol->getField("type")->getAsString();
		        col->type = strType == "String" ? ColumnType::String :
		        		    strType == "Int" ? ColumnType::Int :
		        			strType == "Double" ? ColumnType::Double :
		        			strType == "Date" ? ColumnType::Date :
		        			strType == "Index" ? ColumnType::Index : ColumnType::String;
		        global->parseOptions(resCol->getField("options")->getAsString(),global->optionsDelimiter ,col->options);
		        tmpl->columns.push_back(col);
		        resCol->moveNext();
		    }
		}

		rhea::DbResultSet* resFiles = dbConn->execQuery("SELECT * FROM TemplateFiles WHERE valid = 1 and "
					    											  "templRowId in (select rowId from Templates where templId = "
					    		    			  	  	  	  	  	  + res->getField("templId")->getAsString()
																	  + ") ORDER BY rowId;");
		if (resFiles->rowCount()>0){
			while(!resFiles->isEof()){
				TemplateFile* tmpFile = new TemplateFile();
				tmpFile->setRowId(resFiles->getField("rowId")->getAsLong());
				tmpFile->setPojoId(resFiles->getField("fileId")->getAsLong());
				tmpFile->setUUID(resFiles->getField("uuid")->getAsString());
				tmpFile->setPojoVersion(resFiles->getField("version")->getAsInteger());
				tmpFile->setFullName(resFiles->getField("path")->getAsString()+resFiles->getField("name")->getAsString());
				tmpl->templateFiles.push_back(tmpFile);
				resFiles->moveNext();
			}
		}
	}

	lblName->set_justify(Gtk::Justification::RIGHT);
	lblName->set_size_request(lblSize->width,lblSize->height);

	addCSS(".rhea_formElementLbl { color: #06528e;}");
	addCSS(".rhea_formElementText { color:black; border-color: black; border-style: solid; border-width: 0.7px;} .rhea_formElementText .view, .rhea_formElementText text { background-color: #FFFFFF;}");
	int myHeight = 10;
	for (Column* cl : tmpl->columns){
		ColumnW* col = dynamic_cast<ColumnW*>(cl);
		Gtk::Label* lbl = new Gtk::Label();
		lbl->set_justify(Gtk::Justification::RIGHT);
		lbl->set_size_request(lblSize->width,lblSize->height);
		lbl->set_text(col->label);
		lbl->add_css_class("rhea_formElement");
		lbl->add_css_class("rhea_formElementLbl");
		lbl->get_style_context()->add_provider_for_display(display,cssProvider,GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
		editorData->put(*lbl,lblSize->X,myHeight);
		switch (col->objectType) {
			case ColumnObject::TextView: {
				Gtk::TextView* txt = new Gtk::TextView();
				txt->add_css_class("rhea_formElement");
				txt->add_css_class("rhea_formElementText");
				txt->set_size_request(txtSize->width,txtSize->height);
				txt->get_style_context()->add_provider_for_display(display,cssProvider,GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
				if(col->type == ColumnType::Index){
					Gtk::Box* indexBox = new Gtk::Box(Gtk::Orientation::HORIZONTAL);
					Gtk::Label* lblPrefix = new Gtk::Label();
					lblPrefix->add_css_class("rhea_formElement");
					Gtk::Label* lblPostfix = new Gtk::Label();
					lblPostfix->add_css_class("rhea_formElement");
					lblPrefix->set_size_request(30,txtSize->height);
					lblPostfix->set_size_request(30,txtSize->height);
					txt->set_size_request(30,txtSize->height);
					indexBox->append(*lblPrefix);
					indexBox->append(*txt);
					indexBox->append(*lblPostfix);
					editorData->put(*indexBox,txtSize->X,myHeight);
					col->object = indexBox;
				}else {
					txt->set_size_request(txtSize->width,txtSize->height);
					editorData->put(*txt,txtSize->X,myHeight);
					col->object = txt;
				}
				myHeight += 50;
				break;

			}case ColumnObject::Combo: {
				int index = 0;
				Glib::RefPtr<Gtk::ListStore> listStore = Gtk::ListStore::create(stringColumns);
				listStore->clear();
				auto rowObject = *listStore->append();
				rowObject[stringColumns.col_id] = index;
				rowObject[stringColumns.col_name] = "";
				std::vector<std::string> valOpt = {};
				getDataAsOptions(col->options,valOpt);
				for(std::string str : valOpt){
					index++;
					rowObject = *listStore->append();
			    	rowObject[stringColumns.col_id] = index;
			    	rowObject[stringColumns.col_name] = str;
				}
			    rhea::ComboBox<StringColumns>* cmb = new rhea::ComboBox<StringColumns>();
				cmb->addList(&listStore);
				cmb->set_size_request(txtSize->width,txtSize->height);
				cmb->add_css_class("rhea_formElement");
				cmb->add_css_class("rhea_formElementCmb");
				editorData->put(*cmb,txtSize->X,myHeight);
				Gtk::Button* btn = new Gtk::Button();
			    std::string iconPath =getenv("GRUNF");
			    Gtk::Image* icon = new Gtk::Image();
			    iconPath += "/res/osvezi.png";
			    icon->set(iconPath);
			    icon->set_size_request(2,25);
			    btn->set_child(*icon);
			    btn->set_size_request(3,3);
			    btn->set_tooltip_text("Osveži");
			    btn->signal_clicked().connect(sigc::bind(sigc::mem_fun(*this, &FrmTemplData::btnRefreshOptionCmb),cmb,cl));
			    editorData->put(*btn,txtSize->X + txtSize->width +5 ,myHeight);
				myHeight += 50;
				col->object = cmb;
				break;

			}case ColumnObject::ChkBox : {
				Gtk::Box* myBox = new Gtk::Box(Gtk::Orientation::HORIZONTAL);
				Gtk::Box* chkBox = new Gtk::Box(Gtk::Orientation::VERTICAL);
				int y = 5;
				std::vector<std::string> valOpt = {};
				getDataAsOptions(col->options,valOpt);
				for(std::string str : valOpt){
					Gtk::CheckButton* chk = new Gtk::CheckButton();
					chk->set_label(str);
					chk->add_css_class("rhea_formElement");
					chk->add_css_class("rhea_formElementChk");
					chkBox->append(*chk);
					y+=25;
				}

				Gtk::Button* btn = new Gtk::Button();
			    std::string iconPath =getenv("GRUNF");
			    Gtk::Image* icon = new Gtk::Image();
			    iconPath += "/res/osvezi.png";
			    icon->set(iconPath);
			    icon->set_size_request(2,25);
			    btn->set_child(*icon);
			    btn->set_size_request(3,3);
			    btn->set_tooltip_text("Osveži");
			    btn->signal_clicked().connect(sigc::bind(sigc::mem_fun(*this, &FrmTemplData::btnRefreshOptionChk),chkBox, valOpt, false, true));

			    myBox->append(*chkBox);
			    myBox->append(*btn);
			    btn->set_vexpand(false);
			    btn->set_valign(Gtk::Align::START);

			    editorData->put(*myBox, txtSize->X, myHeight);

				myHeight += y+20;
				col->object = chkBox;

				break;

			}case ColumnObject::Options: {
				Gtk::Box* myBox = new Gtk::Box(Gtk::Orientation::HORIZONTAL);
				Gtk::Box* chkBtnBox = new Gtk::Box(Gtk::Orientation::VERTICAL);
				Gtk::CheckButton* group = {nullptr};
				int y = 5;
				std::vector<std::string> valOpt = {};
				getDataAsOptions(col->options,valOpt);
				for(std::string str : valOpt){
					Gtk::CheckButton* chk = new Gtk::CheckButton();
					chk->add_css_class("rhea_formElement");
					chk->add_css_class("rhea_formElementOpt");
					chk->set_label(str);
					if (!group){
						group = chk;
					}else{
						chk->set_group(*group);
					}
					chkBtnBox->append(*chk);
					y+=25;
				}

				Gtk::Button* btn = new Gtk::Button();
			    std::string iconPath =getenv("GRUNF");
			    Gtk::Image* icon = new Gtk::Image();
			    iconPath += "/res/osvezi.png";
			    icon->set(iconPath);
			    icon->set_size_request(2,25);
			    btn->set_child(*icon);
			    btn->set_size_request(3,3);
			    btn->set_tooltip_text("Osveži");
			    btn->signal_clicked().connect(sigc::bind(sigc::mem_fun(*this, &FrmTemplData::btnRefreshOptionChk),chkBtnBox, valOpt, true, true));

			    myBox->append(*chkBtnBox);
			    myBox->append(*btn);
			    btn->set_vexpand(false);
			    btn->set_valign(Gtk::Align::START);
				editorData->put(*myBox,txtSize->X,myHeight);
				myHeight += y+20;
				col->object = chkBtnBox;
				break;

			}case ColumnObject::PckpDT : {
				rhea::PickupDT* pdt = new rhea::PickupDT();
				pdt->setSize(txtSize->width,txtSize->height);
				editorData->put(*pdt,txtSize->X,myHeight);
				myHeight += 50;
				col->object = pdt;
				break;

			}case ColumnObject::Grid: {
				GridDataSection* grid = new GridDataSection();
				//grid->get_style_context()->add_provider_for_display(display,cssProvider,GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
				std::vector<std::string> valOpt = {};
				getDataAsOptions(col->options,valOpt);
				grid->reset(valOpt);
				editorData->put(*grid,10,myHeight+25);
				grid->setSize(350,grid->getRows()*35);
				myHeight += grid->getRows()*35+60;
				col->object = grid;
				break;

			}default: {
				Gtk::TextView* def = new Gtk::TextView();
				def->set_size_request(txtSize->width,txtSize->height);
				def->add_css_class("rhea_formElement");
				def->add_css_class("rhea_formElementDef");
				editorData->put(*def,txtSize->X,myHeight);
				myHeight += 50;
				col->object = def;
			}
		}
		col->object->get_style_context()->add_provider_for_display(display,cssProvider,GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
	}
	cssProvider->load_from_data(cssData);
	filesBox->set_size_request(450,300);
    Gtk::Label* myLbl = dynamic_cast<Gtk::Label*>(filesBox->getHeader(0)->getWidget());
    myLbl->set_text(langPack->getText("txtfileBox_Files_RHEA"));
    Gtk::Button* myHdrBtn = dynamic_cast<Gtk::Button*>(filesBox->getHeader(1)->getWidget());
    myHdrBtn->set_label(langPack->getText("txtfileBox_Add_RHEA"));

	editorGrids->put(*filesBox,0,0);
	this->getListing();

	newPo();
}

FrmTemplData::~FrmTemplData() {

}


void FrmTemplData::btnRefreshOptionCmb(rhea::ComboBox<StringColumns>* cmb, Column* cl) {
	int index = 0;
	std::vector<std::string> valOpt = {};
	Glib::RefPtr<Gtk::ListStore> listStore = cmb->refTreeModel;
	listStore->clear();
	auto rowObject = *listStore->append();
	rowObject[stringColumns.col_id] = index;
	rowObject[stringColumns.col_name] = "";
	getDataAsOptions(cl->options,valOpt);
	for(std::string str : valOpt){
		index++;
		rowObject = *listStore->append();
    	rowObject[stringColumns.col_id] = index;
    	rowObject[stringColumns.col_name] = str;
	}
	cmb->set_active(0);
}

void FrmTemplData::btnRefreshOptionChk(Gtk::Widget* chkBox, std::vector<std::string> options, bool group, bool refresh ) {
	Gtk::Box* myBox = dynamic_cast<Gtk::Box*>(chkBox);

	Gtk::Widget* prvi = myBox->get_first_child();
	if (prvi){
		while (true){
			Gtk::Widget* element = prvi;

			if(prvi == myBox->get_last_child()){
				break;
			}
			prvi = prvi->get_next_sibling();
			element->unparent();
			delete element;
		}
		prvi->unparent();
		delete prvi;
	}

	int y = 5;
	Gtk::CheckButton* groupChk = {nullptr};
	for(std::string str : options){
		Gtk::CheckButton* chk = new Gtk::CheckButton();
		chk->set_label(str);
		chk->add_css_class("rhea_formElement");
		chk->add_css_class("rhea_formElementChk");
		if (group){
			if (!groupChk){
				groupChk = chk;
			}else{
				chk->set_group(*groupChk);
			}
		}
		myBox->append(*chk);
		y+=25;
	}

	refreshOptions = refresh;

}

void FrmTemplData::getDataAsOptions(std::vector<std::string> &options, std::vector<std::string> &valOptions) {
	std::string templ ="";
	std::string text = "";
	valOptions.clear();
	std::vector<std::string> staticOptions = {};
	std::vector<std::string> cols = {};
	for (std::string str : options) {
		if (str.starts_with(global->gridTemplateDelimiter)) {
			templ = str.substr(global->gridTemplateDelimiter.size());
			size_t pos = templ.find(global->gridTemplateDelimiter);
			if (pos > templ.size()){
				continue;
			}
			text = templ;
			templ = templ.substr(0,pos);
			text = text.substr(pos+global->gridTemplateDelimiter.size());
		    global->parseOptions(text, global->gridTemplateColDelimiter, cols);
		} else {
			staticOptions.push_back(str);
		}
	}

	if (!templ.empty() && cols.size()>0){

		std::string columns = "";
		for (std::string & str: cols) {
			columns += ", " + str;
		}

		columns = columns.substr(2);
		rhea::ListedMap<std::string, Column*> myCols = {};
	    rhea::DbResultSet* resCol = dbConn->execQuery("SELECT * FROM TemplateColumns WHERE valid = 1 and "
	    											  "columnId in (" + columns + ") and"
	    											  "templRowId in (select rowId from Templates where templId = "
	    		    			  	  	  	  	  	  + templ
													  + ");");
	    if (resCol->rowCount()>0) {
	    	while(!resCol->isEof()) {
	    		Column* col = new Column();
		        col->setRowId(resCol->getField("rowId")->getAsLong());
		        col->setPojoId(resCol->getField("columnId")->getAsLong());
		        col->setUUID(resCol->getField("uuid")->getAsString());
		        col->order = resCol->getField("orderWeight")->getAsInteger();
		        col->label = resCol->getField("label")->getAsString();
		        col->libreCode = resCol->getField("libreCode")->getAsString();
		        std::string strObject = resCol->getField("object")->getAsString();
		        col->objectType = strObject == "TextView" ? ColumnObject::TextView :
		        		      strObject == "Combo" ? ColumnObject::Combo :
		        		      strObject == "ChkBox" ? ColumnObject::ChkBox :
		        		      strObject == "Options" ? ColumnObject::Options :
		        		      strObject == "PickupDt" ? ColumnObject::PckpDT :
		        		      strObject == "Grid" ? ColumnObject::Grid : ColumnObject::TextView;
		        std::string strType = resCol->getField("type")->getAsString();
		        col->type = strType == "String" ? ColumnType::String :
		        		    strType == "Int" ? ColumnType::Int :
		        			strType == "Double" ? ColumnType::Double :
		        			strType == "Date" ? ColumnType::Date :
		        			strType == "Index" ? ColumnType::Index : ColumnType::String;
		        global->parseOptions(resCol->getField("options")->getAsString(),global->optionsDelimiter ,col->options);
		        myCols.add(std::to_string(col->getPojoId()), col);
		        resCol->moveNext();
	    	}
	    }

		rhea::DbParameters paramTd = rhea::DbParameters();
		paramTd.add(templ);

		rhea::DbResultSet* res = dbConn->execQuery("SELECT DISTINCT td.* FROM Templates t left join TemplateData td on td.valid = 1 and td.templRowId in (select rowId from Templates t1 where t1.templId = t.templId) "
												   "  WHERE t.templId = ? and (td.dataId is not null or td.dataId > 0)  ORDER BY td.dataId;",paramTd.get());
		rhea::ListedMap<long,TemplateData*> myData = {};
		if (res->rowCount()>0){
	        while(!res->isEof()){
	        	TemplateData* td = populatePojo(res);
	        	myData.add(td->getPojoId(),td);
	            res->moveNext();
	        }
	    }

		for (auto data : myData.getMap()) {
			std::string optData = "";
			for (std::string & tmpCol: cols) {
				for (TemplateValues* val : data.second->values){
					if (std::to_string(val->getColumnId()) ==tmpCol){
						optData += val->getValue() + " ";
					}
				}
			}
			if (!optData.empty()){
				valOptions.push_back (optData);
			}
		}
	}
	for (std::string str : staticOptions){
		valOptions.push_back(str);
	}
}

void FrmTemplData::onListingGridClick(rhea::GridCell* cell) {
	btnSave->set_sensitive(true);
	po = mapOfPo.getValueAt(cell->getRow());
	poFromListing = po;
	getVersions(po);

}

std::string FrmTemplData::dataListing (int col,int row) {

	int colToUse = 0;
	bool found = false;
	if (row < mapOfPo.size() && col < mapOfPo.getValueAt(row)->values.size() ){
		for (TemplateValues* tv : mapOfPo.getValueAt(row)->values){
			if (tmpl->columns.at(col)->getPojoId() == tv->getColumnId()){
				found = true;
				break;
			}
			colToUse++;
		}

		if (found && tmpl->columns.at(col)->objectType == ColumnObject::Combo){
			if (found && global->isNumber(mapOfPo.getValueAt(row)->values.at(colToUse)->getValue())){
				int index =std::stoi(mapOfPo.getValueAt(row)->values.at(colToUse)->getValue());
				if (index>0){
//					return tmpl->columns.at(col)->options.at(index-1);
					return mapOfPo.getValueAt(row)->values.at(colToUse)->options.at(index-1);
				}
					return "";
			}

		} else if (found && (tmpl->columns.at(col)->objectType == ColumnObject::ChkBox || tmpl->columns.at(col)->objectType == ColumnObject::Options) ){
			std::vector<std::string> tmpVect = {};
			global->parseOptions(mapOfPo.getValueAt(row)->values.at(colToUse)->getValue(),",",tmpVect);
			std::string tmp ="";
			int i = 0;
//			for (std::string str : tmpl->columns.at(col)->options) {
			for (std::string str : mapOfPo.getValueAt(row)->values.at(colToUse)->options) {
				if (tmpVect.at(i)=="1"){
					tmp += str + " | ";
				}
				i++;
			}
			if (!tmp.empty()){
				tmp = tmp.substr(0,tmp.size()-3);
			}
			tmpVect.clear();
			return tmp;

		} else if (found && (tmpl->columns.at(col)->objectType == ColumnObject::PckpDT)){
			std::string day = mapOfPo.getValueAt(row)->values.at(colToUse)->getValue().substr(8,2);
			std::string mon = mapOfPo.getValueAt(row)->values.at(colToUse)->getValue().substr(5,2);
			day = day.starts_with('0')? day.substr(1,1): day;
			mon = mon.starts_with('0')? mon.substr(1,1): mon;
			std::string tmpStr = day + ". "
							   + mon + ". "
							   + mapOfPo.getValueAt(row)->values.at(colToUse)->getValue().substr(0,4);
			return tmpStr;
		}

		return found && (tmpl->columns.at(col)->objectType == ColumnObject::TextView) ? mapOfPo.getValueAt(row)->values.at(colToUse)->getValue() : "...";
	}
	return "...";

}

void FrmTemplData::getListing() {

	rhea::DbParameters params = rhea::DbParameters();
	params.add(tmpl->getPojoId());

	rhea::DbResultSet* res = dbConn->execQuery("SELECT DISTINCT td.* FROM Templates t left join TemplateData td on td.valid = 1 and td.templRowId in (select rowId from Templates t1 where t1.templId = t.templId) "
											   "  WHERE t.templId = ? and (td.dataId is not null or td.dataId > 0)  ORDER BY td.dataId;",params.get());
    if (res->rowCount()>0){
        while(!res->isEof()){
        	TemplateData* td = populatePojo(res);
            mapOfPo.add({std::to_string(td->getPojoId()),td});
            res->moveNext();
        }
    }

    std::vector<std::string> hdrs = {};
    int i = 0;
    for (Column* col : tmpl->columns){
    	i++;
    	if (i < 5) {
    		hdrs.push_back(col->label);
    	}
    }

    listingGrid->setHeaders(&hdrs, 0);
    listingGrid->setRows(mapOfPo.size());
    listingGrid->connectString(&FrmTemplData::dataListing, this);
    listingGrid->refresh();
    listingGrid->connectOnClick(&FrmTemplData::onListingGridClick, this);

    listingGrid->show();
}

void FrmTemplData::populateGUI() {

	for (Column* cl : tmpl->columns){
		ColumnW* col = dynamic_cast<ColumnW*>(cl);
		for (TemplateValues* val : po->values){
			if (col->getPojoId() == val->getColumnId()){
				switch (col->objectType){
					case ColumnObject::TextView: {
						if (col->type == ColumnType::Index){
							Gtk::Box* box = dynamic_cast<Gtk::Box*>(col->object);
							if (box){
								Gtk::Label* lblprefix =dynamic_cast<Gtk::Label*>(box->get_first_child());
								if (lblprefix){
									lblprefix->set_text(col->options.at(0));
								}
								Gtk::TextView* txt= (dynamic_cast<Gtk::TextView*>(box->get_first_child()->get_next_sibling()));
								if (txt){
									txt->get_buffer()->set_text(val->getValue());
								}
								Gtk::Label* lblpostfix =dynamic_cast<Gtk::Label*>(box->get_last_child());
								if (lblpostfix){
									lblpostfix->set_text(col->options.at(2));
								}
							}
						} else {
							(dynamic_cast<Gtk::TextView*>(col->object))->get_buffer()->set_text(val->getValue());
						}
						break;
					}case ColumnObject::Combo: {

						rhea::ComboBox<StringColumns>* cmb =(dynamic_cast<rhea::ComboBox<StringColumns>*>(col->object));
						int index = 0;
						Glib::RefPtr<Gtk::ListStore> listStore = cmb->refTreeModel;
						listStore->clear();
						auto rowObject = *listStore->append();
						rowObject[stringColumns.col_id] = index;
						rowObject[stringColumns.col_name] = "";
						if (val->options.empty()) {
							getDataAsOptions(col->options,val->options);
						}
						for(std::string str : val->options){
							index++;
							rowObject = *listStore->append();
					    	rowObject[stringColumns.col_id] = index;
					    	rowObject[stringColumns.col_name] = str;
						}
						std::string tmp = val->getValue();
						int number = Global::isNumber(tmp) ? std::stoi(tmp) : 0;
						cmb->set_active(number);
						break;

					}case ColumnObject::ChkBox : {
						if (val->options.empty()) {
							getDataAsOptions(col->options,val->options);
						}
						btnRefreshOptionChk(col->object, val->options, false);

						std::vector<std::string> tmp= {};
						global->parseOptions(val->getValue(),",",tmp);
						Gtk::Widget* prvi = col->object->get_first_child();
						if (prvi){
							int i = 0;
							while (true){
								if (global->isNumber(tmp.at(i))){
									(dynamic_cast<Gtk::CheckButton*>(prvi))->set_active(tmp.at(i)=="1");
								}
								if(prvi == col->object->get_last_child()){
									break;
								}
								prvi = prvi->get_next_sibling();
								i++;
							}
						}

						break;
					}case ColumnObject::Options: {
						if (val->options.empty()) {
							getDataAsOptions(col->options,val->options);
						}
						btnRefreshOptionChk(col->object, val->options, true);

						std::vector<std::string> tmp= {};
						global->parseOptions(val->getValue(),",",tmp);
						Gtk::Widget* prvi = col->object->get_first_child();
						if (prvi){
							int i = 0;
							while (true){
								if (global->isNumber(tmp.at(i))){
									(dynamic_cast<Gtk::CheckButton*>(prvi))->set_active(tmp.at(i)=="1");
								}
								if(prvi == col->object->get_last_child()){
									break;
								}
								prvi = prvi->get_next_sibling();
								i++;
							}
						}
						break;
					}case ColumnObject::PckpDT : {
						(dynamic_cast<rhea::PickupDT*>(col->object))->setValue(val->getValue());
						break;
					}case ColumnObject::Grid: {
						if (val->options.empty()) {
							getDataAsOptions(col->options,val->options);
						}
						(dynamic_cast<GridDataSection*>(col->object))->reset(val->options);
						(dynamic_cast<GridDataSection*>(col->object))->setData(val->getValue());
						break;
					}default: {

					}
				}
			}
		}
	}

	filesBox->removeAll();
	for(TemplateDataFile* file : po->files){
		filesBox->addFile(file->getFullName());
	}
}

TemplateData* FrmTemplData::populatePojo(rhea::DbResultSet* res,long row) {
	TemplateData* td = new TemplateData();
	td->myTemplate = tmpl;
	td->setRowId(res->getField("rowId")->getAsLong());
	td->setPojoId(res->getField("dataId")->getAsLong());
	td->setPojoVersion(res->getField("version")->getAsInteger());

	rhea::DbParameters params = rhea::DbParameters();
	params.add(td->getRowId());

	rhea::DbResultSet* resVal = dbConn->execQuery("SELECT * FROM TemplateValues WHERE valid = 1 and "
											   "  dataRowId = ? order by rowId;",params.get());
    if (resVal->rowCount()>0){
    	while(!resVal->isEof()){
			TemplateValues* tv = new TemplateValues();
			tv->setValue(resVal->getField("value")->getAsString());
			global->parseOptions(resVal->getField("options")->getAsString(),global->optionsDelimiter ,tv->options);
			tv->setDataRowId(resVal->getField("dataRowId")->getAsLong());
			tv->setColumnRowId(resVal->getField("columnRowId")->getAsLong());
			tv->setPojoId(resVal->getField("valueId")->getAsLong());

			rhea::DbParameters paramCol = rhea::DbParameters();
			paramCol.add((int)tv->getColumnRowId());
			rhea::DbResultSet* resCol = dbConn->execQuery("SELECT columnId FROM TemplateColumns WHERE rowId = ?;",paramCol.get());
			if (resCol->rowCount()>0){
				tv->setColumnId(resCol->getField("columnId")->getAsLong());
			}

			td->values.push_back(tv);
            resVal->moveNext();
        }
    }

	rhea::DbResultSet* resFiles = dbConn->execQuery("SELECT * FROM TemplateDataFiles WHERE valid = 1 and "
											   "  dataRowId = ? order by rowId;",params.get());
    if (resFiles->rowCount()>0){
    	while(!resFiles->isEof()){
			TemplateDataFile* tdf = new TemplateDataFile();
			tdf->setPojoId(resFiles->getField("fileId")->getAsLong());
			tdf->setPojoVersion(resFiles->getField("version")->getAsLong());
			tdf->setDataRowId(resFiles->getField("dataRowId")->getAsLong());
			tdf->setTemplateRowId(resFiles->getField("templRowId")->getAsLong());
			tdf->setTemplateFileRowId(resFiles->getField("templFileRowId")->getAsLong());
			tdf->setUUID(resFiles->getField("uuid")->getAsString());
			tdf->setName(resFiles->getField("name")->getAsString());
			tdf->setPath(resFiles->getField("path")->getAsString(),true);

			td->files.push_back(tdf);
			resFiles->moveNext();
        }
    }


	return td;
}

void FrmTemplData::setDataOnOldPojo(TemplateData po) {

}

long FrmTemplData::setId(){
    rhea::DbResultSet* res = dbConn->execQuery("SELECT COALESCE(MAX(dataId)+1,1) from TemplateData;");
    return res->getField(0)->getAsLong();
}

void FrmTemplData::onNew() {
	po->myTemplate = tmpl;
	for (Column* cl : tmpl->columns){
		ColumnW* col = dynamic_cast<ColumnW*>(cl);

		switch (col->objectType){
			case ColumnObject::TextView: {
				if( col->type == ColumnType::Index){
					long index = -1;
					if (mapOfPo.size()>0){
						TemplateData* previous = mapOfPo.getValueAt(mapOfPo.size()-1);
						if (previous->getPojoId()>0){
							for (TemplateValues* val :previous->values){
								if (val->getColumnId() == col->getPojoId()){
									if (Global::isNumber(val->getValue())){
										index = std::stoi(val->getValue());
									}
								}
							}
							if (Global::isNumber(col->options.at(1))){
								index = index > 0 ? index + std::stoi(col->options.at(1)) : mapOfPo.size()+1;
							}else {
								index = index > 0 ? index+1 : mapOfPo.size()+1;
							}
						}

					}
					index = index < 1 ? 1 : index;
					Gtk::Box* box = dynamic_cast<Gtk::Box*>(col->object);
					if (box){
						Gtk::Label* lblprefix =dynamic_cast<Gtk::Label*>(box->get_first_child());
						if (lblprefix){
							lblprefix->set_text(col->options.at(0));
						}
						Gtk::TextView* txt= (dynamic_cast<Gtk::TextView*>(box->get_first_child()->get_next_sibling()));
						if (txt){
							txt->get_buffer()->set_text(std::to_string(index));
						}
						Gtk::Label* lblpostfix =dynamic_cast<Gtk::Label*>(box->get_last_child());
						if (lblpostfix){
							lblpostfix->set_text(col->options.at(2));
						}
					}

				}else{
					(dynamic_cast<Gtk::TextView*>(col->object))->get_buffer()->set_text("");
				}
				break;

			}case ColumnObject::Combo: {
				int index = 0;
				rhea::ComboBox<StringColumns>* cmb = (dynamic_cast<rhea::ComboBox<StringColumns>*>(col->object));
				Glib::RefPtr<Gtk::ListStore> listStore = cmb->refTreeModel;
				listStore->clear();
				auto rowObject = *listStore->append();
				rowObject[stringColumns.col_id] = index;
				rowObject[stringColumns.col_name] = "";
				std::vector<std::string> valOpt = {};
				getDataAsOptions(col->options,valOpt);
				for(std::string str : valOpt){
					index++;
					rowObject = *listStore->append();
			    	rowObject[stringColumns.col_id] = index;
			    	rowObject[stringColumns.col_name] = str;
				}

				cmb->set_active(0);
				break;

			}case ColumnObject::ChkBox : {
				std::vector<std::string> valOpt = {};
				getDataAsOptions(col->options,valOpt);
				btnRefreshOptionChk(col->object, valOpt, false);

				Gtk::Widget* prvi = col->object->get_first_child();
				if (prvi){
					while (true){
						(dynamic_cast<Gtk::CheckButton*>(prvi))->set_active(false);
						if(prvi == col->object->get_last_child()){
							break;
						}
						prvi = prvi->get_next_sibling();
					}
				}
				break;

			}case ColumnObject::Options: {
				std::vector<std::string> valOpt = {};
				getDataAsOptions(col->options,valOpt);
				btnRefreshOptionChk(col->object, valOpt, true);

				Gtk::Widget* prvi = col->object->get_first_child();
				if (prvi){
					while (true){
						(dynamic_cast<Gtk::CheckButton*>(prvi))->set_active(false);
						if(prvi == col->object->get_last_child()){
							break;
						}
						prvi = prvi->get_next_sibling();
					}
				}
				break;

			}case ColumnObject::PckpDT : {
				(dynamic_cast<rhea::PickupDT*>(col->object))->setValue("");
				break;

			}case ColumnObject::Grid: {
				std::vector<std::string> valOpt = {};
				getDataAsOptions(col->options,valOpt);
				(dynamic_cast<GridDataSection*>(col->object))->reset(valOpt);


				break;
			}default: {

			}
		}
	}

	filesBox->removeAll();
}

void FrmTemplData::onSave() {

	std::string sqlStr = "";
    if( po && po->getRowId() > 0 ){
    	po->setValid(false);
        time_t now = time(0);
        po->validTill->set(now);
        //TO-DO set validTill
        sqlStr = "UPDATE TemplateData SET valid = 0 , validTill= '" + po->validTill->getAsISO8601() + "' WHERE rowId=" + std::to_string(po->getRowId()) + ";";
        rhea::DbResultSet* dbOld = dbConn->execQuery(sqlStr);
        delete dbOld;
//        std::cout << "uspešno postaral item" << std::endl;
    }

    long id = 1;
    if(po && po->getPojoId() > 0){
    	id = po->getPojoId();
    } else {
    	id = setId();
    }

//    std::cout << "nastavil id" << std::endl;

    TemplateData* newPo = new TemplateData();
    newPo->setPojoId(id);
    newPo->setPojoVersion(poVersions.size()+1);
    newPo->setValid(true);
    newPo->myTemplate = tmpl;

	for (Column * cl : tmpl->columns){
		ColumnW* col = dynamic_cast<ColumnW*>(cl);
		TemplateValues* val = new TemplateValues();
	    val->setPojoId(col->getPojoId());
		val->setPojoVersion(newPo->getPojoVersion());
	    val->setColumnId(col->getPojoId());
	    val->setColumnRowId(col->getRowId());

//	    std::cout << "shranjujem element" << std::endl;

	    if(po && !po->values.empty()){
			for (TemplateValues* tv : po->values){
				if (col->getPojoId() == tv->getColumnId()){
					val->setPojoId(tv->getPojoId());
					val->setColumnId(tv->getColumnId());
				    val->setColumnRowId(tv->getColumnRowId());
				    val->options = tv->options;
				}
			}
		}

	    if (val->options.empty() || refreshOptions){
	    	getDataAsOptions(col->options,val->options);
	    }

		switch (col->objectType){
			case ColumnObject::TextView: {
				if (col->type == ColumnType::Index){
					Gtk::Box* box = dynamic_cast<Gtk::Box*>(col->object);

					if (box){
						Gtk::TextView* txt= (dynamic_cast<Gtk::TextView*>(box->get_first_child()->get_next_sibling()));
						if (txt){
							val->setValue(txt->get_buffer()->get_text());
						}else{
							val->setValue("-1");
						}
					}

				}else {
					val->setValue((dynamic_cast<Gtk::TextView*>(col->object))->get_buffer()->get_text());
				}
				break;

			}case ColumnObject::Combo: {
				val->setValue(std::to_string((dynamic_cast<rhea::ComboBox<StringColumns>*>(col->object))->get_active_row_number()));
				break;

			}case ColumnObject::ChkBox : {
				std::vector<std::string> tmp= {};
				Gtk::Widget* prvi = col->object->get_first_child();
				if (prvi){
					while (true){
						tmp.push_back((dynamic_cast<Gtk::CheckButton*>(prvi))->get_active()? "1":"0");
						if (prvi ==col->object->get_last_child()){
							break;
						}
						prvi = prvi->get_next_sibling();
					}
				}
				val->setValue(global->setOptions(tmp,","));
				break;

			}case ColumnObject::Options: {
				std::vector<std::string> tmp= {};
				Gtk::Widget* prvi = col->object->get_first_child();
				if (prvi){
					while (true){
						tmp.push_back((dynamic_cast<Gtk::CheckButton*>(prvi))->get_active()? "1":"0");
						if (prvi ==col->object->get_last_child()){
							break;
						}
						prvi = prvi->get_next_sibling();
					}
				}
				val->setValue(global->setOptions(tmp,","));
				break;

			}case ColumnObject::PckpDT : {
				val->setValue((dynamic_cast<rhea::PickupDT*>(col->object))->getAsISO8601());
				break;
			}case ColumnObject::Grid : {
				val->setValue((dynamic_cast<GridDataSection*>(col->object))->getAllData());
				break;
			}default: {
				val->setValue("#ERROR#");
			}
		}

//		std::cout << "uspešno shranil element" << std::endl;
		newPo->values.push_back(val);
	}

	newPo->files.clear();
	for (TemplateFile* tf : tmpl->templateFiles) {
		TemplateDataFile* tdf = new TemplateDataFile();
	    tdf->setPojoId(tf->getPojoId());
		tdf->setPojoVersion(newPo->getPojoVersion());
	    tdf->setTemplateFileRowId(tf->getRowId());
	    tdf->setTemplateRowId(tf->getTemplateRowId());
	    tdf->setPath(tmpl->getName(),true);
	    tdf->setName(std::to_string(newPo->getPojoId()) + "_" + std::to_string(newPo->getPojoVersion()) +"_"+ tf->getName());
//	    std::cout << "našel template datoteke in naredim novo: " << tdf->getFullName() << std::endl;
		newPo->files.push_back(tdf);
	}

//	std::cout << "shranjujem v bazo" << std::endl;
	rhea::DbParameters params = rhea::DbParameters();
	params.add(newPo->getPojoId());
	params.add(newPo->getPojoVersion());
    params.add(newPo->myTemplate->getRowId());
	params.add(newPo->isValid() ? 1 : 0 );
	params.add("");

	rhea::DbResultSet* res =  dbConn->execQuery("Insert into TemplateData("
	    " dataId,"
	    " version,"
	    " templRowId,"
	    " valid,"
	    " validTill ) VALUES (?,?,?,?,?) RETURNING rowId;",params.get());

	if (res->rowCount()>0){
//		std::cout << "uspešno shranil item v bazo" << std::endl;
		newPo->setRowId(res->getField("rowId")->getAsLong());
		for (TemplateValues* tv : newPo->values ) {
			rhea::DbParameters parVal = rhea::DbParameters();
			parVal.add(tv->getPojoId());
			parVal.add(newPo->getRowId());
			parVal.add(tv->getPojoVersion());
			parVal.add((long long) tv->getColumnRowId());
			parVal.add(tv->getValue());
			std::string optStr = global->setOptions(tv->options,global->optionsDelimiter);
			parVal.add(optStr);
			parVal.add(tv->isValid() ? 1 : 0 );
			parVal.add("");

			rhea::DbResultSet* resVal =  dbConn->execQuery("Insert into TemplateValues ("
    	    		" valueId,"
    		        " dataRowId,"
    		        " version,"
    		        " columnRowId,"
    		        " value,"
					" options,"
    	    	    " valid,"
    	    	    " validTill ) VALUES (?,?,?,?,?,?,?,?) RETURNING rowId;",parVal.get());
			if (resVal->rowCount()>0){
//				std::cout << "uspešno shranil element v bazo" << std::endl;
				tv->setRowId(resVal->getField("rowId")->getAsLong());
			}
		}

		for(TemplateDataFile* tdf : newPo->files){
//			std::cout << "shranjujem datoteke v bazo" << std::endl;
			rhea::DbParameters parFile = rhea::DbParameters();
			parFile.add(tdf->getPojoId());
			parFile.add(newPo->getRowId());
			parFile.add(tdf->getPojoVersion());
			parFile.add((long long) tdf->getTemplateFileRowId());
			parFile.add((long long) tdf->getTemplateRowId());
			parFile.add(tdf->getUUID());
			parFile.add(tdf->getName());
			parFile.add(tdf->getPath());
			parFile.add(tdf->isValid() ? 1 : 0 );
			parFile.add("");

			rhea::DbResultSet* resFile =  dbConn->execQuery("Insert into TemplateDataFiles ("
    	    		" fileId,"
    		        " dataRowId,"
    		        " version,"
					" templFileRowId,"
					" templRowId,"
					" uuid,"
    		        " name,"
					" path,"
    	    	    " valid,"
    	    	    " validTill ) VALUES (?,?,?,?,?,?,?,?,?,?) RETURNING rowId;",parFile.get());
			if (resFile->rowCount()>0){
//				std::cout << "uspešno shranil ime datoteke v bazo" << std::endl;
				tdf->setRowId(resFile->getField("rowId")->getAsLong());
			}
		}
	}

//	std::cout << "listed items" << std::endl;
	if ( mapOfPo.exists(std::to_string(newPo->getPojoId())) ) {
		mapOfPo.replace(std::to_string(newPo->getPojoId()),newPo);
		std::cout << "zamenjal item" << std::endl;
	} else {
		mapOfPo.add({std::to_string(newPo->getPojoId()),newPo});
		listingGrid->setRows(mapOfPo.size());
//		std::cout << "dodal item" << std::endl;
	}
//	std::cout << "listedmapa je urejena" << std::endl;

	poVersions.push_back(newPo);
	version = poVersions.size();
//	std::cout << "nastavljam verzije in gui" << std::endl;
	setVersionGui();
//	std::cout << "nastavljam listing" << std::endl;
	listingGrid->setRows(mapOfPo.size());
	listingGrid->refresh();
//	std::cout << "grem zdej zipat unzipat" << std::endl;
	populateTemplateFiles(newPo);
}

void FrmTemplData::onDelete() {
    std::string sqlStr = "";
    if( po->getRowId() > 0 ){
        po->setValid(false);
        time_t now = time(0);
        po->validTill->set(now);
        sqlStr = "UPDATE TemplateData SET valid = 0 , validTill= '" + po->validTill->getAsISO8601() + "' WHERE rowId=" + std::to_string(po->getRowId()) + ";";
        rhea::DbResultSet* dbOld = dbConn->execQuery(sqlStr);
        delete dbOld;
    }
    mapOfPo.remove(std::to_string(po->getPojoId()));

//    po = new TemplateData();
//    onNew();
//    populateGUI();
}

bool FrmTemplData::onClose() {
	return true;
}

void FrmTemplData::onActivate() {

	btnNew->set_label(langPack->getText("txtEditorNew_RHEA"));
	btnNew->set_tooltip_text("Novi element");

    btnSave->set_label(langPack->getText("txtEditorSave_RHEA"));
    btnSave->set_tooltip_text("Shrani element");

    btnDelete->set_label(langPack->getText("txtEditorDelete_RHEA"));
    btnDelete->set_tooltip_text("Izbriši element");

    btnImport->set_label(langPack->getText("txtEditorImport_RHEA"));
    btnImport->set_tooltip_text("Import");

    btnExport->set_label(langPack->getText("txtEditorExport_RHEA"));
    btnExport->set_tooltip_text("Export");

    btnOneDoc->set_label("1 Doc");
    btnOneDoc->set_tooltip_text("Generate all in one doc.");

    Gtk::Label* myLbl = dynamic_cast<Gtk::Label*>(filesBox->getHeader(0)->getWidget());
    myLbl->set_text(langPack->getText("txtfileBox_Files_RHEA"));
    Gtk::Button* myHdrBtn = dynamic_cast<Gtk::Button*>(filesBox->getHeader(1)->getWidget());
    myHdrBtn->set_label(langPack->getText("txtfileBox_Add_RHEA"));

}

void FrmTemplData::populateVersions(TemplateData* obj, std::vector<TemplateData*> &list) {
	rhea::DbParameters params = rhea::DbParameters();
	params.add(obj->getPojoId());
	rhea::DbResultSet* res = dbConn->execQuery("SELECT * FROM TemplateData WHERE dataId = ? ORDER BY version;",params.get());
	if (res->rowCount()>0){
        while(!res->isEof()){
        	TemplateData* td = populatePojo(res);
        	list.push_back(td);
           	res->moveNext();
        }
    }

}

void FrmTemplData::populateTemplateFiles(TemplateData* myPo) {
	for(rhea::File* file : myPo->files){

		if (file->getFullName().empty()){continue;}
		std::cout << "Vpisan je file: " << file->getFullName() << std::endl;
		rhea::File* tmp = new rhea::File();
		rhea::File* orig = new rhea::File();
		for (rhea::File* smth : myPo->myTemplate->templateFiles) {
			if (file->getName().ends_with(smth->getName())){
				orig->setFullName(smth->getFullName());
			}
		}
		tmp->setPath(file->getPath()+"tmp",false);

		rhea::Files* unzipped = orig->decompresZip(tmp->getBase()+tmp->getPath(),false);
		bool nasel = false;
		int index = 0;
		for (index = 0;index < unzipped->size();index++) {
			if (unzipped->getKey(index).ends_with("content.xml")){
				nasel = true;
				tmp->setName("content.xml");
				break;
			}
		}

		if (!nasel){continue;}
//		std::cout << "našel content" << std::endl;
		std::string content(reinterpret_cast<char *>( unzipped->getValueAt(index)->data),unzipped->getValueAt(index)->size);
		std::cout << "pred tem da zamenjam vsebino" << std::endl;
		std::vector<std::string> grids = {};
		rhea::ListedMap<std::string, std::string>* values= new rhea::ListedMap<std::string, std::string>();
		for (Column* col : myPo->myTemplate->columns) {
			if (col->libreCode.empty()){continue;}
			for(TemplateValues* val : myPo->values){
				if (val->getColumnId() == col->getPojoId()) {

					if (col->objectType == ColumnObject::Combo) {
						if ( global->isNumber(val->getValue())){
							int index =std::stoi(val->getValue());
							if (index>0){
								if ( val->options.size() > 0 ) {
									values->add(col->libreCode,val->options.at(index-1));
								} else {
									values->add(col->libreCode,col->options.at(index-1));
								}
							} else{
								values->add(col->libreCode,"");
							}
						}

					} else if (col->objectType == ColumnObject::ChkBox || col->objectType == ColumnObject::Options) {
						std::vector<std::string> tmpVect = {};
						global->parseOptions(val->getValue(),",",tmpVect);
						std::string tmp ="";
						int i = 0;
						std::vector<std::string>* opts = {nullptr};
						opts = val->options.size() > 0 ? &val->options : &col->options;

						for (std::string str : *opts){
							if (tmpVect.at(i)=="1"){
								tmp += str + ", ";
							}
							i++;
						}
						if (!tmp.empty()){
							tmp = tmp.substr(0,tmp.size()-2);
						}
						tmpVect.clear();
						values->add(col->libreCode,tmp);

					}else if (col->objectType == ColumnObject::PckpDT) {
						if(val->getValue() == "") {
							values->add(col->libreCode,"");
						} else {
							std::string day = val->getValue().substr(8,2);
							std::string mon = val->getValue().substr(5,2);
							day = day.starts_with('0')? day.substr(1,1): day;
							mon = mon.starts_with('0')? mon.substr(1,1): mon;
							std::string tmpStr = day + ". " + mon + ". " + val->getValue().substr(0,4);
							values->add(col->libreCode,tmpStr);

						}

					}else if ((col->objectType == ColumnObject::TextView) && (col->type == ColumnType::Index)) {
						std::string someVal =val->getValue();
						if (col->options.size()==3){
							someVal = col->options.at(0)+val->getValue()+col->options.at(2);
						}
						values->add(col->libreCode,someVal);

					} else if (col->objectType == ColumnObject::TextView) {
						values->add(col->libreCode,val->getValue());

					} else if (col->objectType == ColumnObject::Grid) {
						std::vector<std::string> rows = {};
						global->parseOptions(val->getValue(),global->gridDataRowDelimiter,rows);
						int i = 1;
						for (std::string & str : rows) {
							std::vector<std::string> cols = {};
							global->parseOptions(str,global->gridDataColDelimiter,cols);
							int j=1;
							for (std::string & colStr : cols) {
								std::string strTmp = col->libreCode + "["+ std::to_string(j) + "]["
												   + std::to_string(i) + "]";

								values->add(strTmp,colStr);
								j++;
							}
							i++;
						}
						std::string expGrid = col->libreCode + global->gridDataRowDelimiter + std::to_string(i-1);
						grids.push_back(expGrid);
					}
				}
			}
		}

		xmlDoc* doc = xmlReadMemory(content.c_str(), sizeof(char)*content.size(), unzipped->getKey(index).c_str(), NULL, 0);
		xmlNode* rootElement = xmlDocGetRootElement(doc);
		ExpandGrid(rootElement,grids);
		ReplaceText(rootElement,values);
		int size =-1;
		xmlChar * mem = NULL;
		xmlDocDumpMemory(doc,&mem, &size);
		unzipped->getValueAt(index)->size = size;
		unzipped->getValueAt(index)->data = (unsigned char *)malloc(size); //sizeof(unsigned char)*
		std::memcpy(unzipped->getValueAt(index)->data,mem,size);
		xmlFreeDoc(doc);
		xmlCleanupParser();

//		unzipped->getValueAt(index)->data = (unsigned char *)malloc(sizeof(unsigned char)*content.size());
//		std::copy(content.begin(), content.begin() + content.length(), unzipped->getValueAt(index)->data);
//		unzipped->getValueAt(index)->size = content.size();



		std::string s = file->getFullName();
		const char *zipname = s.c_str();
		zipFile zfile;
//		std::cout << "nastavim ime" << std::endl;
		zfile = zipOpen64(zipname, 0);
		if (zfile == NULL) {
			printf("Could not open %s for zipping\n", zipname);
			zipClose(zfile, NULL);
			continue;
		}
		for (int i = 0;i < unzipped->size();i++) {
//			std::cout << "podtikam content" << std::endl;
			std::string st = unzipped->getKey(i).substr(tmp->getBase().length()+tmp->getPath().length(), unzipped->getKey(i).length());
			const char* name = st.c_str();
			if (unzipped->getKey(i).ends_with("/")){
//				std::cout << "podtikam dir " << st << std::endl;
				if (!file->zipper_add_dir(zfile, name)) {
					printf("failed to write dir %s\n", name);
					zipClose(zfile, NULL);
					continue;
				}
			} else {
//				std::cout << "podtikam file" << st << std::endl;
				if (!file->zipper_add_buf(zfile, name, (const unsigned char *)unzipped->getValueAt(i)->data, unzipped->getValueAt(i)->size)) {
					printf("failed to write %s\n", name);
					zipClose(zfile, NULL);
					continue;
				}
			}
		}
//		std::cout << "še zapakiram in zaprem" << std::endl;
		zipClose(zfile, NULL);
//		std::cout << "DONE!" << std::endl;
	}
}

void FrmTemplData::fillImport() {
	Gtk::Root *root = btnImport->get_root();
	AppWindow *window = dynamic_cast<AppWindow*>(root);
    WndImportTD* wndImp = new WndImportTD();
    wndImp->set_transient_for(*window);

    TemplateData* xy = new TemplateData();
    xy->myTemplate = tmpl;
    wndImp->fill(xy, &mapOfPo, this);
    wndImp->set_modal();
    wndImp->show();
}

void FrmTemplData::fillExport() {
	Gtk::Root *root = btnExport->get_root();
	AppWindow *window = dynamic_cast<AppWindow*>(root);
    WndExportTD* wndExp = new WndExportTD();
    wndExp->set_transient_for(*window);

    TemplateData* xy = new TemplateData();
    xy->myTemplate = tmpl;
    wndExp->fill(xy, &mapOfPo);
    wndExp->set_modal();
    wndExp->show();

}


void FrmTemplData::ReplaceText(xmlNode * aNode, rhea::ListedMap<std::string, std::string>* values) {
    xmlNode *curNode = NULL;
    char p[] = {'p'};
    char span[] = {"span"};
    char s[] = {'s'};
    char tab[] = {"tab"};
    char text[] = {"text"};
//    std::vector<xmlNode *> contentOfP = {};
    std::vector<xmlNode*> forDelete = {};

	for (curNode = aNode; curNode; curNode = curNode->next) {
        if (curNode->type == XML_ELEMENT_NODE) {

        	std::cout << curNode->name << "\t" << xmlNodeGetContent(curNode) << std::endl;
            if (memcmp(curNode->name,p, sizeof(p)) == 0 ) {

  	           	int childCount = 0;
           		if (xmlChildElementCount(curNode)>0) {
           			xmlNode *child = xmlGetLastChild(curNode);
           			while (xmlChildElementCount(curNode)>=childCount) {
           				if (child== NULL){break;}
           				childCount++;
           				bool findPartial = false;

           				std::cout << child->name << " buri "<< xmlNodeGetContent(child) <<std::endl;
           				if ((memcmp(child->name,span, sizeof(span)) == 0) || (memcmp(child->name,text, sizeof(text)) == 0) ) {
           					xmlChar* words =  xmlNodeGetContent(child);
           					std::string content(reinterpret_cast<char *>( words),std::strlen((char*)words));

           					if (content.empty()) {continue;}

           					bool nasel = false;
           					int zgodnjifound = content.size();;
           					for(auto val : *values){
           						int myPos = content.find(val.first);
           						while (myPos != content.npos) {
           							if (zgodnjifound > myPos) {zgodnjifound = myPos;}
           							nasel = true;
           							content = content.replace(myPos,val.first.length(),val.second);
           							myPos = content.find(val.first);
           						}
           					}
           					if (nasel){
           						const xmlChar *temp =(const xmlChar *) content.c_str();
           						xmlNodeSetContent(child,temp);
           					}

           					findPartial = false;
           					for(auto val : *values) {
           						xmlNode *node = child;
           						std::string partial =content;
           						int myPos =zgodnjifound;
           						if (nasel) {

           							partial = content.substr(0,zgodnjifound);
           						}
           						int fog = 0;
           						bool AlredyFoundSpace = false;
           						std::string strAfter = "";
           						if (!AlredyFoundSpace) {
           							int pos = partial.find_first_of(' ');
           					    	if (pos != partial.npos) {
           					    		strAfter = partial.substr(pos);
           					    		partial = partial.substr(0,pos);

           					    		AlredyFoundSpace = true;
           					    	}
           					    }

           						while (val.first.ends_with( partial)) {
           							if (node->prev == NULL){break;}
           							std::cout <<"dogaja se nam" <<std::endl;
           							forDelete.push_back(node);
           							node = node->prev;

           							xmlChar* wordsTmp =  xmlNodeGetContent(node);
           							std::string prevStr (reinterpret_cast<char *>( wordsTmp),std::strlen((char*)wordsTmp));


           							int pos = prevStr.find_last_of(' ');
           							if (pos != prevStr.npos) {
           								prevStr = prevStr.substr(pos+1);
           							}

           							partial = prevStr+ partial;

           							if (partial == val.first) {
           								xmlChar* wordsStr =  xmlNodeGetContent(child);
           								std::string str (reinterpret_cast<char *>( wordsStr),std::strlen((char*)wordsStr));
           								str = str.substr(myPos);
           								const xmlChar *temp =(const xmlChar *) str.c_str();
           								xmlNodeSetContent(child,temp );

           								while(forDelete.size()>0){
           									int j = forDelete.size()-1;
           									xmlNode * toUnlink =forDelete.at(j);
           									xmlUnlinkNode(toUnlink);
           									forDelete.erase(forDelete.begin()+j);

           								}
           								forDelete.clear();
           								std::string mySolution (reinterpret_cast<char *>( wordsTmp),std::strlen((char*)wordsTmp));
           								mySolution = mySolution.substr(0,mySolution.size()-prevStr.length());
           								mySolution += val.second+strAfter;
           								const xmlChar *solution =(const xmlChar *) mySolution.c_str();
           								xmlNodeSetContent(node,solution );
           								findPartial = true;
           								break;
           							}
           							if (findPartial){
           							  	child = node;
           							    break;
           							}
           						}
           						if (findPartial){
           							child = node;
           							break;
           						}
           					}
           				}
           				if (findPartial) {continue;	}
           				child = child->prev;
           			}
  	           	} else {
  	           		xmlChar* sequence =  xmlNodeGetContent(curNode);
  	           		if (sequence){
  	           			std::string content(reinterpret_cast<char *>( sequence),std::strlen((char*)sequence));
  	           			bool nasel = false;
  	           			for(auto val : *values) {
  	           				int myPos = content.find(val.first);
  	           				while (myPos != content.npos) {
  	           					nasel = true;
  	           					content = content.replace(myPos,val.first.length(),val.second);
  	           					myPos = content.find(val.first);
  	           				}
  	           			}
  	           			if (nasel){
  	           				const xmlChar *temp =(const xmlChar *) content.c_str();
  	           				xmlNodeSetContent(curNode,temp);
  	           			}
  	           		}
  	           	}
            }
        }

        if (xmlChildElementCount(curNode)>0 ){
        	ReplaceText(curNode->children,values);
        }
	}

}

void FrmTemplData::ExpandGrid(xmlNode * aNode,std::vector<std::string> &grids) {
	xmlNode *curNode = NULL;
	char p[] = {'p'};
	char row[] = {"table-row"};
	char cell[] = {"table-cell"};

   	static std::vector <int> myRows = {};
   	static std::vector<bool> didExpand = {};
	static std::vector <int> allRows = {};
	for (curNode = aNode; curNode; curNode = curNode->next) {
		if (curNode->type == XML_ELEMENT_NODE) {
			if (memcmp(curNode->name,row, sizeof(row)) == 0 ) {
	           	xmlChar* sequence =  xmlNodeGetContent(curNode);
	           	if (!sequence){ continue;}
	           	std::string content(reinterpret_cast<char *>( sequence),std::strlen((char*)sequence));

	           	int tableIndex = 0;

	           	for (std::string myDelimiter : grids){
	           		int aRow = 1;
	           		myRows.push_back(aRow);
	           		bool expansion = false;
	           		didExpand.push_back(expansion);

	           		int pos  = myDelimiter.find(global->gridDataRowDelimiter);
					int allRowsForTable = std::stoi(myDelimiter.substr(pos+global->gridDataRowDelimiter.size()));
					allRows.push_back(allRowsForTable);
					myDelimiter = myDelimiter.substr(0,pos);
					if (content.find(myDelimiter+"[") != content.npos ){
						if(!didExpand.at(tableIndex)) {
							for (int i = 1 ; i< allRows.at(tableIndex); i++) {
								xmlNode * newRow = xmlCopyNode(curNode,1);
								xmlAddNextSibling(curNode,newRow);
								expansion = true;
								didExpand.at(tableIndex) = expansion;
							}

						}
						xmlNode *myCell = xmlGetLastChild(curNode);
						if (memcmp(myCell->name,cell, sizeof(cell)) == 0 ) {
							while (myCell) {
								xmlNode *myP = xmlGetLastChild(myCell);
								if (memcmp(myP->name,p, sizeof(p)) == 0 ) {
									xmlChar* str =  xmlNodeGetContent(myP);
									if (!str){ continue;}
										std::string textInCell(reinterpret_cast<char *>( str),std::strlen((char*)str));
										textInCell += "[" + std::to_string(myRows.at(tableIndex)) + "]";
										std::cout <<"MAM CELLICO    "  << textInCell << std::endl;
										const xmlChar *temp =(const xmlChar *) textInCell.c_str();
										xmlNodeSetContent(myP,temp);
								}
								myCell = myCell->prev;
							}

						}
						myRows.at(tableIndex)++;
						std::cout << "MATRAM BRAZGOTINE" <<  std::endl;

					}
					tableIndex++;
				}
			}
		}
		if (xmlChildElementCount(curNode)>0 ){
			ExpandGrid(curNode->children,grids);
		}
	}

	if (allRows.empty()) {return;}
	for (int i=myRows.size()-1; i>0;i-- ) {
	   if (myRows.at(i) == allRows.at(i)){
          	myRows.erase(myRows.begin()+i);
          	allRows.erase(allRows.begin()+i);
          	didExpand.erase(didExpand.begin()+i);
	   }
   }

}
