/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#include "main/main.h"
#include "FrmTemplDef.h"
#include "iostream"


FrmTemplDef::FrmTemplDef() : Gtk::Frame() {
	auto display = Gdk::Display::get_default();

	btnDetails->add_css_class("btnDetails");
	btnDetails->set_vexpand(false);
	btnAddColumn->add_css_class("btnAddColumn");
	btnSaveColumns->add_css_class("btnSaveColumns");
	btnAddTemplate->add_css_class("btnAddTemplate");
	btnBack->add_css_class("btnBack");
	lblForTemplate->add_css_class("lblForTemplate");
    std::string cssData =".rhea_gridHeaderLbl {background-color: #5391c1; color: #FFFFFF; border-color: black; border-style: solid solid solid solid;border-width: 0.5px;} ";
    cssData += ".rhea_gridRowEven {background-color: #FFFFFF; color:black; border-color: black; border-style: none none  solid solid; border-width: 0.7px;} ";
    cssData += ".rhea_gridRowEven .view, .rhea_gridRowEven .view text {background-color: #FFFFFF; color:black;} ";
    cssData += ".rhea_gridRowOdd {background-color: #E0E0E0; color:black; border-color: black;border-style: none none solid solid;border-width: 0.7px;} ";
    cssData += ".rhea_gridRowOdd .view, .rhea_gridRowOdd .view text {background-color: #E0E0E0; color:black; } ";
    cssData += ".rhea_selectedRow, .rhea_selectedRow .view, .rhea_selectedRow .view text, .rhea_selectedRow .btnGrid, {caret-color: white; background-image: none; background-color: #466176; color: #FFFFFF;} "; //# //567423
    cssData += ".btnBack {border-radius:7px; margin-top:10px; margin-bottom:10px; margin-left:30px;}";
    cssData += ".btnAddColumn, .btnSaveColumns, .btnAddTemplate {border-radius:7px; margin-top:20px; margin-left:30px;margin-bottom:20px; margin-right:40px;}";
	cssData += ".btnDetails {border-radius:7px; margin-top:20px; margin-left:45px; margin-right:10px ; margin-bottom: 45px; width:60px; height: 45px;}";
	cssData += ".lblForTemplate {margin-top:50px;}";
	cssData += ".btnModifier {border-radius:7px; margin-left:5px; margin-right: 5px; color:#000000;}";
	cssData += ".btnSelectedModifier {background-color: #5391c1; color: #FFFFFF;}";

	cssProvider->load_from_data(cssData);
	btnSaveColumns->get_style_context()->add_provider_for_display(display,cssProvider,GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
	btnAddColumn->get_style_context()->add_provider_for_display(display,cssProvider,GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
	btnAddTemplate->get_style_context()->add_provider_for_display(display,cssProvider,GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);

	objectList->clear();
    auto rowObject = *objectList->append();
    rowObject[stringColumns.col_id] = 1;
    rowObject[stringColumns.col_name] = lang->getText("txtTemplDef_objectList1");

    rowObject = *objectList->append();
    rowObject[stringColumns.col_id] = 2;
    rowObject[stringColumns.col_name] = lang->getText("txtTemplDef_objectList2");

    rowObject = *objectList->append();
    rowObject[stringColumns.col_id] = 3;
    rowObject[stringColumns.col_name] = lang->getText("txtTemplDef_objectList3");

    rowObject = *objectList->append();
    rowObject[stringColumns.col_id] = 4;
    rowObject[stringColumns.col_name] = lang->getText("txtTemplDef_objectList4");

    rowObject = *objectList->append();
    rowObject[stringColumns.col_id] = 5;
    rowObject[stringColumns.col_name] = lang->getText("txtTemplDef_objectList5");


    rowObject = *objectList->append();
    rowObject[stringColumns.col_id] = 6;
    rowObject[stringColumns.col_name] = lang->getText("txtTemplDef_objectList6");

    typeList->clear();
    auto row = *typeList->append();
    row[stringColumns.col_id] = 1;
    row[stringColumns.col_name] = lang->getText("txtTemplDef_typeList1");

    row = *typeList->append();
    row[stringColumns.col_id] = 2;
    row[stringColumns.col_name] = lang->getText("txtTemplDef_typeList2");

    row = *typeList->append();
    row[stringColumns.col_id] = 3;
    row[stringColumns.col_name] = lang->getText("txtTemplDef_typeList3");

    row = *typeList->append();
    row[stringColumns.col_id] = 4;
    row[stringColumns.col_name] = lang->getText("txtTemplDef_typeList4");

    row = *typeList->append();
    row[stringColumns.col_id] = 5;
    row[stringColumns.col_name] = lang->getText("txtTemplDef_typeList5");

    btnAddTemplate->set_label(lang->getText("txtTemplDef_btnAddTemplate"));
    btnAddColumn->set_label(lang->getText("txtTemplDef_btnAddColumn"));
    btnSaveColumns->set_label(lang->getText("txtTemplDef_btnSaveColumns"));
    btnDetails->set_label(lang->getText("txtTemplDef_btnDetails"));
    btnBack->set_label(lang->getText("txtTemplDef_btnBack"));

    rhea::DbResultSet* res = dbConn->execQuery("SELECT * FROM Templates WHERE valid = 1 ORDER BY templId;");
    if (res->rowCount()>0){
    	while(!res->isEof()){
        	Template* tmp = populateTemplates(res);
            templates.add({tmp->getPojoName(),tmp});
            res->moveNext();
    	}
	}
    delete res;

	btnDetails->set_size_request(60,45);
	btnDetails->signal_clicked().connect(sigc::mem_fun(*this, &FrmTemplDef::btnDetailsOnClick));
	btnDetails->set_sensitive(false);
    lblForTemplate->set_text(lang->getText("txtTemplDef_lblForTemplateDefault"));
    boxToDetais->append(*lblForTemplate);
    boxToDetais->append(*btnDetails);

    templateList->connectOnClick(&FrmTemplDef::OnListGridClick,this);
    templateList->connectWidget(&FrmTemplDef::connectWidgetList,this);
    std::vector<std::string> hdrsList = { "", lang->getText("txtTemplDef_hdrsListCol1") , lang->getText("txtTemplDef_hdrsListCol2"), lang->getText("txtTemplDef_hdrsListCol3"), lang->getText("txtTemplDef_hdrsListCol4"), ""};
    templateList->setHeaders(&hdrsList, 0);
    templateList->setRows(templates.size());

    templateDef->connectOnClick(&FrmTemplDef::OnDefGridClick,this);
    templateDef->connectWidget(&FrmTemplDef::connectWidgetDef,this);
    std::vector<std::string> hdrsDef = { lang->getText("txtTemplDef_hdrsDefCol0"),
    		                             lang->getText("txtTemplDef_hdrsDefCol1"),
										 lang->getText("txtTemplDef_hdrsDefCol2"),
										 lang->getText("txtTemplDef_hdrsDefCol3"),
										 lang->getText("txtTemplDef_hdrsDefCol4"),
										 lang->getText("txtTemplDef_hdrsDefCol5"),""};
    templateDef->setHeaders(&hdrsDef, 0);

    btnAddTemplate->signal_clicked().connect(sigc::mem_fun(*this, &FrmTemplDef::btnAddTemplateClick));
//    btnAddTemplate->set_image_from_icon_name("document-new");
    boxListedTemplates->append(*btnAddTemplate);
    boxListedTemplates->append(*templateList);


    btnBack->signal_clicked().connect(sigc::mem_fun(*this, &FrmTemplDef::btnBackClick));
    btnAddColumn->signal_clicked().connect(sigc::mem_fun(*this, &FrmTemplDef::btnAddColumnClick));
//    btnAddColumn->set_image_from_icon_name("document-new");
    btnSaveColumns->signal_clicked().connect(sigc::mem_fun(*this, &FrmTemplDef::btnSaveColumnsClick));
//    btnSaveColumns->set_image_from_icon_name("document-save");


    boxButtonBox->append(*btnAddColumn);
	boxButtonBox->append(*btnSaveColumns);
	boxButtonsDef->append(*btnBack);
	boxButtonsDef->append(*boxButtonBox);

	boxButtonBox->set_hexpand(true);
    boxButtonBox->set_halign(Gtk::Align::CENTER);
    boxTemplDef->append(*boxButtonsDef);
    boxTemplDef->append(*templateDef);
    boxTemplDef->append(*fileBox);

    Gtk::Label* myLbl = dynamic_cast<Gtk::Label*>(fileBox->getHeader(0)->getWidget());
    myLbl->set_text(lang->getText("txtfileBox_Files_RHEA"));
    Gtk::Button* myHdrBtn = dynamic_cast<Gtk::Button*>(fileBox->getHeader(1)->getWidget());
    myHdrBtn->set_label(lang->getText("txtfileBox_Add_RHEA"));


    fileBox->connectOnAddFile( &FrmTemplDef::addFile, this);
	fileBox->connectOnDeleteFile( &FrmTemplDef::removeFile, this);

	box->append(*boxListedTemplates);
	box->append(*boxToDetais);


    set_child(*box);

	templateList->addCSS(cssData);
	templateDef->addCSS(cssData);

    templateList->refresh();

    this->set_name("kuskus");
}

FrmTemplDef::~FrmTemplDef() {

}

long FrmTemplDef::setId() {
    rhea::DbResultSet* res = dbConn->execQuery("SELECT COALESCE(MAX(templId)+1,1) from Templates;");
    long result = res->getField(0)->getAsLong();
    delete res;
    return result;
}

Template* FrmTemplDef::populateTemplates(rhea::DbResultSet* res) {
	Template* tmp = new Template();
	tmp->setRowId(res->getField("rowId")->getAsLong());
	tmp->setMenuVisibility(res->getField("showInMenu")->getAsInteger()==1);
	tmp->setName(res->getField("name")->getAsString());
	tmp->setPojoId(res->getField("templId")->getAsLong());
	tmp->setPojoName(res->getField("templId")->getAsString());

	std::string shcut = res->getField("shortcut")->getAsString();
	int kmask =std::stoi(shcut.substr(0,1));
	shcut = shcut.substr(1);
	if (shcut.starts_with(" ")){
		shcut = shcut.substr(1);
	}
	tmp->setShortcut(kmask,shcut);

	rhea::DbParameters params = rhea::DbParameters();
	params.add(res->getField("templId")->getAsString());
    rhea::DbResultSet* resCol = dbConn->execQuery("SELECT * FROM TemplateColumns WHERE valid = 1 and "
    						  " templRowId in (select rowId from Templates where templId = ?) ORDER BY orderWeight;", params.get());
	if (resCol->rowCount()>0){

		while(!resCol->isEof()){
			ColumnW* col = new ColumnW();
			col->setRowId(resCol->getField("rowId")->getAsLong());
			col->setPojoId(resCol->getField("columnId")->getAsLong());
			col->setUUID(resCol->getField("uuid")->getAsString());
			col->order = resCol->getField("orderWeight")->getAsInteger();
			col->label = resCol->getField("label")->getAsString();
			col->libreCode = resCol->getField("libreCode")->getAsString();
			std::string object = resCol->getField("object")->getAsString();
			col->objectType = object == "TextView" ? ColumnObject::TextView :
						  object == "Combo" ? ColumnObject::Combo :
						  object == "ChkBox" ? ColumnObject::ChkBox :
						  object == "Options" ? ColumnObject::Options :
						  object == "PickupDt" ? ColumnObject::PckpDT :
						  object == "Grid"	? ColumnObject::Grid	  :ColumnObject::TextView;
			std::string type = resCol->getField("type")->getAsString();
			col->type = type == "String" ? ColumnType::String :
						type == "Int" ? ColumnType::Int :
						type == "Double" ? ColumnType::Double :
						type == "Date" ? ColumnType::Date :
						type == "Index" ? ColumnType::Index: ColumnType::String;
			global->parseOptions(resCol->getField("options")->getAsString(),global->optionsDelimiter,col->options);
			tmp->columns.push_back(col);
			resCol->moveNext();
		}
	}
	delete resCol;

	rhea::DbResultSet* resFiles = dbConn->execQuery("SELECT * FROM TemplateFiles WHERE valid = 1 and "
	    						  " templRowId in (select rowId from Templates where templId = ?) ORDER BY rowId;", params.get());
	if (resFiles->rowCount()>0){
		while(!resFiles->isEof()){
			TemplateFile* tmpFile = new TemplateFile();
			tmpFile->setRowId(resFiles->getField("rowId")->getAsLong());
			tmpFile->setPojoId(resFiles->getField("fileId")->getAsLong());
			tmpFile->setUUID(resFiles->getField("uuid")->getAsString());
			tmpFile->setPojoVersion(resFiles->getField("version")->getAsInteger());
			tmpFile->setFullName(resFiles->getField("path")->getAsString()+resFiles->getField("name")->getAsString());
			tmp->templateFiles.push_back(tmpFile);
			resFiles->moveNext();
		}
	}

	delete resFiles;
	return tmp;
}

void FrmTemplDef::btnDetailsOnClick () {
	if (selectedTemplate < 0) {return;}
	templateDef->setRows(templates.getValueAt(selectedTemplate)->columns.size());
	templateDef->refresh();
	fileBox->removeAll();
	for (auto& tmp : templates.getValueAt(selectedTemplate)->templateFiles){
		fileBox->addFile(tmp->getFullName());
	}

	boxListedTemplates->unparent();
	boxToDetais->unparent();
	box->append(*boxTemplDef);
}

void FrmTemplDef::OnListGridClick(rhea::GridCell *cell) {
	selectedTemplate = cell->getRow();
	lblForTemplate->set_text(lang->getText("txtTemplDef_lblForTemplate") +templates.getValueAt(selectedTemplate)->getName());
	btnDetails->set_sensitive(true);
}

void FrmTemplDef::btnModifierOnClick (Gtk::Button *btn, int row) {
	if (btn->has_css_class("btnSelectedModifier")) {
		btn->remove_css_class("btnSelectedModifier");
	} else {
		btn->add_css_class("btnSelectedModifier");
	}
	selectedTemplate = row;
	lblForTemplate->set_text(lang->getText("txtTemplDef_lblForTemplate") +templates.getValueAt(selectedTemplate)->getName());
	btnDetails->set_sensitive(true);

}

Gtk::Widget* FrmTemplDef::connectWidgetList(int col, int row) {
	if (col == 0) {
		Gtk::Button* btn = new Gtk::Button();
		btn->set_label(lang->getText("txtTemplDef_btnDelete"));
		btn->signal_clicked().connect(sigc::bind(sigc::mem_fun(*this, &FrmTemplDef::btnDeleteListOnClick),btn,row));
		btn->add_css_class("btnGrid");
		return btn;

	}else if (col == 1) {
        Gtk::TextView* tv = new Gtk::TextView();
        tv->set_size_request(20, 35);
        tv->show();
        tv->set_editable(false);
        tv->get_buffer()->set_text(templates.getValueAt(row)->getPojoName());
        return tv;

    } else if (col == 2) {
        Gtk::TextView* tv = new Gtk::TextView();
        tv->set_size_request(100, 35);
        tv->show();
        tv->get_buffer()->set_text(templates.getValueAt(row)->getName());
        return tv;

    } else if (col == 3) {
    	int mask = 0;
    	std::string key ="";
    	templates.getValueAt(row)->getShortCut(mask,key);
    	Gtk::Box* scBox = new Gtk::Box();
    	Gtk::Button* btnShift = new Gtk::Button("Shift");
    	btnShift->add_css_class("btnModifier");
    	btnShift->signal_clicked().connect(sigc::bind(sigc::mem_fun(*this, &FrmTemplDef::btnModifierOnClick),btnShift,row));
    	if (KeyMasks::Shift & mask) {
    		btnShift->add_css_class("btnSelectedModifier");
    	}
    	Gtk::Button* btnCtrl = new Gtk::Button("CTRL");
    	btnCtrl->add_css_class("btnModifier");
    	btnCtrl->signal_clicked().connect(sigc::bind(sigc::mem_fun(*this, &FrmTemplDef::btnModifierOnClick),btnCtrl,row));
    	if (KeyMasks::Ctrl & mask) {
    		btnCtrl->add_css_class("btnSelectedModifier");
    	}

    	Gtk::Button* btnAlt = new Gtk::Button("Alt");
    	btnAlt->add_css_class("btnModifier");
    	btnAlt->signal_clicked().connect(sigc::bind(sigc::mem_fun(*this, &FrmTemplDef::btnModifierOnClick),btnAlt,row));
    	if (KeyMasks::Alt & mask) {
    		btnAlt->add_css_class("btnSelectedModifier");
    	}

        Gtk::TextView* tv = new Gtk::TextView();
        tv->set_size_request(40, 35);
        tv->show();
        tv->get_buffer()->set_text(key);
        scBox->append(*btnShift);
        scBox->append(*btnCtrl);
        scBox->append(*btnAlt);
        scBox->append(*tv);
        return scBox;

    } else if (col == 4) {
    	Gtk::CheckButton* chk = new Gtk::CheckButton();
    	chk->set_size_request(20,35);
    	chk->set_active(templates.getValueAt(row)->getMenuVisibility());
    	chk->signal_toggled().connect(sigc::bind(sigc::mem_fun(*this, &FrmTemplDef::btnToggleListOnClick),chk,row));
    	return chk;
    } else if (col == 5) {
        Gtk::Button* btn = new Gtk::Button();
        btn->set_label(lang->getText("txtTemplDef_btnSave"));
        btn->signal_clicked().connect(sigc::bind(sigc::mem_fun(*this, &FrmTemplDef::btnSaveListOnClick),btn,row));
        btn->add_css_class("btnGrid");
        return btn;

    }
    Gtk::Label *tv = new Gtk::Label();
    return tv;
}

void FrmTemplDef::cmbTypeOnChange( rhea::ComboBox<StringColumns>* cmb, Gtk::TreeRow oldChoice, Gtk::TreeRow newChoice) {
    if (oldChoice){
    	if ( oldChoice[stringColumns.col_id] == 5) {
        	rhea::GridCell* parent = dynamic_cast<rhea::GridCell*> (cmb->get_parent());
        	if (parent) {
        		templates.getValueAt(selectedTemplate)->columns.at(parent->getRow())->options.clear();
    			Gtk::Button* btn = dynamic_cast<Gtk::Button*>(templateDef->getCell(5,parent->getRow())->widget);
           		if (btn) {
           			btn->set_label("NULL");
           		}
        	}
    	}
    }

    if (newChoice){
        if ( newChoice[stringColumns.col_id] == 5) {
        	rhea::GridCell* parent = dynamic_cast<rhea::GridCell*> (cmb->get_parent());
        	if (parent) {
        		templates.getValueAt(selectedTemplate)->columns.at(parent->getRow())->options.clear();
        		templates.getValueAt(selectedTemplate)->columns.at(parent->getRow())->options.push_back("");
        		templates.getValueAt(selectedTemplate)->columns.at(parent->getRow())->options.push_back("1");
        		templates.getValueAt(selectedTemplate)->columns.at(parent->getRow())->options.push_back("");
        		Gtk::Button* btn = dynamic_cast<Gtk::Button*>(templateDef->getCell(5,parent->getRow())->widget);
               	if (btn) {
                	btn->set_label(" 1 ");
               	}

        	}
        }
    }

}

Gtk::Widget *FrmTemplDef::connectWidgetDef(int col, int row) {
	Template* temp = templates.getValueAt(selectedTemplate);

	switch (col) {
		case 0: {
			Gtk::TextView *tv = new Gtk::TextView();
			tv->set_size_request(100, 35);
			tv->show();
			tv->get_buffer()->set_text(temp->columns.at(row)->label);
			return tv;

		} case 1: {
			Gtk::TextView *tv = new Gtk::TextView();
			tv->set_size_request(100, 35);
			tv->show();
			tv->get_buffer()->set_text(temp->columns.at(row)->libreCode);
			return tv;

		} case 2: {
			Gtk::TextView *tv = new Gtk::TextView();
			tv->set_size_request(20, 35);
			tv->show();
			tv->get_buffer()->set_text(std::to_string(temp->columns.at(row)->order));
			return tv;

		} case 3: {
			rhea::ComboBox<StringColumns> *cmb = new rhea::ComboBox<StringColumns>();
			cmb->setOnChange(&FrmTemplDef::cmbTypeOnChange,this);
			cmb->set_size_request(100, 35);
			cmb->show();
			cmb->add_css_class("cmbBox");
			cmb->addList(&typeList);

			switch(temp->columns.at(row)->type){
	    		case ColumnType::Date: {
	    			cmb->set_active(1);
	    			break;
	    		} case ColumnType::Double: {
	    			cmb->set_active(2);
	    			break;
	    		} case ColumnType::Int: {
	    			cmb->set_active(3);
	    			break;
	    		} case ColumnType::String: {
	    			cmb->set_active(0);
	    			break;
	    		} case ColumnType::Index : {
	    			cmb->set_active(4);
	    			break;
	    		} default: {
	    			cmb->set_active(0);
	    		}
			}
			return cmb;

		} case 4: {
			rhea::ComboBox<StringColumns> *cmb = new rhea::ComboBox<StringColumns>();
			cmb->set_size_request(100, 35);
			cmb->show();
			cmb->add_css_class("cmbBox");
			cmb->addList(&objectList);

			switch(temp->columns.at(row)->objectType){
	    		case ColumnObject::TextView : {
	    			cmb->set_active(0);
	    			break;
	    		} case ColumnObject::ChkBox: {
	    			cmb->set_active(1);
	    			break;
	    		} case ColumnObject::Combo: {
	    			cmb->set_active(2);
	    			break;
	    		} case ColumnObject::Options: {
	    			cmb->set_active(3);
	    			break;
	    		}case ColumnObject::PckpDT: {
	    			cmb->set_active(4);
	    			break;
	    		}case ColumnObject::Grid: {
	    			cmb->set_active(5);
	    			break;
	    		} default: {
	    			cmb->set_active(0);
	    		}
			}
			return cmb;

		} case 5: {
			std::string tmp  = "";
			for(std::string str : temp->columns.at(row)->options){
				if(str.starts_with(global->gridColumnDelimiter)){
					continue;
				}
				if (str.starts_with(global->gridTemplateDelimiter)) {
					std::string tempId = str.substr(global->gridTemplateDelimiter.size());
					int pos = tempId.find(global->gridTemplateDelimiter);
					tempId = tempId.substr(0,pos);
					for(auto& item : templates.getMap()) {
						if (item.second->getPojoId() == std::stol(tempId)){
							tmp += item.second->getName() + "\n";
						}
					}
				} else {
					tmp += str + "\n";
				}
			}
			if(tmp.empty()){
				tmp = "...";
			}
			Gtk::Button *opt = new Gtk::Button(tmp);
			opt->signal_clicked().connect(sigc::bind(sigc::mem_fun(*this, &FrmTemplDef::btnOptionsOnClick),opt,row));
			opt->add_css_class("btnGrid");
			return opt;
		} case 6: {
			Gtk::Button *del = new Gtk::Button(lang->getText("txtTemplDef_btnDelete"));
			del->signal_clicked().connect(sigc::bind(sigc::mem_fun(*this, &FrmTemplDef::btnDelColumnOnClick),del,row));
			del->add_css_class("btnGrid");
			return del;
		}
	}
    Gtk::TextView *tv = new Gtk::TextView();
    tv->set_size_request(100, 35);
    tv->show();
	return tv;
}

void FrmTemplDef::btnAddTemplateClick() {
	Template* tmp = new Template();
	tmp->setPojoId(setId());
	tmp->setPojoName(std::to_string(tmp->getPojoId()));
	templates.add({tmp->getPojoName(),tmp});
	templateList->setRows(templates.size());
	templateList->refresh();
}

void FrmTemplDef::btnBackClick() {

	boxTemplDef->unparent();

	box->append(*boxListedTemplates);
	box->append(*boxToDetais);

}

void FrmTemplDef::btnAddColumnClick(){
	if (selectedTemplate<0) {return;}
	btnSaveColumnsClick();
	ColumnW* col = new ColumnW();
	templates.getValueAt(selectedTemplate)->columns.push_back(col);
	templateDef->setRows(templates.getValueAt(selectedTemplate)->columns.size());
	templateDef->refresh();
}

void FrmTemplDef::btnDeleteListOnClick(Gtk::Button *btn, int row) {

	Template* templ = templates.getValueAt(row);
    updateTemplate(templ);

    templates.removeAt(row);

	templateList->setRows(templates.size());
	templateList->refresh();


    global->createMenuItems();

	selectedTemplate = -1;
	btnDetails->set_sensitive(false);
    lblForTemplate->set_text(lang->getText("txtTemplDef_lblForTemplateDefault"));

}

void FrmTemplDef::btnToggleListOnClick(Gtk::CheckButton *chk, int row) {
	selectedTemplate = row;
	lblForTemplate->set_text(lang->getText("txtTemplDef_lblForTemplate") +templates.getValueAt(selectedTemplate)->getName());
	btnDetails->set_sensitive(true);
}

void FrmTemplDef::btnSaveListOnClick(Gtk::Button *btn, int row) {
    Template* templ = templates.getValueAt(row);
    updateTemplate(templ);

    templ->setName(dynamic_cast<Gtk::TextView*>(templateList->getWidget(2,row))->get_buffer()->get_text());
    templ->setMenuVisibility(dynamic_cast<Gtk::CheckButton*>(templateList->getWidget(4,row))->get_active());

    int mask=0;
    Gtk::Box* box= dynamic_cast<Gtk::Box*>(templateList->getWidget(3,row));
    Gtk::Button* shift = dynamic_cast<Gtk::Button*>(box->get_first_child());
    if (shift->has_css_class("btnSelectedModifier")) {
    	mask = KeyMasks::Shift;
    }
    Gtk::Button* ctrl = dynamic_cast<Gtk::Button*>(shift->get_next_sibling());
    if (ctrl->has_css_class("btnSelectedModifier")) {
    	mask += KeyMasks::Ctrl;;
    }
    Gtk::Button* alt = dynamic_cast<Gtk::Button*>(ctrl->get_next_sibling());
    if (alt->has_css_class("btnSelectedModifier")) {
    	mask += KeyMasks::Alt;
    }

    Gtk::TextView* tv = dynamic_cast<Gtk::TextView*>(alt->get_next_sibling());
    std::string txt = tv->get_buffer()->get_text();
    if (!(txt.empty()) && (txt !=" ") && (txt !="\t") ) {
    	templ->setShortcut(mask,txt);
    }

	insertTemplate(templ);

    global->createMenuItems();
    global->createShortCuts();

	selectedTemplate = row;
	lblForTemplate->set_text(lang->getText("txtTemplDef_lblForTemplate") +templates.getValueAt(selectedTemplate)->getName());
	btnDetails->set_sensitive(true);
}


void FrmTemplDef::OnDefGridClick(rhea::GridCell *cell) {

}

void FrmTemplDef::onActivate() {
	if (selectedTemplate>0){
		lblForTemplate->set_text(lang->getText("txtTemplDef_lblForTemplate") +templates.getValueAt(selectedTemplate)->getName());
	}else{
		lblForTemplate->set_text(lang->getText("txtTemplDef_lblForTemplateDefault"));
	}
	btnAddTemplate->set_label(lang->getText("txtTemplDef_btnAddTemplate"));
    btnAddColumn->set_label(lang->getText("txtTemplDef_btnAddColumn"));
    btnSaveColumns->set_label(lang->getText("txtTemplDef_btnSaveColumns"));
    btnDetails->set_label(lang->getText("txtTemplDef_btnDetails"));
    btnBack->set_label(lang->getText("txtTemplDef_btnBack"));

    templateList->getHeader(1)->setCaption(lang->getText("txtTemplDef_hdrsListCol1"));
    templateList->getHeader(2)->setCaption(lang->getText("txtTemplDef_hdrsListCol2"));
    templateList->getHeader(3)->setCaption(lang->getText("txtTemplDef_hdrsListCol3"));
    templateList->getHeader(4)->setCaption(lang->getText("txtTemplDef_hdrsListCol4"));
    templateList->refresh();

    int index=1;
    for (auto row: objectList->children()) {
    	if(row[stringColumns.col_id]==index){
    		row[stringColumns.col_name] = lang->getText("txtTemplDef_objectList"+std::to_string(index));
    	}
    	index++;
    }

    index=1;
    for (auto row: typeList->children()) {
    	if(row[stringColumns.col_id]==index){
    		row[stringColumns.col_name] = lang->getText("txtTemplDef_typeList"+std::to_string(index));
    	}
    	index++;
    }

    templateDef->getHeader(0)->setCaption(lang->getText("txtTemplDef_hdrsDefCol0"));
    templateDef->getHeader(1)->setCaption(lang->getText("txtTemplDef_hdrsDefCol1"));
    templateDef->getHeader(2)->setCaption(lang->getText("txtTemplDef_hdrsDefCol2"));
    templateDef->getHeader(3)->setCaption(lang->getText("txtTemplDef_hdrsDefCol3"));
    templateDef->getHeader(4)->setCaption(lang->getText("txtTemplDef_hdrsDefCol4"));
    templateDef->getHeader(5)->setCaption(lang->getText("txtTemplDef_hdrsDefCol5"));
    templateDef->refresh();

    Gtk::Label* myLbl = dynamic_cast<Gtk::Label*>(fileBox->getHeader(0)->getWidget());
    myLbl->set_text(lang->getText("txtfileBox_Files_RHEA"));
    Gtk::Button* myHdrBtn = dynamic_cast<Gtk::Button*>(fileBox->getHeader(1)->getWidget());
    myHdrBtn->set_label(lang->getText("txtfileBox_Add_RHEA"));

}

bool FrmTemplDef::onClose() {
    return true;
}

void FrmTemplDef::insertTemplate(Template* templ) {
	rhea::DbParameters params = rhea::DbParameters();
    params.add(templ->getPojoId());
    params.add(templ->getPojoVersion()+1);
    params.add(templ->getName());
    params.add(templ->getLabel());
    params.add(templ->getShortCut());
    params.add(templ->getMenuVisibility() ? 1 : 0 );
    params.add(templ->isValid() ? 1 : 0 );
    params.add(templ->validTill->getAsISO8601());

    rhea::DbResultSet* res =  dbConn->execQuery("Insert into Templates ("
    " templId,"
    " version,"
    " name,"
    " menuLabel,"
	" shortcut,"
	" showInMenu,"
    " valid,"
    " validTill ) VALUES (?,?,?,?,?,?,?,?) RETURNING rowId;",params.get());

    if (res->rowCount()>0){
    	templ->setRowId(res->getField("rowId")->getAsLong());
    }
    delete res;
}

void FrmTemplDef::updateTemplate (Template* templ) {
	rhea::DbResultSet* res =  dbConn->execQuery("Update Templates set valid = 0 where rowId = " + std::to_string(templ->getRowId())+";");
	delete res;
}

void FrmTemplDef::addFile(rhea::GridCell* cell, std::string fileName) {

	Template* templ = templates.getValueAt(selectedTemplate);
	TemplateFile* tmp = new TemplateFile;
	tmp->setTemplateRowId(templ->getRowId());
	tmp->setFullName(fileName);
	tmp->setPojoVersion(1);
	tmp->setValid(true);
	for(TemplateFile* file : templ->templateFiles){
		if(file->getFullName() ==tmp->getFullName()) {
			return;
		}
	}

	templ->templateFiles.push_back(tmp);
}

void FrmTemplDef::removeFile(rhea::GridCell* cell, std::string fileName) {
	Template* templ = templates.getValueAt(selectedTemplate);
	TemplateFile* myFile = templ->templateFiles.at(cell->getRow());
	if (myFile->getPojoId()>0){
		rhea::DbResultSet* res =  dbConn->execQuery("Update TemplateFiles set valid = 0 where templRowId = " + std::to_string(templ->getRowId())
							   + "  and path = '" + myFile->getPath()
						       + "' and name = '" + myFile->getName()
						       + "' and rowId = " + std::to_string(myFile->getRowId()) +";");
		delete res;
	}

	delete (templ->templateFiles.at(cell->getRow()));
	templ->templateFiles.erase(templ->templateFiles.begin()+cell->getRow());
}

void FrmTemplDef::btnSaveColumnsClick (){
	Template* templ = templates.getValueAt(selectedTemplate);

    for (int i = 0;i < templateDef->getRows();i++){
    	Column* col = templ->columns.at(i);
    	if ( col->getPojoId() < 1 ) {
    		rhea::DbResultSet* resCol = dbConn->execQuery("SELECT COALESCE(MAX(columnId)+1,1) from TemplateColumns;");
    		col->setPojoId(resCol->getField(0)->getAsLong());
    		col->setPojoVersion(1);
    		delete resCol;
    	}
    	col->setPojoName(std::to_string(col->getPojoId()));

    	col->label = dynamic_cast<Gtk::TextView*>(templateDef->getWidget(0,i))->get_buffer()->get_text();
    	std::string strTmp = dynamic_cast<Gtk::TextView*>(templateDef->getWidget(1,i))->get_buffer()->get_text();
    	global->replaceAll(strTmp, " ","");
    	global->replaceAll(strTmp, "\t","");
    	global->replaceAll(strTmp, "\n","");
    	global->replaceAll(strTmp, "\r","");
    	global->replaceAll(strTmp, "\f","");
    	global->replaceAll(strTmp, "\v","");
    	dynamic_cast<Gtk::TextView*>(templateDef->getWidget(1,i))->get_buffer()->set_text(strTmp);
    	col->libreCode = strTmp;
    	col->order = std::stoi(dynamic_cast<Gtk::TextView*>(templateDef->getWidget(2,i))->get_buffer()->get_text());
    	int tmp = dynamic_cast<rhea::ComboBox<StringColumns>*>(templateDef->getWidget(3,i))->get_active_row_number();
    	col->type = tmp == 0 ? ColumnType::String :
    		        tmp == 3 ? ColumnType::Int :
    		        tmp == 2 ? ColumnType::Double :
    		        tmp == 1 ? ColumnType::Date :
    		        tmp == 4 ? ColumnType::Index: ColumnType::String;
        tmp = dynamic_cast<rhea::ComboBox<StringColumns>*>(templateDef->getWidget(4,i))->get_active_row_number();
        col->objectType = tmp == 0 ? ColumnObject::TextView :
    				  tmp == 2 ? ColumnObject::Combo :
    				  tmp == 1 ? ColumnObject::ChkBox :
    				  tmp == 3 ? ColumnObject::Options :
    				  tmp == 4 ? ColumnObject::PckpDT :
    				  tmp == 5 ? ColumnObject::Grid	  :ColumnObject::TextView;
        if (col->type == ColumnType::Index){
        	col->objectType = ColumnObject::TextView;
        	dynamic_cast<rhea::ComboBox<StringColumns>*>(templateDef->getWidget(4,i))->set_active(0);
        }
    }


	for (Column* col : templ->columns){
		rhea::DbParameters parCol = rhea::DbParameters();
		parCol.add(col->getPojoId());
		parCol.add(templ->getRowId());
		parCol.add(col->getPojoVersion());
		parCol.add(col->getUUID());
		parCol.add(col->label);
		parCol.add(col->libreCode);
		switch (col->objectType){
			case ColumnObject::TextView :
				parCol.add("TextView");
				break;
			case ColumnObject::Combo:
				parCol.add("Combo");
				break;
			case ColumnObject::ChkBox:
				parCol.add("ChkBox");
		    	break;
			case ColumnObject::Options:
				parCol.add("Options");
				break;
			case ColumnObject::PckpDT:
				parCol.add("PickupDt");
				break;
			case ColumnObject::Grid:
				parCol.add("Grid");
				break;
			default:
				parCol.add("TextView");

		}
		switch (col->type){
			case ColumnType::String:
				parCol.add("String");
				break;
			case ColumnType::Date:
				parCol.add("Date");
				break;
			case ColumnType::Double:
			    parCol.add("Double");
			    break;
			case ColumnType::Int:
				parCol.add("Int");
				break;
			case ColumnType::Index :
				parCol.add("Index");
				break;
			default:
				parCol.add("String");
		}

		parCol.add(col->order);
		std::string optStr = global->setOptions(col->options,global->optionsDelimiter);
		optStr = optStr == "NULL" ? "" : optStr ;
		parCol.add(optStr);
		parCol.add(col->isValid() ? 1 : 0 );
		parCol.add(col->validTill->getAsISO8601());

		if (col->getRowId()>0){
			std::string sql = "SELECT 1 FROM TemplateColumns WHERE columnId = ? "
			        "AND templRowId = ? "
			        "AND version = ? "
			        "AND uuid = ? "
			        "AND label = ? "
			        "AND libreCode = ? "
			        "AND object = ? "
			        "AND type = ? "
		    		"AND orderWeight = ? "
		    		"AND options = ? "
		    	    "AND valid = ? "
		    	    "AND validTill = ?; ";

			rhea::DbResultSet* resCol = dbConn->execQuery(sql,parCol.get());


			if (resCol->rowCount() == 1 && resCol->getField(0)->getAsInteger() == 1 ) {
				continue;
			} else {
				rhea::DbResultSet* upd =  dbConn->execQuery("Update TemplateColumns set valid = 0 where rowId = " + std::to_string(col->getRowId())+";");
				delete upd;
			}
		}

		parCol.add(col->getPojoVersion()+1);

	    rhea::DbResultSet* resCol =  dbConn->execQuery("Insert into TemplateColumns ("
	    		" columnId,"
		        " templRowId,"
		        " version,"
		        " uuid,"
		        " label,"
		        " libreCode,"
		        " object,"
		        " type,"
	    		" orderWeight,"
	    		" options,"
	    	    " valid,"
	    	    " validTill ) VALUES (?,?,?,?,?,?,?,?,?,?,?,?) RETURNING rowId;",parCol.get());
	    if (resCol->rowCount()>0){
	    	col->setRowId(resCol->getField("rowId")->getAsLong());
	    }
	    delete resCol;
	}

	for(TemplateFile* file : templ->templateFiles) {
		std::cout << "mam file " << file->getPojoId() << std::endl;
		if (file->getPojoId()<1){
			rhea::DbResultSet* resFID = dbConn->execQuery("SELECT COALESCE(MAX(fileId)+1,1) from TemplateFiles;");
			file->setPojoId(resFID->getField(0)->getAsLong());
			delete resFID;
			file->setPojoVersion(1);

			rhea::DbParameters parFile = rhea::DbParameters();
			parFile.add(file->getPojoId());
			parFile.add(templ->getRowId());
			parFile.add(file->getPojoVersion());
			parFile.add(file->getUUID());
			parFile.add(file->getPath());
			parFile.add(file->getName());
			parFile.add(file->isValid() ? 1 : 0 );
			parFile.add(file->validTill->getAsISO8601());

		    rhea::DbResultSet* resFile =  dbConn->execQuery("Insert into TemplateFiles ("
		    		" fileId,"
			        " templRowId,"
			        " version,"
			        " uuid,"
			        " path,"
			        " name,"
		    	    " valid,"
		    	    " validTill ) VALUES (?,?,?,?,?,?,?,?) RETURNING rowId;",parFile.get());
		    if (resFile->rowCount()>0){
		    	file->setRowId(resFile->getField("rowId")->getAsLong());
		    }
		    delete resFile;
		}
	}
}

void FrmTemplDef::getOtherTemplates(std::vector<std::string*> &alternatives) {
	alternatives.clear();
	for(auto& item : templates.getMap()){
		bool found = false;

		if (item.second->getRowId() ==templates.getValueAt(selectedTemplate)->getRowId()) {continue;}

		std::string strCols = "";
		for (Column* col: item.second->columns ){
			strCols += std::to_string(col->getPojoId()) + global->gridTemplateColDelimiter;
			strCols += col->label + global->gridTemplateColDelimiter;
		}
		strCols = strCols.substr(0,strCols.size()-global->gridTemplateColDelimiter.size());
		std::string* alternative = new std::string( global->gridTemplateDelimiter
				                + item.second->getName()
								+ global->gridTemplateDelimiter
								+ std::to_string(item.second->getPojoId())
								+ global->gridTemplateDelimiter
								+ strCols);
		alternatives.push_back(alternative);
	}

}
void FrmTemplDef::btnOptionsOnClick(Gtk::Button *btn, int row){

	Gtk::Root *root = btn->get_root();
	AppWindow *window = dynamic_cast<AppWindow*>(root);
	int Index = dynamic_cast<rhea::ComboBox<StringColumns>*>(templateDef->getWidget(3,row))->get_active_row_number();
	int pregl = dynamic_cast<rhea::ComboBox<StringColumns>*>(templateDef->getWidget(4,row))->get_active_row_number();
	if (Index == 4){
		DialogNumberFunct* donf = new DialogNumberFunct();
		donf->set_transient_for(*window);
		donf->set_modal();
		donf->fill(&templates.getValueAt(selectedTemplate)->columns.at(row)->options,btn);
		donf->show();
	} else if (pregl == 5){
		DialogOptionsGrid* diop = new DialogOptionsGrid();
		diop->set_transient_for(*window);
		diop->set_modal();
		std::vector<std::string*> alternatives = {};
		getOtherTemplates(alternatives);
		diop->fill(alternatives,&templates.getValueAt(selectedTemplate)->columns.at(row)->options,btn);
		diop->show();

	}else {
		DialogOptions* diop = new DialogOptions();
		diop->set_transient_for(*window);
		diop->set_modal();
		std::vector<std::string*> alternatives = {};
		getOtherTemplates(alternatives);
		diop->fill(alternatives,&templates.getValueAt(selectedTemplate)->columns.at(row)->options,btn);
		diop->show();
	}
}

void FrmTemplDef::btnDelColumnOnClick(Gtk::Button *btn, int row){
	Template* templ = templates.getValueAt(selectedTemplate);
	Column* col = templ->columns.at(row);
	if (col->getRowId()>0){
		rhea::DbResultSet* res =  dbConn->execQuery("Update TemplateColumns set valid = 0 where rowId = " + std::to_string(col->getRowId()));
		delete res;
	}
	delete (templ->columns.at(row));
	templ->columns.erase(templ->columns.begin()+row);
	templateDef->setRows(templates.getValueAt(selectedTemplate)->columns.size());
	templateDef->refresh();
}
