/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#include "GridDataSection.h"
#include <iostream>

GridDataSection::GridDataSection() : rhea::Grid() {

	connectWidget(&GridDataSection::dataListing, this);
}

GridDataSection::~GridDataSection() {

}

void GridDataSection::btnAddOnClick () {

	std::vector<std::string> cols = {};
	int key = data.getKey(data.size()-1);
	std::string col="";
	for (int i = 0; i < getColumns()-1;i++){
		cols.push_back(col);
	}
	data.add(key+1,cols);
	setRows(data.size());
	refresh();
}

void GridDataSection::btnDeleteOnClick(Gtk::Button *btn, int row) {

	if (onDeleteCallback) {
		onDeleteCallback(btn,row);
	}
	data.removeAt(row);
	setRows(data.size());
	refresh();
}

Gtk::Widget* GridDataSection::dataListing (int col,int row) {

	if ((col< getColumns()-1) && (col <= data.getValueAt(row).size()-1)) {

		Gtk::TextView* tv = new Gtk::TextView();
		tv->set_size_request(90,10);
		tv->get_buffer()->set_text(data.getValueAt(row).at(col));
	   return tv;
	} else if (col == getColumns()-1){
	   Gtk::Button* btnDelete = new Gtk::Button(lang->getText("txtTemplDef_btnDelete"));
	   btnDelete->set_size_request(45,10);
	   btnDelete->signal_clicked().connect(sigc::bind(sigc::mem_fun(*this, &GridDataSection::btnDeleteOnClick),btnDelete,row));
	   return btnDelete;
    }
	Gtk::TextView* tv = new Gtk::TextView();
	tv->set_size_request(90,25);


   return tv;

}

void GridDataSection::fillHeaders(std::vector<std::string> cols) {

	header.clear();
    for (std::string txt : cols) {
    	Gtk::Label* lblHdr = new Gtk::Label();
    	lblHdr->add_css_class("rhea_gridHeaderLbl");
    	lblHdr->set_text(txt);
    	lblHdr->set_name("filesHeaderLbl");
    	lblHdr->set_hexpand(true);
    	header.push_back(lblHdr);
    }
    Gtk::Button* btnAdd = new Gtk::Button();
	btnAdd->set_label("Dodaj");
	btnAdd->signal_clicked().connect(sigc::mem_fun(*this, &GridDataSection::btnAddOnClick));
	header.push_back(btnAdd);
	setHeaders(&header,0);
}

void GridDataSection::setData(std::string strValues) {

	data.clear();
	std::vector<std::string> rows = {};
	global->parseOptions(strValues,global->gridDataRowDelimiter,rows);
	int i = 0;
	for (std::string & str : rows) {
		std::vector<std::string> cols = {};
		global->parseOptions(str,global->gridDataColDelimiter,cols);
		data.add(i,cols);
		i++;
	}

	setRows(data.size());
	refresh();

}

void GridDataSection::addRow() {

}

void GridDataSection::reset(std::vector<std::string>& options) {
	data.clear();
	int i = 0;
	for (std::string val : options) {

		if (val.starts_with(global->gridColumnDelimiter)){
			val = val.substr(global->gridColumnDelimiter.size());
			std::vector<std::string> cols = {};
			global->parseOptions(val,global->gridColumnDelimiter,cols);
			fillHeaders(cols);
		}else{
		    std::vector<std::string> firstCol = {};
			firstCol.push_back(val);
			data.add(i,firstCol);
			i++;
		}
	}

	setRows(i);
	refresh();
}

std::string GridDataSection::getAllData() {

	std::string myData = "";
	for (int i = 0; i < getRows(); i++){
		myData += global->gridDataRowDelimiter;
		std::string columnsData = "";
		for (int j = 0; j < getColumns()-1; j++) {
			Gtk::TextView* txt = dynamic_cast<Gtk::TextView*>(getWidget(j,i));
			columnsData += global->gridDataColDelimiter +txt->get_buffer()->get_text();
		}
		columnsData = columnsData.substr(global->gridDataColDelimiter.size());
		myData += columnsData;
	}
	myData = myData.substr(global->gridDataRowDelimiter.size());

	return myData;
}
