/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#ifndef GUI_GRIDDATASECTION_H_
#define GUI_GRIDDATASECTION_H_

#include "rhea/Grid/Grid.h"
#include "rhea/ListedMap/ListedMap.h"
#include "po/DataSection.h"
#include "gtkmm.h"
#include "../main/Global.h"
#include "main/LanguagePack.h"

class GridDataSection : public rhea::Grid {

private:
	Global* global = Global::getInstance();
	LanguagePack* lang = LanguagePack::getInstance();
	std::function<void (Gtk::Button *btn, int row)> onDeleteCallback = {nullptr};
	std::function<void (rhea::GridCell*, std::string)> onAddCallback = {nullptr};
	std::vector<Gtk::Widget*> header = {};
	rhea::ListedMap<int,std::vector<std::string>> data;
	DataSection* ds = new DataSection();

	void btnAddOnClick ();
	void btnDeleteOnClick(Gtk::Button *btn, int row);
	Gtk::Widget* dataListing (int col,int row);
    void fillHeaders(std::vector<std::string> cols);

public:
	GridDataSection();
	~GridDataSection();
	void setData(std::string data);
	void addRow();
	void reset(std::vector<std::string> &options);
	std::string getAllData();
	template<typename A, typename B> void connectOnDeleteRow (A funcPtr, B objPtr) {
			onDeleteCallback = (std::bind(funcPtr, objPtr, std::placeholders::_1, std::placeholders::_2));}
	template<typename A, typename B> void connectOnAddRow (A funcPtr, B objPtr) {
		onAddCallback = (std::bind(funcPtr, objPtr, std::placeholders::_1, std::placeholders::_2));}
};

#endif /* GUI_GRIDDATASECTION_H_ */
