/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#include "WndExport.h"

template<class T>
WndExport<T>::WndExport() {
	btnExport->set_label(lang->getText("txtExport_btnExport"));
	btnExport->signal_clicked().connect(sigc::mem_fun(*this, &WndExport::onExportClick));

	btnFile->set_label(lang->getText("txtExport_btnFile"));
	btnFile->signal_clicked().connect(sigc::mem_fun(*this, &WndExport::onFileClick));

    lblFile->set_size_request(130,40);
	fileBox->append(*lblFile);
	fileBox->append(*btnFile);


	btnExport->set_sensitive(false);
	btnExport->set_halign(Gtk::Align::END);
	bottomBox->set_halign(Gtk::Align::FILL);
	lblMessage->set_halign(Gtk::Align::FILL);
	lblMessage->set_hexpand(true);
	bottomBox->set_hexpand(true);
	bottomBox->append(*lblMessage);
	bottomBox->append(*btnExport);

	mainBox->append(*fileBox);
	mainBox->append(*fieldGrid);
	mainBox->append(*bottomBox);

	this->set_child(*mainBox);
	this->set_size_request(600,450);

}

template<class T>
WndExport<T>::~WndExport() {
	// TODO Auto-generated destructor stub
}

template<class T>
void WndExport<T>::onFileClick () {
	  auto dialog = new Gtk::FileChooserDialog(lang->getText("txtExport_selectFile"), Gtk::FileChooser::Action::SAVE);
	  Gtk::Window *window = dynamic_cast<Gtk::Window*>(this->get_root());
	  dialog->set_transient_for(*window);
	  dialog->set_modal(true);
	  dialog->signal_response().connect(sigc::bind( sigc::mem_fun(*this, &WndExport::onDialogResponse), dialog));

	  //Add response buttons to the dialog:
	  dialog->add_button(lang->getText("txtExport_cancel"), Gtk::ResponseType::CANCEL);
	  dialog->add_button(lang->getText("txtExport_save"), Gtk::ResponseType::OK);

	  //Add filters, so that only certain file types can be selected:

	  auto filterText = Gtk::FileFilter::create();
	  filterText->set_name("CSV files");
	  filterText->add_mime_type("text/csv");
	  dialog->add_filter(filterText);

	  dialog->set_current_name("grunfExport.csv");
	  //Show the dialog and wait for a user response:
	  dialog->show();
}


template<class T>
void WndExport<T>::onDialogResponse(int response_id, Gtk::FileChooserDialog* dialog) {
	  switch (response_id) {
	    case Gtk::ResponseType::OK: {
	    	file->setFullName(dialog->get_file()->get_path());
	    	lblFile->set_text(file->getFullName());
	    	btnExport->set_sensitive(true);
	      break;

	    } case Gtk::ResponseType::CANCEL: {
	      break;

	    } default: {

	    }

	  }

	  delete dialog;
	  if (response_id == Gtk::ResponseType::CANCEL) {
		  return;
	  }

}
