/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#include "WndImport.h"
#include <iostream>

template<class T>
WndImport<T>::WndImport() :  Gtk::Window() {

	btnFile->set_label(lang->getText("txtImport_File"));
	btnFile->signal_clicked().connect(sigc::mem_fun(*this, &WndImport::onFileClick));

	btnImport->set_label(lang->getText("txtImport_import"));
	btnImport->signal_clicked().connect(sigc::mem_fun(*this, &WndImport::onImportClick));

	lblFile->set_size_request(130,40);

	fileBox->append(*lblFile);
	fileBox->append(*btnFile);

	bottomBox->set_halign(Gtk::Align::END);
	btnImport->set_sensitive(false);
	bottomBox->append(*btnImport);

	mainBox->append(*fileBox);
	mainBox->append(*fieldGrid);
	mainBox->append(*bottomBox);

	this->set_child(*mainBox);
	fieldGrid->connectString(&WndImport::getListing, this);
	this->set_size_request(500,300);
}

template<class T>
void WndImport<T>::onFileClick () {
	  auto dialog = new Gtk::FileChooserDialog(lang->getText("txtExport_selectFile"), Gtk::FileChooser::Action::OPEN);
	  Gtk::Window *window = dynamic_cast<Gtk::Window*>(this->get_root());
	  dialog->set_transient_for(*window);
	  dialog->set_modal(true);
	  dialog->signal_response().connect(sigc::bind( sigc::mem_fun(*this, &WndImport::onDialogResponse), dialog));

	  //Add response buttons to the dialog:
	  dialog->add_button(lang->getText("txtExport_cancel"), Gtk::ResponseType::CANCEL);
	  dialog->add_button(lang->getText("txtImport_Open"), Gtk::ResponseType::OK);

	  //Add filters, so that only certain file types can be selected:

	  auto filterText = Gtk::FileFilter::create();
	  filterText->set_name("CSV files");
	  filterText->add_mime_type("text/csv");
	  dialog->add_filter(filterText);

	  //Show the dialog and wait for a user response:
	  dialog->show();
}

template<class T>
void WndImport<T>::onDialogResponse(int response_id, Gtk::FileChooserDialog* dialog) {
	switch (response_id) {
		case Gtk::ResponseType::OK: {
	    	file->setFullName(dialog->get_file()->get_path());
	    	lblFile->set_text(file->getFullName());
	    	btnImport->set_sensitive(true);
	    	break;

	    } case Gtk::ResponseType::CANCEL: {
	      break;

	    } default: {

	    }

	}

	delete dialog;
	if (response_id == Gtk::ResponseType::OK) {
		readData();
	}
}

template<class T>
void WndImport<T>::readData(){
	std::string podatki = "";
	podatki = reinterpret_cast<char *>(file->readFile());
	std::vector<std::string> line = {};
	size_t pos = 0;
	for(size_t i = 0; i < podatki.size(); i++){
		if (podatki.at(i) == '\n'){
			line.push_back(podatki.substr(pos,i-pos));
			pos=i+1;
		}
	}
	for(std::string& str :line){
		if (!str.ends_with(",")){
			str+=",";
		}
	}
	int colona =0;

	for(std::string str :line){
		pos = 0;
		colona++;
		std::vector<std::string> vrednosti = {};
		for(size_t i = 0; i<str.size();i++){
			if(str.at(i)==',') {
				std::string vrstica = "";
				if (str.substr(pos,i-pos).length()!=0){
					vrstica = str.substr(pos,i-pos);
				}
				vrednosti.push_back(vrstica);
				pos=i+1;
			}
		}
		if (!vrednosti.empty()) {
			myData.add(rhea::Pair{colona,vrednosti});
		}
	}

	std::cout << "pridem do tle 2" <<std::endl;
	for(int i = 0;i < myData.size();i++ ){
		if (importColumns < myData.getValueAt(i).size()){
			importColumns = myData.getValueAt(i).size();
		}
	}
	std::cout << "pridem do tle 3" <<std::endl;
	for(int i = 0; i < importColumns; i++ ){
		rhea::ComboBox<StringColumns> *tv = new rhea::ComboBox<StringColumns>();
		tv->set_size_request(100, 35);
		tv->show();
		tv->addList(&objectList);
		if ( i < fields->myTemplate->columns.size() ){
			tv->set_active(i+1);
		} else{
			tv->set_active(0);
		}
		header.push_back(tv);
	}
    fieldGrid->setHeaders(&header, 0);

	fieldGrid->setRows(myData.size());
	fieldGrid->connectString(&WndImport::getListing, this);
	fieldGrid->refresh();

}


template<class T>
WndImport<T>::~WndImport() {
	// TODO Auto-generated destructor stub
}
