/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#ifndef GUI_WNDIMPORT_H_
#define GUI_WNDIMPORT_H_

#include "string.h"
#include "gtkmm.h"
#include "../rhea/File/File.h"
#include "../rhea/ListedMap/ListedMap.h"
#include "../rhea/Grid/Grid.h"
#include "../rhea/ComboBox/ComboBox.h"
#include "cbColumns/StringColumns.h"
#include "../po/Template.h"
#include "FrameBase.h"
#include "../rhea/SqlLite/DbConn.h"
#include "main/LanguagePack.h"

template<class T> class WndImport : public Gtk::Window {
private:


	void onFileClick();
	virtual void onImportClick() = 0;
    void onDialogResponse(int response_id, Gtk::FileChooserDialog* dialog);

protected:
	LanguagePack* lang = LanguagePack::getInstance();
    rhea::ListedMap<std::string, T*>* map = {};
    rhea::ListedMap <int,std::vector<std::string>> myData= {};
    rhea::DbConn* dbConn = rhea::DbConn::getInstance();
	rhea::File* file = new rhea::File();
	Gtk::Box*   mainBox    = new Gtk::Box(Gtk::Orientation::VERTICAL);
	Gtk::Box*   fileBox    = new Gtk::Box(Gtk::Orientation::HORIZONTAL);
	rhea::Grid* fieldGrid  = new rhea::Grid();
	Gtk::Box*   bottomBox  = new Gtk::Box(Gtk::Orientation::HORIZONTAL);
	Gtk::Label* lblFile    = new Gtk::Label();
	FrameBase<T>* originGui;
	TemplateData* fields;
	StringColumns stringColumns;
	int importColumns = 0;
	Glib::RefPtr<Gtk::ListStore> objectList = Gtk::ListStore::create(stringColumns);
public:
	std::vector<Gtk::Widget*> header = {};
	Gtk::Button* btnFile    = new Gtk::Button("Add");
	Gtk::Button* btnImport  = new Gtk::Button("Import");
	WndImport();
	~WndImport();
    virtual void fill (T* myPo, rhea::ListedMap<std::string, T*>* myMap, FrameBase<T>* gui) = 0;
    void readData();
    virtual std::string getListing(int col,int row) = 0;
};

#include "WndImport.cpp"

#endif /* GUI_WNDIMPORT_H_ */
