/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#include "Global.h"
#include "iostream"
#include "po/Template.h"

rhea::Menu* Global::appMenu = {nullptr};

Global::Global(){};

Global* Global::getInstance() {

    if (!instance) {
        instance = new Global;
    }

    return instance;

}

bool Global::isNumber(const std::string& s) {
    std::string::const_iterator it = s.begin();
    while (it != s.end() && std::isdigit(*it)) ++it;
    return !s.empty() && it == s.end();
}


void Global::deleteMenuItems(){
	rhea::MenuItem* work = appMenu->getItem("work");
	if (!work) {return; }
	for (int i = work->children.size()-1;i >= 0 ;i--) {
		appMenu->deleteItem("template_"+std::to_string(i));
	}
}

void Global::createMenuItems(){
	deleteMenuItems();
	rhea::DbResultSet* res = dbConn->execQuery("SELECT * FROM Templates WHERE valid = 1 and showInMenu = 1 ORDER BY templId;");
	if (res->rowCount()>0) {
		while(!res->isEof()) {
			rhea::MenuItem* item = appMenu->createItem("template_" + std::to_string(res->getRow()),
	            									   res->getField("menuLabel")->getAsString(),
													   "win.data_template", res->getField("templId")->getAsString() + " " + res->getField("menuLabel")->getAsString());
			res->moveNext();
	      	appMenu->addChild(item,appMenu->getItem("work"));
	    }
	}
}

void Global::createShortCuts() {

	for(auto sc : shortcuts) {
		shortcutController->remove_shortcut(sc);
	}
	shortcuts.clear();

	rhea::DbResultSet* res = dbConn->execQuery("SELECT * FROM Templates WHERE valid = 1 ORDER BY templId;");

	if (res->rowCount()>0) {
		while(!res->isEof()) {
			std::string shcut = res->getField("shortcut")->getAsString();
			int kmask =std::stoi(shcut.substr(0,1));
			shcut = shcut.substr(1);
			if (shcut.starts_with(" ")){
				shcut = shcut.substr(1);
			}
			if (KeyMasks::Shift & kmask){
				shcut = "<Shift>"+shcut;
			}
			if (KeyMasks::Ctrl & kmask){
				shcut = "<Control>"+shcut;
			}
			if (KeyMasks::Alt & kmask){
				shcut = "<Alt>"+shcut;
			}
		    Glib::RefPtr<Gtk::ShortcutTrigger> st = Gtk::ShortcutTrigger::parse_string(shcut);
		    Glib::RefPtr<Gtk::ShortcutAction>  sa = Gtk::NamedAction::create("win.data_template");
		    std::string value = res->getField("templId")->getAsString() + " " + res->getField("menuLabel")->getAsString();
		    Glib::Variant<std::string> val = Glib::Variant<std::string>(g_variant_new_string(value.c_str()));
		    Glib::RefPtr<Gtk::Shortcut> sc =  Gtk::Shortcut::create(st,sa);
		    sc->set_arguments(val);
		    shortcutController->add_shortcut(sc);
		    shortcuts.push_back(sc);
			res->moveNext();
		}
	}

}

void Global::parseOptions(std::string in, std::string delimiter, std::vector<std::string> &out) {
	out.clear();
	size_t pos = 0;
	std::string token;
	while ((pos = in.find(delimiter)) != std::string::npos) {
	    token = in.substr(0, pos);
	    out.push_back(token);
	    in.erase(0, pos + delimiter.length());
	}
	out.push_back(in);
}

std::string Global::setOptions( std::vector<std::string> &values,std::string delimiter) {
	std::string optStr = "";
	for(std::string &text : values) {
		optStr += text + delimiter;
	}
	if (!values.empty()){
		optStr = optStr.substr(0,optStr.size()-delimiter.size());
	}
	return optStr;
}

void Global::replaceAll(std::string& str, const std::string from, const std::string to) {
    if(from.empty())
        return;
    size_t start_pos = 0;
    while((start_pos = str.find(from, start_pos)) != std::string::npos) {
        str.replace(start_pos, from.length(), to);
        start_pos += to.length(); // In case 'to' contains 'from', like replacing 'x' with 'yx'
    }
}
