/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#ifndef MAIN_GLOBAL_H_
#define MAIN_GLOBAL_H_

#include <memory>
#include "../rhea/SqlLite/DbConn.h"
#include "../rhea/Menu/Menu.h"
#include "main.h"

class AppWindow;

class Global {
private:
	inline static Global* instance = {nullptr};
	Global();
public:
	static Global* getInstance();
	std::string name = "";
	rhea::DbConn* dbConn = rhea::DbConn::getInstance();
	AppWindow* appWindow;
    Glib::RefPtr<Gtk::ShortcutController> shortcutController = Gtk::ShortcutController::create();
	std::vector<Glib::RefPtr<Gtk::Shortcut>> shortcuts = {};
	inline static std::string indexPreDelimiter        = "<!¤#>RHEA_OPTION_INDEX_PREFIX:";
	inline static std::string indexStepDelimiter       = "<!¤#>RHEA_OPTION_INDEX_STEP:";
	inline static std::string indexPostDelimiter       = "<!¤#>RHEA_OPTION_INDEX_POSTFIX:";
	inline static std::string gridTemplateDelimiter    = "<!¤#>RHEA_OPTION_GRID_CANDIDATE<!¤#>";
	inline static std::string gridTemplateUsed         = "<!¤#>RHEA_OPTION_GRID_TEMPLATE:";
	inline static std::string gridTemplateColDelimiter = "<!¤#>RHEA_OPTION_GRID_TEMPLATE_COL_DELIMITER<!¤#>";
	inline static std::string gridDataRowDelimiter 	   = "<!¤#>RHEA_DATA_GRID_ROW_DELIMITER<!¤#>";
	inline static std::string gridDataColDelimiter 	   = "<!¤#>RHEA_DATA_GRID_COL_DELIMITER<!¤#>";
	inline static std::string gridColumnDelimiter 	   = "<!¤#>RHEA_OPTION_GRID_COL_DELIMITER<!¤#>";
	inline static std::string gridSelectedCol          = "<!¤#>RHEA_OPTION_GRID_COL_SELECTED:";
	inline static std::string gridRowDelimiter         = "<!¤#>RHEA_OPTION_GRID_ROW_DELIMITER<!¤#>";
	inline static std::string filterDelimiter          = "<!¤#>RHEA_OPTION_FILTER_DELIMITER<!¤#>";
	inline static std::string optionsDelimiter         = "<!¤#>RHEA_OPTION_DELIMITER<!¤#>";
	static rhea::Menu* appMenu;
    static bool isNumber(const std::string& s);
    void deleteMenuItems();
	void createMenuItems();
	void createShortCuts();
	void parseOptions(std::string in, std::string delimiter, std::vector<std::string> &out);
	std::string setOptions(std::vector<std::string> &values, std::string delimiter);
	void replaceAll(std::string& str, const std::string from, const std::string to);

};

#endif /* MAIN_GLOBAL_H_ */
