/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#include "GrunfApp.h"

#include <iostream>
#include "main.h"

GrunfApp::GrunfApp() : Gtk::Application("si.rhea.grunf") {

	Glib::set_application_name("Grunf");
}

void GrunfApp::create_window() {
    auto win = new AppWindow();

    //Make sure that the application runs for as long this window is still open:
    add_window(*win);
    win->signal_hide().connect(sigc::bind(sigc::mem_fun(*this, &GrunfApp::on_window_hide), win));

    win->set_show_menubar();
    win->show();
}

void GrunfApp::on_window_hide(Gtk::Window *window) {
    delete window;
}

void GrunfApp::on_menu_file_quit() {

    quit(); // Not really necessary, when Gtk::Widget::hide() is called.

    std::vector<Gtk::Window*> windows = get_windows();
    if (windows.size() > 0) {
        for (unsigned long i = windows.size();i>0;i--){
            windows[i]->hide(); // In this simple case, we know there is only one window.
        }
    }

}


void GrunfApp::on_startup() {
    Gtk::Application::on_startup();

    auto action = add_action("quit", sigc::mem_fun(*this, &GrunfApp::on_menu_file_quit));

    appMenu->nameXY = "main menu";
    rhea::MenuItem* file = appMenu->createItem("file",lang->getText("txtMainMenu_FileMenu"),"");
    rhea::MenuItem* templates = appMenu->createItem("templates",lang->getText("txtMainMenu_FileStructures"),"win.templates");
    rhea::MenuItem* settings = appMenu->createItem("setings",lang->getText("txtMainMenu_FileSettings"),"win.settings");
    rhea::MenuItem* quit = appMenu->createItem("quit",lang->getText("txtMainMenu_FileQuit"),"app.quit");
    rhea::MenuItem* work = appMenu->createItem("work",lang->getText("txtMainMenu_WorkMenu"),"");
    rhea::MenuItem* help = appMenu->createItem("help",lang->getText("txtMainMenu_HelpMenu"),"");
    rhea::MenuItem* content = appMenu->createItem("content",lang->getText("txtMainMenu_HelpInstr"),"win.help");
    rhea::MenuItem* about = appMenu->createItem("about",lang->getText("txtMainMenu_HelpAbout"),"win.about");

    appMenu->addChild(templates,file);
    appMenu->addChild(settings,file);
    appMenu->addChild(quit,file);
    appMenu->addChild(content,help);
    appMenu->addChild(about,help);
    appMenu->addToMenu(file);
    appMenu->addToMenu(work);
    appMenu->addToMenu(help);

    set_accel_for_action("app.quit","<Control><Shift>q");


//    appMenu->setAccelator(quit,"<Alt>Z");
    set_menubar(appMenu->getMenu());
    global->appMenu = appMenu;
    global->createMenuItems();
}

void GrunfApp::on_activate() {
    create_window();
}

Glib::RefPtr<GrunfApp> GrunfApp::create() {
    return Glib::make_refptr_for_instance<GrunfApp>(new GrunfApp);
}

GrunfApp::~GrunfApp(){

}
