/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#include <main/TestDB.h>
#include <iostream>
#include "../rhea/SqlLite/DbResultSet.h"
#include "../rhea/SqlLite/DbParameters.h"

TestDB::TestDB(sqlite3 *db) {
 TestDB::db = db;
}

TestDB::~TestDB() {

}

void TestDB::populateTemplates() {
    Template* tmp1 = new Template();
    Column* clm11 = new Column();
    clm11->setPojoId(1);
    clm11->label = "Ime:";
    clm11->libreCode = "#IME#";
    clm11->objectType = ColumnObject::TextView;
    clm11->type = ColumnType::String;
    clm11->order = 1;

    Column* clm12 = new Column();
    clm12->setPojoId(2);
    clm12->label = "Priimek:";
    clm12->libreCode = "#PRIIMEK#";
    clm12->objectType = ColumnObject::TextView;
    clm12->type = ColumnType::String;
    clm12->order = 2;

    tmp1->columns.push_back(clm11);
    tmp1->columns.push_back(clm12);
    tmp1->setMenuVisibility(true);
    tmp1->setName("Prostovoljec");
    tmp1->setShortcut(4,"p");
    tmp1->setPojoId(1);
    tmp1->setPojoName("1");
    tmp1->setPojoVersion(1);
    tmp1->setValid(true);

    insertTemplate(tmp1);

    TemplateData* td1 = new TemplateData();
    td1->setPojoId(1);
    td1->setPojoVersion(1);
    td1->setValid(true);
    td1->myTemplate = tmp1;

    TemplateValues* tval11 = new TemplateValues();
    tval11->setColumnRowId(clm11->getRowId());
    tval11->setPojoId(1);
    tval11->setPojoVersion(1);
    tval11->setValue("Marko");

    TemplateValues* tval12 = new TemplateValues();
    tval12->setColumnRowId(clm12->getRowId());
    tval12->setPojoId(2);
    tval12->setPojoVersion(1);
    tval12->setValue("Klinar");

    td1->values.push_back(tval11);
    td1->values.push_back(tval12);
    insertData(td1);

    delete clm11;
    delete clm12;
    delete tval11;
    delete tval12;
	delete td1;
	delete tmp1;

    Template* tmp2 = new Template();
    Column* clm21 = new Column();
    clm21->setPojoId(3);
    clm21->label = "Ime:";
    clm21->libreCode = "#IME#";
    clm21->objectType = ColumnObject::TextView;
    clm21->type = ColumnType::String;
    clm21->order = 1;

    Column* clm22 = new Column();
    clm22->setPojoId(4);
    clm22->label = "Priimek:";
    clm22->libreCode = "#PRIIMEK#";
    clm22->objectType = ColumnObject::TextView;
    clm22->type = ColumnType::String;
    clm22->order = 2;

    Column* clm23 = new Column();
    clm23->setPojoId(5);
    clm23->label = "Članarina:";
    clm23->libreCode = "#CLANARINA#";
    clm23->objectType = ColumnObject::TextView;
    clm23->type = ColumnType::String;
    clm23->order = 3;

    tmp2->columns.push_back(clm21);
    tmp2->columns.push_back(clm22);
    tmp2->columns.push_back(clm23);
    tmp2->setMenuVisibility(true);
    tmp2->setName("Članarina");
    tmp2->setShortcut(4,"č");
    tmp2->setPojoId(2);
    tmp2->setPojoName("2");
    tmp2->setPojoVersion(1);
    tmp2->setValid(true);

    insertTemplate(tmp2);

    TemplateData* td2 = new TemplateData();
    td2->setPojoId(2);
    td2->setPojoVersion(1);
    td2->setValid(true);
    td2->myTemplate = tmp2;

    TemplateValues* tval21 = new TemplateValues();
    tval21->setColumnRowId(clm21->getRowId());
    tval21->setPojoId(3);
    tval21->setPojoVersion(1);
    tval21->setValue("Gorenjka");

    TemplateValues* tval22 = new TemplateValues();
    tval22->setColumnRowId(clm22->getRowId());
    tval22->setPojoId(4);
    tval22->setPojoVersion(1);
    tval22->setValue("Markušek");

    TemplateValues* tval23 = new TemplateValues();
    tval23->setColumnRowId(clm23->getRowId());
    tval23->setPojoId(5);
    tval23->setPojoVersion(1);
    tval23->setValue("54 €");

    td2->values.push_back(tval21);
    td2->values.push_back(tval22);
    td2->values.push_back(tval23);

    insertData(td2);

    delete clm21;
    delete clm22;
    delete clm23;
    delete tval21;
    delete tval22;
    delete tval23;
    delete td2;
    delete tmp2;

}

void TestDB::insertTemplate(Template* templ) {

    rhea::DbParameters params = rhea::DbParameters();
    params.add(templ->getPojoId());
    params.add(templ->getPojoVersion());
    params.add(templ->getName());
    params.add(templ->getLabel());
    params.add(templ->getMenuVisibility() ? 1 : 0 );
    params.add(templ->getShortCut());
    params.add(templ->isValid() ? 1 : 0 );
    params.add(templ->validTill->getAsISO8601());

    rhea::DbResultSet res =  rhea::DbResultSet(db,"Insert into Templates("
    " templId,"
    " version,"
    " name,"
    " menuLabel,"
    " showInMenu,"
    " shortcut,"
    " valid,"
    " validTill ) VALUES (?,?,?,?,?,?,?,?) RETURNING rowId;",params.get());

    if (res.rowCount()>0){
    	templ->setRowId(res.getField("rowId")->getAsLong());
    	for (Column* col : templ->columns){
    		rhea::DbParameters parCol = rhea::DbParameters();
    		parCol.add(col->getPojoId());
    		parCol.add(templ->getRowId());
    		parCol.add(col->getPojoVersion());
    		parCol.add(col->getUUID());
    		parCol.add(col->label);
    		parCol.add(col->libreCode);
    		switch (col->objectType){
    			case ColumnObject::TextView :
    				parCol.add("TextView");
    				break;
    			case ColumnObject::Combo:
    				parCol.add("Combo");
    				break;
    			case ColumnObject::ChkBox:
    				parCol.add("ChkBox");
    		    	break;
    			case ColumnObject::Options:
    				parCol.add("Options");
    				break;
    			case ColumnObject::PckpDT:
    				parCol.add("PickupDt");
    				break;
    			default:
    				parCol.add("TextView");

    		}
    		switch (col->type){
    			case ColumnType::String:
    				parCol.add("String");
    				break;
    			case ColumnType::Date:
    				parCol.add("Date");
    				break;
    			case ColumnType::Double:
    			    parCol.add("Double");
    			    break;
    			case ColumnType::Int:
    				parCol.add("Int");
    				break;
    			default:
    				parCol.add("String");
    		}
    		parCol.add(col->order);
    		parCol.add(col->isValid() ? 1 : 0 );
    		parCol.add(col->validTill->getAsISO8601());

    	    rhea::DbResultSet resCol =  rhea::DbResultSet(db,"Insert into TemplateColumns ("
    	    		" columnId,"
    		        " templRowId,"
    		        " version,"
    		        " uuid,"
    		        " label,"
    		        " libreCode,"
    		        " object,"
    		        " type,"
    		        " orderWeight,"
    	    	    " valid,"
    	    	    " validTill ) VALUES (?,?,?,?,?,?,?,?,?,?,?) RETURNING rowId;",parCol.get());
    	    if (resCol.rowCount()>0){
    	    	col->setRowId(resCol.getField("rowId")->getAsLong());
    	    }
    	}
    }
}

void TestDB::insertData (TemplateData* td){

	rhea::DbParameters params = rhea::DbParameters();
	params.add(td->getPojoId());
	params.add(td->getPojoVersion());
    params.add(td->myTemplate->getRowId());
	params.add(td->isValid() ? 1 : 0 );
	params.add(td->validTill->getAsISO8601());

	rhea::DbResultSet res =  rhea::DbResultSet(db,"Insert into TemplateData("
	    " dataId,"
	    " version,"
	    " templRowId,"
	    " valid,"
	    " validTill ) VALUES (?,?,?,?,?) RETURNING rowId;",params.get());

	if (res.rowCount()>0){
		int i = 0;
		td->setRowId(res.getField("rowId")->getAsLong());
		for (TemplateValues* tv : td->values ) {
			rhea::DbParameters parVal = rhea::DbParameters();
			parVal.add(tv->getPojoId());
			parVal.add(td->getRowId());
			parVal.add(tv->getPojoVersion());
			parVal.add(td->myTemplate->columns.at(i)->getRowId());
			parVal.add(tv->getValue());
			parVal.add(tv->isValid() ? 1 : 0 );
			parVal.add(tv->validTill->getAsISO8601());

			rhea::DbResultSet resVal =  rhea::DbResultSet(db,"Insert into TemplateValues ("
    	    		" valueId,"
    		        " dataRowId,"
    		        " version,"
    		        " columnRowId,"
    		        " value,"
    	    	    " valid,"
    	    	    " validTill ) VALUES (?,?,?,?,?,?,?) RETURNING rowId;",parVal.get());
			i++;

		}
	}

}
