/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#include <main/GrunfApp.h>
#include <iostream>
#include "main.h"
#include "rhea/File/File.h"
#include "rhea/SqlLite/DbConn.h"
#include "UpdateDB.h"
#include "gui/FrmTemplDef.h"
#include "gui/FrmTemplData.h"
#include "gui/FrmAbout.h"
#include "gui/FrmHelpContent.h"
#include "gui/FrmSettings.h"
#include "string"


int main(int argc, char *argv[]) {
    rhea::File* file = new rhea::File();
    file->setBase("~/rhea-Nm/grunf", true);

    rhea::DbConn* dbConn = rhea::DbConn::getInstance();
    UpdateDB* updDB = new UpdateDB();
    dbConn->setFile(file->getBase()+"grunf-data.db");
    dbConn->setOnUpdate(&UpdateDB::makeUpdate,updDB);
    dbConn->openDataBase();

    LanguagePack* lang = LanguagePack::getInstance();
    rhea::DbResultSet* res = dbConn->execQuery("SELECT lang FROM Settings;");

    std::string langStr ="";

    if (res->rowCount()==1) {
    	langStr = res->getField("lang")->getAsString();
    }

    if (langStr == ""){
    	std::cout << "LANGUAGE IS NOT SET" << std::endl;
    	lang->setLanguage(Languages::en);
    } else if (langStr == "si") {
    	lang->setLanguage(Languages::si);
    } else if (langStr == "hr") {
    	lang->setLanguage(Languages::hr);
    } else if (langStr == "bih") {
    	lang->setLanguage(Languages::bih);
    } else if (langStr == "en") {
    	lang->setLanguage(Languages::en);
    } else if (langStr == "de") {
    	lang->setLanguage(Languages::de);
    } else if (langStr == "fr") {
    	lang->setLanguage(Languages::fr);
    } else if (langStr == "it") {
    	lang->setLanguage(Languages::it);
    }

    auto app = GrunfApp::create();
    int result = app->run(argc, argv);
    delete(file);
    return result;

}

AppWindow::AppWindow() {
	global = Global::getInstance();
 //   maximize();
	set_size_request(800,650);
    set_title("grunf - Rhea Novo mesto");
    set_child(*windowBox); // put a MenuBar at the top of the box and other stuff below it.

//Create actions for menus and toolbars:

    add_action("settings", sigc::mem_fun(*this, &AppWindow::onMenuSettings));
    add_action("templates", sigc::mem_fun(*this, &AppWindow::onMenuTemplates));
    add_action("help", sigc::mem_fun(*this, &AppWindow::onMenuContent));
    add_action("about", sigc::mem_fun(*this, &AppWindow::onMenuAbout));

    Glib::VariantType* type = new Glib::VariantType("s");
    add_action_with_parameter("data_template",*type, sigc::mem_fun(*this, &AppWindow::onMenuDataTemplates));


    global->appWindow = this;
    add_controller(global->shortcutController);
    global->createShortCuts();

    m_refBuilder = Gtk::Builder::create();

    Glib::ustring ui_info =
            "<!-- Generated with glade 3.18.3 and then changed manually -->"
            "<interface>"
            "  <object class='GtkBox' id='toolbar'>"
            "    <property name='can_focus'>False</property>"
//            "    <child>"
//            "      <object class='GtkButton' id='toolbutton_new'>"
//            "        <property name='can_focus'>False</property>"
//            "        <property name='tooltip_text' translatable='yes'>New Standard</property>"
//            "        <property name='action_name'>app.newstandard</property>"
//            "        <property name='icon_name'>document-new</property>"
//           "        <property name='hexpand'>False</property>"
//            "        <property name='vexpand'>False</property>"
//            "      </object>"
//            "    </child>"
            "    <child>"
            "      <object class='GtkButton' id='toolbutton_quit'>"
            "        <property name='can_focus'>False</property>"
            "        <property name='tooltip_text' translatable='yes'>Quit</property>"
            "        <property name='action_name'>app.quit</property>"
            "        <property name='icon_name'>application-exit</property>"
            "        <property name='hexpand'>False</property>"
            "        <property name='vexpand'>False</property>"
            "      </object>"
            "    </child>"
            "  </object>"
            "</interface>";

    try
    {
        m_refBuilder->add_from_string(ui_info);
    }
    catch (const Glib::Error& ex)
    {
        std::cerr << "Building toolbar failed: " <<  ex.what();
    }

    auto toolbar = m_refBuilder->get_widget<Gtk::Box>("toolbar");
    if (!toolbar)
        g_warning("toolbar not found");
    else
        windowBox->append(*toolbar);

    toolbar->append(*frmMngr->getToolBox());
    myBox->append(*frmMngr->getMainBox());
    frmMngr->setHeaderStyle(rhea::WndHeaderStyle::HORZ);
    windowBox->append(*myBox);
    this->set_name("sora");

}

AppWindow::~AppWindow() {}

void AppWindow::on_menu_file_new_generic() {
    std::cout << "Was selected." << std::endl;
}

void AppWindow::onMenuTemplates() {
    FrmTemplDef* frm = new FrmTemplDef();
	frmMngr->addFrame(frm,lang->getText("txtMainMenu_FileStructures"));
    frmMngr->setOnActivate(&FrmTemplDef::onActivate,frm);
    frmMngr->setOnClose(&FrmTemplDef::onClose,frm);
    frmMngr->showFrame(frm);
}

void AppWindow::onMenuSettings() {
    FrmSettings* frm = new FrmSettings();
	frmMngr->addFrame(frm,lang->getText("txtMainMenu_FileSettings"));
    frmMngr->setOnActivate(&FrmSettings::onActivate,frm);
    frmMngr->setOnClose(&FrmSettings::onClose,frm);
    frmMngr->showFrame(frm);
}

void AppWindow::onMenuDataTemplates(Glib::VariantBase data) {
	auto derived = Glib::VariantBase::cast_dynamic<Glib::Variant<std::string>>(data);
    std::string val = derived.get();
    std::string id = val.substr(0,val.find(' '));
    val = val.substr(val.find(' ')+1,val.size());
    if ( val.find('_') != std::string::npos ){
    	val = val.replace(val.find('_'), 1, "");
    }
    FrmTemplData* frm = new FrmTemplData(id);
    frmMngr->addFrame(frm,val);
    frmMngr->setOnActivate(&FrmTemplData::onActivate,frm);
    frmMngr->setOnClose(&FrmTemplData::onClose,frm);
    frmMngr->showFrame(frm);

}

void AppWindow::onMenuAbout() {
    FrmAbout* frm = new FrmAbout();
	frmMngr->addFrame(frm,lang->getText("txtMainMenu_HelpAbout"));
    frmMngr->setOnActivate(&FrmAbout::onActivate,frm);
    frmMngr->setOnClose(&FrmAbout::onClose,frm);
    frmMngr->showFrame(frm);
}

void AppWindow::onMenuContent() {
    FrmHelpContent* frm = new FrmHelpContent();
	frmMngr->addFrame(frm,lang->getText("txtMainMenu_HelpInstr"));
    frmMngr->setOnActivate(&FrmHelpContent::onActivate,frm);
    frmMngr->setOnClose(&FrmHelpContent::onClose,frm);
    frmMngr->showFrame(frm);
}
