/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#include "ComboBox.h"

template <class T> rhea::ComboBox<T>::ComboBox() : Gtk::ComboBox () {
    refTreeModel = Gtk::ListStore::create(columns);
}

template <class T> void rhea::ComboBox<T>::addList(Glib::RefPtr<Gtk::ListStore>* dataColumns) {

    refTreeModel = *dataColumns;
    this->set_model(refTreeModel);
//    this->pack_start(m_Columns.m_col_id);
    this->pack_start(columns.col_name);

    //An example of adding a cell renderer manually,
    //instead of using pack_start(model_column, Gtk::PackOptions::EXPAND_WIDGET)
    //so we have more control:
    this->set_cell_data_func(cell,
                             sigc::mem_fun(*this, &ComboBox::on_cell_data_extra));
    this->pack_start(cell);

    this->signal_changed().connect( sigc::mem_fun(*this, &ComboBox::on_combo_changed) );

}

template <class T> void rhea::ComboBox<T>::on_cell_data_extra(const Gtk::TreeModel::const_iterator& iter) {
    auto row = *iter;
    const Glib::ustring extra = row[columns.col_extra];

    //Some arbitrary logic just to show that this is where you can do such things:

    //Transform the value, deciding how to represent it as text:
    if(extra.empty())
        cell.property_text() = "";
    else
        cell.property_text() = "-" + extra + "-";

    //Change other cell renderer properties too:
    cell.property_foreground() = (extra == "yadda" ? "red" : "green");
}

template <class T> void rhea::ComboBox<T>::on_combo_changed() {
    const auto iter = get_active();
    if (iter) {
        const Gtk::TreeRow choice = *iter;

        if (onChangeCallback){
            onChangeCallback(this, previousChoice, choice);
        }

        previousChoice = choice;
    }

}

template <class T> rhea::ComboBox<T>::~ComboBox() {

}
