/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#include "DateTime.h"

rhea::DateTime::DateTime() {

}

rhea::DateTime::DateTime(int day, int month, int year, int hour, int min, int sec) {
    setDate(day,month,year);
    setTime(hour,min,sec);
}

rhea::DateTime::~DateTime() {

}

std::string rhea::DateTime::getAsISO8601() {
//ISO8601 format je sledeči YYYY-MM-DD HH:MM:SS.SSS

    std::string day  = std::to_string(timestamp.tm_mday);
    std::string mon  = std::to_string(timestamp.tm_mon);
    std::string year = std::to_string(timestamp.tm_year);
    std::string hour = std::to_string(timestamp.tm_hour);
    std::string min  = std::to_string(timestamp.tm_min);
    std::string sec  = std::to_string(timestamp.tm_sec);

    day  = day.length()  < 2 ? "0" + day  : day;
    mon  = mon.length()  < 2 ? "0" + mon  : mon;
    hour = hour.length() < 2 ? "0" + hour : hour;
    min  = min.length()  < 2 ? "0" + min  : min;
    sec  = sec.length()  < 2 ? "0" + sec  : sec;

    return year + "-" + mon + "-" + day + " " + hour + ":" + min + ":" + sec;
}

std::string rhea::DateTime::getAsString(Languages lng, int format, int showPart) {
    // format is by defaultu 2
    // 1 = shortest d.m.yy h:m:s
    // 2 = with zero prefixes dd.mm.yy hh:mm:ss
    // 3 = with four digit year dd.mm.yyyy hh:mm:ss
    // 4 = with three letter month year dd.mmm.yyyy hh:mm:ss

    //showPart is 1-time, 2-date, 3-all

    std::string retStr = "";
    std::string day  = {""};
    std::string mon  = {""};
    std::string year = {""};
    std::string hour = {""};
    std::string min  = {""};
    std::string sec  = {""};

    if ((getDay()==0 ) && (getMonth()==0) && (getYear()==0) && (getHour()==0) && (getMin()==0) && (getSec()==0)){
        return retStr;
    }

    if(showPart !=1) {
        day = std::to_string(timestamp.tm_mday);
        mon = std::to_string(timestamp.tm_mon);
        if (format > 1) {
            day = day.length() < 2 ? "0" + day : day;
            mon = mon.length() < 2 ? "0" + mon : mon;
        }

        year = std::to_string(timestamp.tm_year);
        if (format<3){
            year = year.erase(0,2);
        }
    }

    if(showPart != 2){
        hour = std::to_string(timestamp.tm_hour);
        min  = std::to_string(timestamp.tm_min);
        sec  = std::to_string(timestamp.tm_sec);
        if (format != 1){
            hour = hour.length() < 2 ? "0" + hour : hour;
            min  = min.length() < 2 ? "0" + min : min;
            sec  = sec.length() < 2 ? "0" + sec : sec;
        }
    }


    switch (lng) {
        case Languages::si :{
            if (showPart != 1) {
                retStr = day + "." + mon + "." + "." + year;
            }
            if (showPart != 2){
                retStr = retStr.empty() ? "" : retStr + " ";
                retStr +=  hour + ":" + min + ":" + sec;
            }
            break;
        }

        case Languages::hr :{
            if (showPart != 1) {
                retStr = day + "." + mon + "." + "." + year;
            }
            if (showPart != 2){
                retStr = retStr.empty() ? "" : retStr + " ";
                retStr +=  hour + ":" + min + ":" + sec;
            }
            break;
        }

        case Languages::en :{
            if (showPart != 1) {
                retStr = mon + "/" + day + "/" + "/" + year;
            }
            if (showPart != 2){
                retStr = retStr.empty() ? "" : retStr + " ";
                retStr +=  hour + ":" + min + ":" + sec;
            }
            break;
        }

        case Languages::de :{
            if (showPart != 1) {
                retStr = year + "-" + mon + "-" + "-" + day;
            }
            if (showPart != 2){
                retStr = retStr.empty() ? "" : retStr + " ";
                retStr +=  hour + ":" + min + ":" + sec;
            }

        }

        case Languages::fr :{
            if (showPart != 1) {
                retStr = day + "/" + mon + "/" + "/" + year;
            }
            if (showPart != 2){
                retStr = retStr.empty() ? "" : retStr + " ";
                retStr +=  hour + ":" + min + ":" + sec;
            }
            break;
        }

        case Languages::it :{
            if (showPart != 1) {
                retStr = day + "/" + mon + "/" + "/" + year;
            }
            if (showPart != 2){
                retStr = retStr.empty() ? "" : retStr + " ";
                retStr +=  hour + ":" + min + ":" + sec;
            }
        }

        default:{
            if (showPart != 1) {
                retStr = day + "." + mon + "." + "." + year;
            }
            if (showPart != 2){
                retStr = retStr.empty() ? "" : retStr + " ";
                retStr +=  hour + ":" + min + ":" + sec;
            }

        }
    }

    return retStr;
}

int rhea::DateTime::getDay() {
    return timestamp.tm_mday;
}

int rhea::DateTime::getMonth() {
    return timestamp.tm_mon;
}

int rhea::DateTime::getYear() {
    return timestamp.tm_year;
}

int rhea::DateTime::getHour() {
    return timestamp.tm_hour;
}

int rhea::DateTime::getMin() {
    return timestamp.tm_min;
}

int rhea::DateTime::getSec() {
    return timestamp.tm_sec;
}


void rhea::DateTime::set(time_t &theTime) {
    struct tm* structure = localtime(&theTime);
    setDate(structure->tm_mday,structure->tm_mon,structure->tm_year);
    setTime(structure->tm_hour,structure->tm_min,structure->tm_sec);
}

void rhea::DateTime::setDate(int day, int month, int year) {
    timestamp.tm_mday = day;
    timestamp.tm_mon  = month;
    timestamp.tm_year = year;
}

void rhea::DateTime::setTime(int hour, int minute, int sec) {
    timestamp.tm_hour = hour;
    timestamp.tm_min  = minute;
    timestamp.tm_sec  = sec;
}

void rhea::DateTime::setAsISO8601(std::string str) {
//ISO8601 format je sledeči YYYY-MM-DD HH:MM:SS.SSS
    if (str.empty()) {
        clear();
        return;
    }

    int posStart = 0;
    int pos = str.find('-');
    std::string tmp = str.substr(0,pos);
    timestamp.tm_year = std::stoi(tmp);

    posStart = pos+1;
    pos = str.find('-',posStart);
    tmp = str.substr(posStart,pos-posStart);
    timestamp.tm_mon = std::stoi(tmp);

    posStart = pos+1;
    pos = str.find(' ',posStart);
    tmp = str.substr(posStart,pos-posStart);
    timestamp.tm_mday = std::stoi(tmp);

    posStart = pos+1;
    pos = str.find(':',posStart);
    tmp = str.substr(posStart,pos-posStart);
    timestamp.tm_hour = std::stoi(tmp);

    posStart = pos+1;
    pos = str.find(':',posStart);
    tmp = str.substr(posStart,pos-posStart);
    timestamp.tm_min = std::stoi(tmp);

    posStart = pos+1;
    tmp = str.substr(posStart,posStart+2);
    timestamp.tm_min = std::stoi(tmp);

}

void rhea::DateTime::clear() {
    setDate(0,0,0);
    setTime(0,0,0);
}
