/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#ifndef RHEA_FILE_H
#define RHEA_FILE_H

#include "../../po/Po.h"
#include <string>
#include "vector"
#include "stdbool.h"
#include <stdio.h>
#include <stdlib.h>
#include "minizip/unzip.h"
#include "minizip/zip.h"
#include "str_builder.h"
#include <cstring>
#include "../ListedMap/ListedMap.h"
#ifdef _WIN32
#  include <Windows.h>
#else
#  include <sys/stat.h>
#endif

namespace rhea {

#ifdef _WIN32
const char SEP = '\\';
#else
const char SEP = '/';
#endif

struct FileContent {
    unsigned char* data = {nullptr};
    size_t		   size = 0;
};

typedef enum {
    ZIPPER_RESULT_ERROR = 0,
    ZIPPER_RESULT_SUCCESS,
    ZIPPER_RESULT_SUCCESS_EOF
} zipper_result_t;

typedef void (*zipper_read_cb_t)(const unsigned char *buf, size_t size, void *thunk);

#define BUF_SIZE 8192
#define MAX_NAMELEN 256

typedef rhea::ListedMap<std::string,FileContent*> Files;

    class File : public Po {

    private:

        inline static std::string base = {initStr};
        inline static std::string pathDelimiter = "/";
        std::string path = {initStr};
        std::string name = {initStr};
        std::string type = {initStr};
        long long length = {initPoId};
        std::vector<char> content;

        void setType(std::string val);
        void clearObject() override;
        bool recurse_mkdir(const char *dirname);

        zipper_result_t zipper_read(unzFile zfile, zipper_read_cb_t cb, void *thunk);
        zipper_result_t zipper_read_buf(unzFile zfile, unsigned char **buf, size_t *buflen);
        static void zipper_read_buf_cb(const unsigned char *buf, size_t buflen, void *thunk);

        bool zipper_skip_file(unzFile zfile);
        char *zipper_filename(unzFile zfile, bool *isutf8);
        bool zipper_isdir(unzFile zfile);
        uint64_t zipper_filesize(unzFile zfile);
    public:

        File();
        ~File();

        bool copyFrom(std::string fullFileName);
        std::string getFullName();
        void setFullName(std::string file);
        std::string &getBase();
        static void setBase(std::string val, bool create);
        std::string &getPath();
        void setPath(std::string val, bool create);
        std::string &getName();
        void setName(std::string val);
        std::string &getType();
        static bool createPath(std::string &path);
        static bool dirExists(std::string &path);
        static std::string replaceTilda(std::string &val);
        bool getFileContent(std::string &out);
        bool compresZip(std::string &out);
        Files* decompresZip(std::string dir, bool writeToDisk);

        size_t writeFile(const char *filename, const unsigned char *data, size_t len, bool append);
        unsigned char *readFile(const char *filename = NULL, size_t *len = NULL);

        bool zipper_add_file(zipFile zfile, const char *filename,const char* destName);
        bool zipper_add_buf(zipFile zfile, const char *zfilename, const unsigned char *buf, size_t buflen);
        bool zipper_add_dir(zipFile zfile, const char *dirname);

    };

}
#endif //RHEA_FILE_H
