/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#include "FilesInBox.h"
#include <iostream>

rhea::FilesInBox::FilesInBox() : rhea::Grid() {

	grid->set_hexpand(false);
	grid->set_halign(Gtk::Align::START);
	grid->set_size_request(400,-1);
	set_column_homogeneous(false);
	std::string cssData =".rhea_gridHeaderCell {background-color: #5391c1;} .rhea_gridHeaderLbl {background-color: #5391c1; color: #FFFFFF; border-color: black; border-style: none none solid none ; border-width: 1px;}";
	cssData += ".rhea_gridRowEven {background-color: #FFFFFF; color:black; border-color: black; border-style: none none solid solid;border-width: 1px;}";
	cssData += ".rhea_gridRowOdd {background-color: #E0E0E0; color:black; border-color: black; border-style: none none solid solid;border-width: 1px;}";
	cssData += ".rhea_gridRowEven:hover {background-color: #D0D0EF;} .gridCellOdd:hover {background-color: #D0D0EF;}";
	cssData += ".rhea_filesHeaderLbl {background-color: #5391c1; color: #FFFFFF; border-color: black; border-style: none none solid none ; border-width: 1px;}";

	lblHdr->add_css_class("rhea_filesHeaderLbl");
	lblHdr->set_text(lang->getText("txtfileBox_Files_RHEA")+"          ");
	lblHdr->set_name("filesHeaderLbl");
	lblHdr->set_hexpand(true);

	btnAdd->set_label(lang->getText("txtfileBox_Add_RHEA"));
//	btnAdd->set_image_from_icon_name("document-new");
    btnAdd->signal_clicked().connect(sigc::mem_fun(*this, &FilesInBox::btnAddOnClick));
	btnAdd->set_hexpand(false);
	btnAdd->set_size_request(80,-1);
	btnAdd->set_halign(Gtk::Align::END);

	std::vector<Gtk::Widget*> header = {};
	header.push_back(lblHdr);
	header.push_back(btnAdd);
	setHeaders(&header,0);

	addCSS(cssData);
	getHeader(1)->set_hexpand(false);
//	getHeader(1)->setHAlign(Gtk::Align::FILL);


	connectWidget(&FilesInBox::dataListing, this);
	connectOnClick(&FilesInBox::onGridClick, this);
}

rhea::FilesInBox::~FilesInBox() {

}

void rhea::FilesInBox::btnAddOnClick() {

	  auto dialog = new Gtk::FileChooserDialog(lang->getText("txtExport_selectFile"),
	          Gtk::FileChooser::Action::OPEN);
	  Gtk::Window *window = dynamic_cast<Gtk::Window*>(this->get_root());
	  dialog->set_transient_for(*window);
	  dialog->set_modal(true);
	  dialog->signal_response().connect(sigc::bind( sigc::mem_fun(*this, &FilesInBox::onDialogResponse), dialog));

	  //Add response buttons to the dialog:
	  dialog->add_button(lang->getText("txtExport_cancel"), Gtk::ResponseType::CANCEL);
	  dialog->add_button(lang->getText("txtExport_save"), Gtk::ResponseType::OK);

	  //Add filters, so that only certain file types can be selected:

	  auto filterLibre = Gtk::FileFilter::create();
	  filterLibre->set_name("Libre office files");
	  filterLibre->add_mime_type("application/vnd.oasis.opendocument.text");
	  filterLibre->add_mime_type("application/vnd.oasis.opendocument.spreadsheet");
	  dialog->add_filter(filterLibre);

	  auto filterText = Gtk::FileFilter::create();
	  filterText->set_name("Text files");
	  filterText->add_mime_type("text/plain");
	  dialog->add_filter(filterText);

	  auto filter_any = Gtk::FileFilter::create();
	  filter_any->set_name("Any files");
	  filter_any->add_pattern("*");
	  dialog->add_filter(filter_any);

	  //Show the dialog and wait for a user response:
	  dialog->show();
}

void rhea::FilesInBox::btnDeleteOnClick(rhea::GridCell* cell,std::string file) {
	if ( cell->getRow() >= files.size() ) {
		return;
	}
	if (onDeleteCallback) {
		onDeleteCallback(cell, file);
	}
	files.erase(files.begin()+cell->getRow());
	setRows(files.size());
	refresh();
}

Gtk::Widget* rhea::FilesInBox::dataListing (int col,int row) {
	if ( row >= files.size() ) {
		return nullptr;
	}

	if (col == 0){
		Gtk::Label* name = new Gtk::Label();
		name->set_text(files.at(row));
		return name;
	} else if (col == 1){
		Gtk::Button* btnDelete = new Gtk::Button();
		btnDelete->set_label(lang->getText("txtDialog_btnRemove"));
		btnDelete->set_hexpand(false);
		btnDelete->set_size_request(80,-1);
//		btnDelete->set_halign(Gtk::Align::END);
		rhea::GridCell* cell = dynamic_cast<rhea::GridCell*>(this->getCell(col, row));
		cell->set_halign(Gtk::Align::END);
		btnDelete->signal_clicked().connect(sigc::bind(sigc::mem_fun(*this, &FilesInBox::btnDeleteOnClick),cell,files.at(row)));
        return btnDelete;
	}

	return nullptr;
}

void rhea::FilesInBox::onDialogResponse(int response_id, Gtk::FileChooserDialog* dialog) {
	  switch (response_id) {
	    case Gtk::ResponseType::OK: {
	    	addFile(dialog->get_file()->get_path());
	      break;

	    } case Gtk::ResponseType::CANCEL: {
	      break;

	    } default: {

	    }

	  }

	  delete dialog;

}

void rhea::FilesInBox::addFile(std::string fileName) {
    if (fileName.empty()) {
    	return;
    }

    files.push_back(fileName);
    setRows(files.size());
    refresh();
	if (onAddCallback) {
		onAddCallback(getCell(0,files.size()-1), fileName);
	}
}

void rhea::FilesInBox::addFiles(std::vector<std::string> newFiles) {
	if (newFiles.empty()) {
		return;
	}

	for (std::string &file : newFiles){
		if (!file.empty()) {
			addFile(file);
		}
	}

}

void rhea::FilesInBox::setID(size_t index,std::string id){
	uuids.insert({id, index});
}

void rhea::FilesInBox::removeFile(std::string fileName){
    if (fileName.empty()) {
    	return;
    }

    for (int i = files.size(); i < 0; i-- ){
    	if (fileName == files.at(i)){
    		files.erase(files.begin()+i);
    		for (auto it = uuids.begin(); it != uuids.end(); ++it) {
				if (it->second == i) {
					uuids.erase(it->first);
					break;
				}
    		}
    		setRows(files.size());
    		refresh();

    		break;
    	}
    }
}

void rhea::FilesInBox::removeByID(std::string id){
	auto pos = uuids.find(id);
	if (pos != uuids.end()) {
	    size_t value = pos->second;
		uuids.erase(id);
		files.erase(files.begin()+value);
	}
}

void  rhea::FilesInBox::removeAll(){
	files.clear();
	setRows(files.size());
	refresh();
}

std::string rhea::FilesInBox::getFile(std::string id) {
	auto pos = uuids.find(id);
	if (pos != uuids.end()) {
		return files.at(pos->second);
	}
	return "";
}

void rhea::FilesInBox::changeFile(size_t index, std::string filename){
	if(files.empty()){return;}
	if(index <0 || index >=files.size()){return;}
	files.at(index) = filename;
}

void rhea::FilesInBox::changeFileByID(std::string id, std::string filename){
	auto pos = uuids.find(id);
	if (pos != uuids.end()) {
		files.at(pos->second) = filename;
	}

}

std::vector<std::string> rhea::FilesInBox::getAllFiles(){
	return files;
}

void rhea::FilesInBox::onGridClick(rhea::GridCell* cell) {
	std::string str ="xdg-open \"" + files.at(cell->getRow())+"\"";
	system(str.c_str());
}
