/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#ifndef RHEA_FILESINBOX_FILESINBOX_H_
#define RHEA_FILESINBOX_FILESINBOX_H_

#include <vector>
#include <map>
#include "string.h"
#include "../Grid/Grid.h"
#include "../../main/LanguagePack.h"

namespace rhea {

class FilesInBox: public rhea::Grid {
private:
	LanguagePack* lang = LanguagePack::getInstance();
	std::function<void (rhea::GridCell*, std::string)> onDeleteCallback = {nullptr};
	std::function<void (rhea::GridCell*, std::string)> onAddCallback = {nullptr};
	std::vector<std::string> files = {};
	std::map<std::string, size_t> uuids = {};
	Gtk::Label* lblHdr = new Gtk::Label();
	Gtk::Button* btnAdd = new Gtk::Button();
	void btnAddOnClick ();
	void btnDeleteOnClick(rhea::GridCell* cell, std::string);
	Gtk::Widget* dataListing (int col,int row);
	void onDialogResponse(int response_id, Gtk::FileChooserDialog* dialog);

public:
	FilesInBox();
	~FilesInBox();
	void addFile(std::string fileName);
	void addFiles(std::vector<std::string> files);
	void setID(size_t index,std::string id);
	void removeFile(std::string fileName);
	void removeByID(std::string id);
	void removeAll();
	std::string getFile(std::string id);
	void changeFile(size_t index, std::string filename);
	void changeFileByID(std::string id, std::string filename);
	std::vector<std::string> getAllFiles();
	void onGridClick(rhea::GridCell* cell);
	template<typename A, typename B> void connectOnDeleteFile (A funcPtr, B objPtr) {
		onDeleteCallback = (std::bind(funcPtr, objPtr, std::placeholders::_1, std::placeholders::_2 ));}
	template<typename A, typename B> void connectOnAddFile (A funcPtr, B objPtr) {
		onAddCallback = (std::bind(funcPtr, objPtr, std::placeholders::_1, std::placeholders::_2 ));}

};


}

#endif /* RHEA_FILESINBOX_FILESINBOX_H_ */
