/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#include "Grid.h"
#include <iostream>

rhea::Grid::Grid() : Gtk::ScrolledWindow() {

    grid->set_vexpand(true);
    grid->set_hexpand(true);
    this->set_hexpand(true);
    this->set_vexpand(true);
    this->set_child(*grid);
}

rhea::Grid::Grid(int cols,int rows) : Gtk::ScrolledWindow() {
    grid->set_vexpand(true);
    grid->set_hexpand(true);
    this->set_hexpand(true);
    this->set_vexpand(true);

    this->set_child(*grid);
    colCount = cols ;
    rowCount = rows;
}


rhea::Grid::~Grid() {
  delete(this);
}

std::vector<rhea::GridCell *> rhea::Grid::getChildren(){
    return children;
}

void rhea::Grid::deleteChildren(){
    for (unsigned long i = getChildren().size()-1;i>0;i--){
        delete(getChildren().at(i));
    }

}

int rhea::Grid::getColumns() {
    int cols=0;
    for(auto rows: headersMap){
        cols = cols < rows.second ? rows.second :cols;
    };
    colCount = colCount < cols? cols : colCount;
    return colCount;
}

int rhea::Grid::getRows() {
    return rowCount;
}

void rhea::Grid::setColumns(int val) {
    colCount = val;
}

void rhea::Grid::setRows(int val) {
    if (val < rowCount) {
    	if (grid){
    		for (int row = rowCount-1; row>val-1;row--){
    			for (int col = 0;col<getColumns();col++){
    				rhea::GridCell* cell = dynamic_cast<rhea::GridCell *>(grid->get_child_at(col, row+headerHeight));
    				grid->remove(*cell);
    	            delete(cell);
    			}
    		}
    	}
    }
    rowCount = val;
}

void rhea::Grid::set_column_homogeneous(bool val){
    grid->set_column_homogeneous(val);
}

void rhea::Grid::setHeaders(std::vector<std::string>* labels, int row = 0) {
    int pos = 0;
    for(auto &str : *labels){
        rhea::GridHeader* lblHdr = new rhea::GridHeader();
        lblHdr->setColumn(pos);
        lblHdr->setRow(row);
        lblHdr->add_css_class("rhea_header_"+std::to_string(pos));
        lblHdr->setCaption(str);
        lblHdr->addCSSProvider(cssProvider);
        lblHdr->applyCSS();
        grid->attach(*lblHdr->draw(),pos,row);
        children.push_back(lblHdr);
        pos++;
    }
    setColumns(pos);
}

void rhea::Grid::setHeaders(std::vector<Gtk::Widget*>* headerCells, int row = 0) {
    int pos = 0;
    for(Gtk::Widget* hdr: *headerCells) {
    	rhea::GridHeader* lblHdr = new rhea::GridHeader();
        lblHdr->setColumn(pos);
        lblHdr->setRow(row);
        lblHdr->set_hexpand(hdr->get_hexpand());
        lblHdr->set_halign(hdr->get_halign());
        lblHdr->add_css_class("rhea_header_"+std::to_string(pos));
        lblHdr->attachWidget(hdr);
        lblHdr->addCSSProvider(cssProvider);
        lblHdr->applyCSS();
        grid->attach(*lblHdr,pos,row);
        children.push_back(lblHdr);
        pos++;
    }
    setColumns(pos);
}

void rhea::Grid::addHeader(std::string caption, bool sortupDown, int col, int row, int width, int height, Gtk::Image* icon) {

	row = row == -1 ? row = 0: row;
	col = col == -1 ?  getColumns() : col;

	setHeaderMap(col,row,width,height);
    rhea::GridHeader* hdr = new rhea::GridHeader();

    hdr->setIcon(icon);
    hdr->setCaption(caption);
    hdr->setShowSort(sortupDown);
    hdr->setColumn(col);
    hdr->setRow(row);
    hdr->setSpanH(width);
    hdr->setSpanV(height);
    hdr->addCSSProvider(cssProvider);
    hdr->applyCSS();
    grid->attach(*hdr->draw(),hdr->getColumn(),hdr->getRow(),hdr->getSpanH(),hdr->getSpanV());
    children.push_back(hdr);
    setColumns(getColumns()+1);
}

void rhea::Grid::addHeader( Gtk::Widget *widget, bool sortupDown, int col, int row, int width, int height, Gtk::Image *icon){
		row = row == -1 ? row = 0: row;
		col = col == -1 ?  getColumns() : col;

	    rhea::GridHeader* hdr = new rhea::GridHeader();

	    hdr->setShowSort(sortupDown);
	    hdr->setColumn(col);
	    hdr->setRow(row);
	    hdr->setSpanH(width);
	    hdr->setSpanV(height);
		hdr->attachWidget(widget);
	    hdr->addCSSProvider(cssProvider);
    	grid->attach(*hdr,col,row,width,height);
    	children.push_back(hdr);
    	setColumns(getColumns()+1);
}

void rhea::Grid::addHeader(rhea::GridHeader* hdr) {
    int x= hdr->getColumn();
    int y= hdr->getRow();
    setHeaderMap(x,y,hdr->getSpanH(),hdr->getSpanV());
    hdr->setColumn(x);
    hdr->setRow(y) ;
    hdr->addCSSProvider(cssProvider);
    hdr->applyCSS();
    grid->attach(*hdr->draw(),hdr->getColumn(),hdr->getRow(),hdr->getSpanH(),hdr->getSpanV());
    children.push_back(hdr);
    setColumns(getColumns()+1);
}

void rhea::Grid::removeColumn(int col){

}

void rhea::Grid::setHeaderMap(int &col, int &row,int width,int height){

    row = row == -1 ? 0 : row;

    int hght = row + height;
    headerHeight = headerHeight < hght ? hght : headerHeight;
    int myCol = 0;
    if (headersMap.find(row) == headersMap.end()) {
        headersMap.insert({row, myCol});
    } else {
        myCol = headersMap.at(row);
    }

    myCol ++;
    if (col < 0){
        col = myCol;
    }
    headersMap.at(row) = myCol + width-1;
}

rhea::GridHeader* rhea::Grid::getHeader(long col){
	rhea::GridHeader* ret = dynamic_cast<rhea::GridHeader*>(grid->get_child_at(col,0));
	if (ret){return ret;}
	return nullptr;
}

void rhea::Grid::refresh(){
    updateValues(getColumns(),getRows());
    for(int i = 0; i < getColumns();i++) {
    	getHeader(i)->set_hexpand(true);
//    	getHeader(i)->set_halign(Gtk::Align::FILL);
    	getHeader(i)->set_vexpand(false);
    	getHeader(i)->set_valign(Gtk::Align::BASELINE);
    	getHeader(i)->set_size_request (getHeader(i)->getWidget()->get_width(),getHeader(i)->getWidget()->get_height());
    }

    grid->show();

}

void rhea::Grid::updateValues(int cols, int rows) {
	children.clear();
    if (stringCallback || widgetCallback) {
        for(int i = 0; i < cols; i++){
            for (int j = 0; j<rows; j++){
                if (stringCallback) {
                    Gtk::Label* lbl = {nullptr};
                    rhea::GridCell* cellTxt = dynamic_cast<rhea::GridCell *>(grid->get_child_at(i, j+headerHeight));
                    if (!cellTxt) {
                    	cellTxt = new rhea::GridCell();
                    	lbl = new Gtk::Label();
                        cellTxt->attachWidget(lbl);
                        SetUpCell(cellTxt,i,j);
                        grid->attach(*cellTxt, i, j+headerHeight);
                    }
                    lbl = static_cast<Gtk::Label*>(cellTxt->widget);
                    lbl->set_label(stringCallback(i, j));
                    cellTxt->set_size_request(lbl->get_width(),lbl->get_height());
                    children.push_back(cellTxt);
                    applyCSS(cellTxt);
                }
                if (widgetCallback) {
                	rhea::GridCell* someCell = dynamic_cast<rhea::GridCell *>(grid->get_child_at(i, j + headerHeight));
                	if (!someCell){
                		someCell = new rhea::GridCell();
                		SetUpCell(someCell,i,j);
                		grid->attach(*someCell, i, j + headerHeight);
                    }
                	someCell->removeWidget();
                	Gtk::Widget* someWidget = widgetCallback(i,j );
                	if (someWidget) {
                		someCell->attachWidget(someWidget);
                	    someCell->show();
                	    someCell->set_size_request(someWidget->get_width(),someWidget->get_height());
                	}
                	children.push_back(someCell);
                    applyCSS(someCell);
                }
            }

        }
    }
}

void rhea::Grid::attach(Gtk::Widget &widget, int col, int row, int spanCol, int spanRow){
	rhea::GridCell* cell = new rhea::GridCell();
	cell->attachWidget(&widget);
	SetUpCell(cell,col,row);
	cell->show();
	grid->attach(*cell,col,row,spanCol,spanRow);
    children.push_back(cell);
}

void rhea::Grid::SetUpCell(rhea::GridCell* cell,int col,int row){
    cell->setColumn(col);
    cell->setRow(row);
    cell->connectOnClick(&rhea::Grid::OnClick,this);
}

void rhea::Grid::applyCSS(rhea::GridCell* cell){
    if (cell->getRow() % 2 == 0) {
    	cell->add_css_class("rhea_gridRowEven");
		if (cssForEvenRow !=""){
			cell->add_css_class(cssForEvenRow);
		}
    	if (cell->widget){
    		cell->widget->add_css_class("rhea_gridRowEven");
    		if (cssForEvenRow != ""){
    			cell->widget->add_css_class(cssForEvenRow);
    		}
    	}
    }else {
    	cell->add_css_class("rhea_gridRowOdd");
		if (cssForOddRow !=""){
			cell->add_css_class(cssForOddRow);
		}
    	if (cell->widget){
    		cell->widget->add_css_class("rhea_gridRowOdd");
    		if (cssForOddRow !=""){
    			cell->widget->add_css_class(cssForOddRow);
    		}

    	}
    }
    cell->addCSSProvider(cssProvider);
    cell->applyCSS();

}

void rhea::Grid::setCssForEvenRow(std::string val) {
	cssForEvenRow = val;
}

std::string rhea::Grid::getCssForEvenRow() {
	return cssForEvenRow;
}

void rhea::Grid::setCssForOddRow(std::string val) {
	cssForOddRow = val;
}

std::string rhea::Grid::getCssForOddRow() {
	return cssForOddRow;
}

void rhea::Grid::setCssForSelectedRow(std::string val) {
	cssForSelectedRow = val;

}

std::string rhea::Grid::getCssForSelectedRow() {
	return cssForSelectedRow;
}

void rhea::Grid::OnClick(rhea::GridCell *cell) {

	for (rhea::GridCell* some : children ) {
		if( some->getRow() == cell->getRow() ) {
			some->add_css_class("rhea_selectedRow");
			if (cssForSelectedRow !=""){
				some->add_css_class(cssForSelectedRow);
			}
			if (cell->widget){
				some->widget->add_css_class("rhea_selectedRow");
				if (cssForSelectedRow !=""){
					some->widget->add_css_class(cssForSelectedRow);
				}
			}
		} else {
			some->remove_css_class("rhea_selectedRow");
			if (cssForSelectedRow !=""){
				some->remove_css_class(cssForSelectedRow);
			}
			if (cell->widget){
				some->widget->remove_css_class("rhea_selectedRow");
				if (cssForSelectedRow !=""){
					some->widget->remove_css_class(cssForSelectedRow);
				}
			}
		}
	}

	if(onClickCallback){
		onClickCallback(cell);
	}
	cell->addCSSProvider(cssProvider);
    cell->applyCSS();
}

void rhea::Grid::connectOnClick(std::function<void (rhea::GridCell* cell)> funcPtr) {
    if(funcPtr){
    	onClickCallback = funcPtr;
    }
}

void rhea::Grid::connectString (std::function<std::string (int col, int row)> funcPtr)  {
    stringCallback =funcPtr;
};

void rhea::Grid::connectWidget (std::function<Gtk::Widget* (int col, int row)> funcPtr)  {
    widgetCallback = funcPtr;
}

int rhea::Grid::getWidth() {
    return grid->get_width();
}

int rhea::Grid::getHeight() {
    return grid->get_height();
}

void rhea::Grid::resize() {
    set_size_request(getWidth(), getHeight());
}

void rhea::Grid::setSize(int width,int height) {
	set_size_request(width, height);
}


rhea::GridCell* rhea::Grid::getCell (int col,int row) {
	rhea::GridCell* cell = dynamic_cast <rhea::GridCell*>(grid->get_child_at(col,row+headerHeight));
	return cell;
}

Gtk::Widget* rhea::Grid::getWidget (int col,int row) {
	auto cell = dynamic_cast <rhea::GridCell*>(grid->get_child_at(col,row+headerHeight));
	return cell->widget;
}

Glib::RefPtr<Gtk::StyleContext> rhea::Grid::get_style_context() {
	return grid->get_style_context();
}

void rhea::Grid::addCSS(std::string css) {
	cssData = css;
	cssProvider->load_from_data(cssData);
}
