/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#ifndef RHEA_GRID_H
#define RHEA_GRID_H

#include "gtkmm.h"
#include "GridCell.h"
#include "GridHeader.h"

namespace rhea {

class Grid : public Gtk::ScrolledWindow {
private:
    int rowCount = 0;
    int colCount = 0;
    int headerHeight = {1};
    std::string cssForEvenRow = "";
    std::string cssForOddRow = "";
    std::string cssForSelectedRow = "";
    std::string cssData = "";
    std::vector<rhea::GridCell*> children;
    std::function<void (rhea::GridCell* cell)> onClickCallback    = {nullptr};
    std::function<std::string  (int col, int row)> stringCallback = {nullptr};
    std::function<Gtk::Widget* (int col, int row)> widgetCallback = {nullptr};
    std::map<int, int> headersMap = {{0, 0}};
    void OnClick(rhea::GridCell* cell);
    void setHeaderMap(int &col, int &row, int width, int height);
    void SetUpCell(rhea::GridCell* cell,int col,int row);

public:
    Gtk::Grid *grid = new Gtk::Grid();
    Grid();
    Grid(int cols,int rows);
    ~Grid();
    Glib::RefPtr<Gtk::CssProvider> cssProvider = Gtk::CssProvider::create();
    std::vector<rhea::GridCell *> getChildren();
    void deleteChildren();
    int getColumns();
    int getRows();
    void setColumns(int val);
    void setRows(int val);
    void set_column_homogeneous(bool val);
    void addHeader(std::string caption, bool sortupDown = false, int col = -1, int row = -1, int width = 1,
                   int height = 1, Gtk::Image *icon = nullptr);
    void addHeader( Gtk::Widget *widget,bool sortupDown = false, int col = -1, int row = -1, int width = 1,
            int height = 1, Gtk::Image *icon = nullptr);
    void addHeader(rhea::GridHeader *hdr);
    void setHeaders(std::vector<std::string> *labels, int row);
    void setHeaders(std::vector<Gtk::Widget *> *headerCells, int row );
    void refresh();
    void removeColumn(int col);
    void updateValues(int col, int row);
    void applyCSS(rhea::GridCell* cell);
    void attach(Gtk::Widget &widget, int col, int row, int spanCol = 1, int spanRow = 1);
    void connectOnClick(std::function<void (rhea::GridCell* cell)> funcPtr);
    void connectString (std::function<std::string (int col, int row)> funcPtr);
    void connectWidget (std::function<Gtk::Widget* (int col, int row)> funcPtr);
    template<typename A, typename B> void connectOnClick (A funcPtr, B objPtr);
    template<typename A, typename B> void connectString  (A funcPtr, B objPtr);
    template<typename A, typename B> void connectWidget  (A funcPtr, B objPtr);
    int getWidth();
    int getHeight();
    void setSize(int width,int height);
    void resize();
    rhea::GridHeader* getHeader(long col);
    rhea::GridCell* getCell (int col,int row);
    Gtk::Widget* getWidget (int col,int row);
    Glib::RefPtr<Gtk::StyleContext> get_style_context();
    void addCSS(std::string css);
    void setCssForEvenRow(std::string val);
    std::string getCssForEvenRow();
    void setCssForOddRow(std::string val);
    std::string getCssForOddRow();
    void setCssForSelectedRow(std::string val);
    std::string getCssForSelectedRow();

};

}

#include "Grid.tpp"

#endif //RHEA_GRID_H
