/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#include "GridCell.h"
#include <iostream>

rhea::GridCell::GridCell() : Gtk::Box(Gtk::Orientation::HORIZONTAL) {
    std::shared_ptr<Gtk::GestureClick> click =  Gtk::GestureClick::create();
    click->signal_released().connect(sigc::mem_fun(*this, &GridCell::OnClick));
    this->add_controller(click);
    this->add_css_class("rhea_gridCell");
}

rhea::GridCell::GridCell(Gtk::Orientation orientation) : Gtk::Box(orientation) {
    std::shared_ptr<Gtk::GestureClick> click =  Gtk::GestureClick::create();
    click->signal_released().connect(sigc::mem_fun(*this, &GridCell::OnClick));
    this->add_controller(click);
    this->add_css_class("rhea_gridCell");
}


rhea::GridCell::~GridCell() {

}

void rhea::GridCell::attachWidget(Gtk::Widget* wdg){
	if (!wdg) {return;}

	widget = wdg;
	widget->add_css_class("rhea_gridCellWdg");
	append(*widget);
	widget->set_halign(Gtk::Align::FILL);
	widget->set_hexpand(true);
	widget->set_valign(Gtk::Align::BASELINE);
	widget->set_vexpand(false);
}

void rhea::GridCell::removeWidget(){
//	Gtk::Box::remove(*widget);
	if (widget){
		widget->unparent();
		delete(widget);
		widget = nullptr;
	}
}

int rhea::GridCell::getColumn() {
    return column;
}

void rhea::GridCell::setColumn(int val) {
    column = val;
}

int rhea::GridCell::getRow() {
    return row;
}

void rhea::GridCell::setRow(int val) {
    row = val;
}

int rhea::GridCell::getSpanH() {
    return spanH;
}

void rhea::GridCell::setSpanH(int val) {
    spanH = val;
}

int rhea::GridCell::getSpanV() {
    return spanV;
}

void rhea::GridCell::setSpanV(int val) {
    spanV = val;
}

void rhea::GridCell::setHAlign(Gtk::Align align) {
	set_halign(align);
}

Gtk::Widget* rhea::GridCell::draw() {
    return this;
}

void rhea::GridCell::OnClick(int button, double x, double y){
	if (onClickCallback) {
	   onClickCallback(this);

   }
}

void rhea::GridCell::addCSSProvider(Glib::RefPtr<Gtk::CssProvider> provider){
	cssProvider = provider;
}

void rhea::GridCell::applyCSS() {
	if (!cssProvider) {return;}
	auto display = Gdk::Display::get_default();
	this->get_style_context()->add_provider_for_display(display,cssProvider,GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
	if (!widget) {return;}
	widget->get_style_context()->add_provider_for_display(display,cssProvider,GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);//(cssProvider,GTK_STYLE_PROVIDER_PRIORITY_APPLICATION); //GTK_STYLE_PROVIDER_PRIORITY_USER
}
