/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */
#ifndef RHEA_GRIDCELL_H
#define RHEA_GRIDCELL_H

#include "gtkmm.h"

namespace rhea {

class GridCell : public Gtk::Box {

private:
    int column = {-1};
    int row    = {-1};
    int spanH  = {1};
    int spanV  = {1};
    std::function<void (rhea::GridCell*)> onClickCallback = {nullptr};

public:
    Glib::RefPtr<Gtk::CssProvider> cssProvider = nullptr;
    Gtk::Widget* widget = {nullptr};
    GridCell();
    GridCell(Gtk::Orientation orientation);
    ~GridCell();
    void attachWidget(Gtk::Widget* wdg);
    void removeWidget();
    int getColumn();
    void setColumn(int val);
    int getRow();
    void setRow(int val);
    int getSpanH();
    void setSpanH(int val);
    int getSpanV();
    void setSpanV(int val);
    void setHAlign(Gtk::Align align);
    virtual Gtk::Widget* draw();
    void OnClick(int button, double x, double y);
    template<typename A, typename B> void connectOnClick (A funcPtr, B objPtr);
    void addCSSProvider(Glib::RefPtr<Gtk::CssProvider> provider);
    virtual void applyCSS();
};

}

#include "GridCell.tpp"

#endif //RHEA_GRIDCELL_H
