/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#include "GridHeader.h"
#include "iostream"

rhea::GridHeader::GridHeader(): rhea::GridCell() {
	add_css_class("rhea_gridHeaderCell");
	lblCaption->add_css_class("rhea_gridHeaderLbl");
	iconSort->add_css_class("rhea_gridHeadeIcon");
	lblCaption->set_size_request(get_width(), get_height());

	lblCaption->set_halign(Gtk::Align::FILL);
	lblCaption->set_hexpand(true);
	lblCaption->set_valign(Gtk::Align::BASELINE);
	lblCaption->set_vexpand(false);
	append(*lblCaption);
    append(*iconSort);
	setShowSort(showSort);

	show();
}

rhea::GridHeader::~GridHeader() {
  delete(this);
}

void rhea::GridHeader::setCaption(const std::string val) {
   lblCaption->set_label(val);
}

std::string rhea::GridHeader::getCaption() {
    return lblCaption->get_label();
}

void rhea::GridHeader::attachWidget(Gtk::Widget* wdg, bool showLabel) {
	GridCell::attachWidget(wdg);
	widget->add_css_class("rhea_gridHeaderWdg");
	iconSort->unparent();
	if (showLabel){
		lblCaption->show();
	} else {
		lblCaption->hide();
	}

	append(*widget);
	append(*iconSort);
	setShowSort(showSort);
	show();

}

Gtk::Widget* rhea::GridHeader::getWidget(){
	if (!widget){
		return lblCaption;
	}
	return widget;
}

void rhea::GridHeader::setIcon(std::string path) {
	if (path.empty()) {return;}
	icon->set_from_resource(path);
	prepend(*icon);
}

void rhea::GridHeader::setIcon(Gtk::Image* img) {
	if (!img) {return;}
	icon = img;
	prepend(*icon);
}

void rhea::GridHeader::setShowSort(bool show) {
    showSort = show;
	if (showSort){
		iconSort->show();
	}else{
		iconSort->hide();
	}
}

bool rhea::GridHeader::doesShowSort() {
    return showSort;
}

void rhea::GridHeader::applyCSS() {
	if (!cssProvider) {return;}
	get_style_context()->add_provider(cssProvider,GTK_STYLE_PROVIDER_PRIORITY_USER);
	lblCaption->get_style_context()->add_provider(cssProvider,GTK_STYLE_PROVIDER_PRIORITY_USER);
	if (widget){
		widget->get_style_context()->add_provider(cssProvider,GTK_STYLE_PROVIDER_PRIORITY_USER);
	}
}
