/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#ifndef RHEA_LISTEDMAP_H
#define RHEA_LISTEDMAP_H

#include "vector"
#include "map"
#include <iterator>
#include <cstddef>

namespace rhea {
template<class T, class C> struct Pair{
    T key   = {};
    C value = {};
};


template<class T, class C> class ListedMap {

private:
    std::vector<T> myVector = {};
    std::map<T,C>  myMap    = {};
public:

    ListedMap();
    ~ListedMap();

    typename std::map<T,C>::iterator begin();
    typename std::map<T,C>::const_iterator begin() const;
    typename std::map<T,C>::iterator end();
    typename std::map<T,C>::const_iterator end() const;

    void add(T key, C value);
    void add(Pair<T,C> pair);
    void insert(size_t index, T key, C value);
    void insert(size_t index, Pair<T,C> pair);
    T getKey(size_t index);
    C getValueAt(size_t index);
    C getValueByKey(T key);
    bool replace(size_t index, C newValue);
    bool replaceValues(C oldValue, C newValue);
    bool replace(T key, C newValue);
    bool replace(size_t index, T key, C newValue);
    bool replace(size_t index, Pair<T,C> pair);
    void removeAt (size_t index);
    void remove (T key);
    void removeValue (C value);
    bool exists(T key);
    bool existsValue(C value);
    long size();
    std::map<T,C> getMap();
    void clear();

};

#include "ListedMap.tpp"

}

#endif //RHEA_LISTEDMAP_H

