/* 
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 * 
 * SPDX-License-Identifier: EUPL-1.2
 */

#include "ListedMap.h"

template<class T, class C>
rhea::ListedMap<T, C>::ListedMap() = default;

template<class T, class C>
rhea::ListedMap<T, C>::~ListedMap() {

}

template<class T, class C>
typename std::map<T,C>::iterator rhea::ListedMap<T, C>::begin() {
	return this->myMap.begin();
}

template<class T, class C>
typename std::map<T,C>::const_iterator rhea::ListedMap<T, C>::begin() const {
	return this->myMap.begin();
}

template<class T, class C>
typename std::map<T,C>::iterator rhea::ListedMap<T, C>::end() {
	return this->myMap.end();
}

template<class T, class C>
typename std::map<T,C>::const_iterator rhea::ListedMap<T, C>::end() const {
	return this->myMap.end();
}

template<class T, class C>
void rhea::ListedMap<T, C>::add(T key, C value) {
    myVector.push_back(key);
    myMap.insert_or_assign(key, value);
}

template<class T, class C>
void rhea::ListedMap<T, C>::add(Pair<T, C> pair) {
   add(pair.key,pair.value);
}

template<class T, class C>
void rhea::ListedMap<T, C>::insert(size_t index, T key, C value) {
    myMap.insert({key, value});
    myVector.insert(myVector.begin()+index, key);
}

template<class T, class C>
void rhea::ListedMap<T, C>::insert(size_t index, Pair<T, C> pair) {
    myMap.insert({pair.key, pair.value});
    myVector.insert(myVector.begin()+index, pair.key);
}

template<class T, class C>
T rhea::ListedMap<T, C>::getKey(size_t index){
	return myVector.at(index);
}

template<class T, class C>
C rhea::ListedMap<T, C>::getValueAt(size_t index) {
    return myMap.at(myVector.at(index));
}

template<class T, class C>
C rhea::ListedMap<T, C>::getValueByKey(T key) {
    return myMap.at(key);
}

template<class T, class C>
bool rhea::ListedMap<T, C>::replaceValues(C oldValue, C newValue) {
    for (auto it = myMap.begin(); it != myMap.end(); ++it) {
        if (it->second == oldValue) {
            return replace(it->first, newValue);
        }
    }
}

template<class T, class C>
bool rhea::ListedMap<T, C>::replace(size_t index, C newValue) {
    if ((index < 0 ) || (index >= myVector.size())) { return false;}
    myMap.insert(myMap.erase(myMap.find(myVector.at(index))),std::make_pair(myVector.at(index),newValue));
    return true;
}

template<class T, class C>
bool rhea::ListedMap<T, C>::replace(T key, C newValue){
    if (myMap.find(key) == myMap.end()) {return false;}
    myMap.insert(myMap.erase(myMap.find(key)),std::make_pair(key,newValue));
    return true;
}

template<class T, class C>
bool rhea::ListedMap<T, C>::replace(size_t index, T key, C newValue) {
    if ((index < 0 ) || (index >= myVector.size())) { return false;}
    myMap.insert(myMap.erase(myMap.find(myVector.at(index))),std::make_pair(key,newValue));
    myVector.at(index) = key;
    return true;
}

template<class T, class C>
bool rhea::ListedMap<T, C>::replace(size_t index, Pair<T, C> pair) {
    return replace(index,pair.key,pair.value);

}


template<class T, class C>
void rhea::ListedMap<T, C>::removeAt(size_t index) {
    if ((index >= 0 ) && (index < myVector.size())) {
        myMap.erase(myVector.at(index));
        myVector.erase(myVector.begin() + index);
    }
}

template<class T, class C>
void rhea::ListedMap<T, C>::remove(T key) {

    int i=0;
    bool found = false;
    for(i = 0; i < myVector.size() ; i++){
        if(myVector.at(i) == key) {
            found = true;
            break;
        }
    }
    if (found) {
        myVector.erase(myVector.begin()+i);
        myMap.erase(key);

    }
}

template<class T, class C>
void rhea::ListedMap<T, C>::removeValue(C value) {
    for (auto it = myMap.begin(); it != myMap.end(); ++it) {
        if (it->second == value) {
            return remove(it->first);
        }
    }

}

template<class T, class C>
bool rhea::ListedMap<T, C>::exists(T key) {
    return myMap.find(key) != myMap.end();
}

template<class T, class C>
bool rhea::ListedMap<T, C>::existsValue(C value) {
    for (auto it = myMap.begin(); it != myMap.end(); ++it) {
        if (it->second == value) {
            return true;
        }
    }
    return false;
}

template<class T, class C>
long rhea::ListedMap<T, C>::size() {
    return myVector.size();
}

template<class T, class C>
std::map<T,C> rhea::ListedMap<T, C>::getMap() {
	return myMap;
}

template<class T, class C>
void rhea::ListedMap<T, C>::clear() {
	myVector.clear();
	myMap.clear();
}