/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#include "Menu.h"

#include <utility>
#include <iostream>
#include "gtkmm.h"

rhea::Menu::Menu() : Gio::Menu() {

}

rhea::Menu::~Menu() {

}

rhea::MenuItem* rhea::Menu::createItem(std::string id, std::string label, std::string action, std::string additional) {
    rhea::MenuItem* menuItem = new rhea::MenuItem;
    menuItem->id = id;
    menuItem->item->set_label(std::move(label));
    if (additional.empty()){
    	menuItem->item->set_action(action);
    } else {
        Glib::Variant<std::string> val = Glib::Variant<std::string>(g_variant_new_string(additional.c_str()));
        menuItem->item->set_action_and_target(action,val);
    }
    menuItem->additinal = additional;
    items.insert({menuItem->id,menuItem});
    return menuItem;
}

void rhea::Menu::setLabel(rhea::MenuItem* item, std::string label) {
    if (item->parent && item->parent->submenu) {
        item->parent->submenu->remove((int)item->index);
    }
    item->item->set_label(std::move(label));
    if (item->parent) {
        item->parent->submenu->insert_item((int)item->index,item->item);
    }

    try {
        if (item->topLevel) {
            this->remove((int) item->index);
            this->insert_item((int) item->index, item->item);
        }
    } catch (...) {
        std::cout << "Error creating label " << label << std::endl;
    }

}

void rhea::Menu::setLabel(std::string id, std::string label) {
    this->setLabel(getItem(std::move(id)),std::move(label));
}

void  rhea::Menu::setAccelator(rhea::MenuItem* item, std::string accel){

//	item->item->set_attribute_value("accel", Glib::Variant("s", accel.c_str()));

}

void rhea::Menu::setAction(rhea::MenuItem* item, std::string action) {
    item->item->set_action(std::move(action));
}

void rhea::Menu::setAction(std::string id, std::string action) {
    setAction(getItem(std::move(id)), std::move(action));
}

void rhea::Menu::setParent(rhea::MenuItem* item, rhea::MenuItem* parent) {
    unparent(item);
    item->parent = parent;
    parent->children.insert({item->id,item});
    item->index = parent->children.size()-1;
    item->topLevel = false;
    if (!parent->submenu){
        parent->submenu->create();
    }
    parent->submenu->append_item(item->item);
}

void rhea::Menu::setParent(std::string id, rhea::MenuItem* parent) {
    setParent(getItem(std::move(id)),parent);
}

void rhea::Menu::setParent(std::string id, std::string parentId) {
    setParent(getItem(std::move(id)),getItem(std::move(parentId)));
}

void rhea::Menu::addChild(rhea::MenuItem* item, rhea::MenuItem* toParent) {
	unparent(item);
    toParent->children.insert({item->id,item});
    item->index = toParent->children.size()-1;
    item->parent = toParent;
    if (item->topLevel){
        std::remove(this->topLevel.begin(), this->topLevel.end(),item);
    }

    item->topLevel= false;
    if (!toParent->submenu){
        toParent->submenu = Gio::Menu::create();
        toParent->item->set_submenu(toParent->submenu);

    }
    toParent->submenu->append_item(item->item);
}

void rhea::Menu::addChild(std::string id, rhea::MenuItem* toParent) {
	addChild(getItem(std::move(id)),toParent);
}

void rhea::Menu::addChild(std::string id, std::string toParentId) {
	addChild(getItem(std::move(id)),getItem(std::move(toParentId)));
}

void rhea::Menu::addChildAt(rhea::MenuItem* item, rhea::MenuItem* parent, long pos) {

}

void rhea::Menu::addChildAt(std::string id, rhea::MenuItem* parent, long pos) {

}

void rhea::Menu::addChildAt(std::string id, std::string parentId, long pos) {

}

void rhea::Menu::putBefore(rhea::MenuItem* item, rhea::MenuItem* before) {

}

void rhea::Menu::putBefore(std::string id, rhea::MenuItem* before) {

}

void rhea::Menu::putBefore(std::string id, std::string beforeId) {

}

void rhea::Menu::putAfter(rhea::MenuItem* item, rhea::MenuItem* after) {

}

void rhea::Menu::putAfter(std::string id, rhea::MenuItem* after) {

}

void rhea::Menu::putAfter(std::string id, std::string afterId) {

}

void rhea::Menu::addToMenu(rhea::MenuItem* item) {
    unparent(item);
    item->topLevel = true;
    if (!item->submenu){
        item->submenu = Gio::Menu::create();
        item->item->set_submenu(item->submenu);
    }
    this->append_item(item->item);
    item->index= this->get_n_items()-1;
    this->topLevel.push_back(item);
}

void rhea::Menu::addToMenu(const Glib::RefPtr<Gio::MenuItem>& item) {
	if (!item) {return;}
	this->append_item(item);
}

void rhea::Menu::addToMenu(std::string id) {
    addToMenu(getItem(std::move(id)));
}

void rhea::Menu::unparent(rhea::MenuItem *item) {
    if (!item) {return;}

	if (item->parent){
        item->parent->submenu->remove(item->index);
        item->parent->children.erase(item->id);
        int i = 0;
        for (auto tmp :item->parent->children){
            tmp.second->index = i;
            i++;
        }
        item->parent = nullptr;
    }
    if (item->topLevel) {
        this->remove(item->index);
        int i = 0;
        for (auto tmp : this->topLevel){
            tmp->index= i;
            i++;
        }
    }
}

void rhea::Menu::unparent(std::string id) {
	rhea::MenuItem* gg = getItem(id);
	if (gg){
		unparent(gg);
	}
}

rhea::MenuItem* rhea::Menu::getItem(std::string id) {
	auto it =items.find(id);
	if (it == items.end()){
    	return nullptr;
    }
    return this->items.at(id);
}

Glib::RefPtr<Gio::Menu> rhea::Menu::getMenu() {
	return  Glib::make_refptr_for_instance<Gio::Menu>(this);
	//   return std::shared_ptr<Gio::Menu>(this);
}

void rhea::Menu::deleteItem(rhea::MenuItem* item){
	unparent(item);
	items.erase(item->id);
}

void rhea::Menu::deleteItem(std::string id){
	unparent(id);
	items.erase(id);
}

