/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#ifndef RHEA_MENU_H
#define RHEA_MENU_H

#include <giomm/menu.h>
#include "map"

namespace rhea {

    struct MenuItem {
        long index= {0};
        std::string id = {""};
        Glib::RefPtr<Gio::MenuItem> item =  Gio::MenuItem::create("", "");
        rhea::MenuItem* parent = {};
        Glib::RefPtr<Gio::Menu> submenu = {nullptr};
        std::map<std::string, rhea::MenuItem*> children = {};
        bool topLevel = false;

        std::string additinal = {};
    };

class Menu : public Gio::Menu {

private:
    std::map<std::string, rhea::MenuItem*> items = {};
    std::vector<rhea::MenuItem*> topLevel = {};

public:
    Menu();
    ~Menu();
    std::string nameXY = {""};
    rhea::MenuItem* createItem(std::string id, std::string label, std::string action, std::string additional = "");
    void setLabel(rhea::MenuItem* item, std::string label);
    void setLabel(std::string id, std::string label);
    void setAccelator(rhea::MenuItem* item, std::string label);
    void setAction(rhea::MenuItem* item, std::string action);
    void setAction(std::string id, std::string action);
    void setParent(rhea::MenuItem* item, rhea::MenuItem* parent);
    void setParent(std::string id, rhea::MenuItem* parent);
    void setParent(std::string id, std::string parentId);
    void addChild(rhea::MenuItem* item, rhea::MenuItem* toParent);
    void addChild(std::string id, rhea::MenuItem* toParent);
    void addChild(std::string id, std::string toParentId);
    void addChildAt(rhea::MenuItem* item, rhea::MenuItem* parent, long pos);
    void addChildAt(std::string id, rhea::MenuItem* parent, long pos);
    void addChildAt(std::string id, std::string parentId, long pos);
    void putBefore(rhea::MenuItem* item, rhea::MenuItem* before);
    void putBefore(std::string id, rhea::MenuItem* before);
    void putBefore(std::string id, std::string beforeId);
    void putAfter(rhea::MenuItem* item, rhea::MenuItem* after);
    void putAfter(std::string id, rhea::MenuItem* after);
    void putAfter(std::string id, std::string afterId);
    void addToMenu(rhea::MenuItem* item);
    void addToMenu(std::string id);
    void unparent (rhea::MenuItem* item);
    void unparent (std::string id);
    void deleteItem(rhea::MenuItem* item);
    void deleteItem(std::string id);
    rhea::MenuItem* getItem(std::string id);
    Glib::RefPtr<Gio::Menu> getMenu();
    void addToMenu(const Glib::RefPtr<Gio::MenuItem>& item);

};

}

#endif //RHEA_MENU_H
