/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#include "PickupDT.h"
#include <iostream>
rhea::PickupDT::PickupDT() : Gtk::Box(Gtk::Orientation::HORIZONTAL) {

    popBox = new Gtk::Popover();
    text = new Gtk::TextView();
    text->set_editable(false);
    std::string iconPath =getenv("GRUNF");
    btn = new Gtk::Button();
    Gtk::Image* icon = new Gtk::Image();
    iconPath += "/res/PickupBtn.png";
    icon->set(iconPath);
    btn->set_child(*icon);
    calendar = new Gtk::Calendar();
    oldDate.create_now_local();
    newDate.create_now_local();
    btn->signal_clicked().connect( sigc::mem_fun(*this,&PickupDT::onBtnClicked));
    calendar->signal_day_selected().connect(sigc::mem_fun(*this,&PickupDT::on_my_day_selected));
    popBox->set_child(*calendar);
    popBox->set_parent(*btn);
    append(*text);
    append(*btn);
    setSize(352,35);

}

void rhea::PickupDT::onBtnClicked() {
    popBox->popup();
}

void rhea::PickupDT::on_my_day_selected(){

    newDate = calendar->get_date();
    popBox->popdown();
    std::string str = std::to_string(newDate.get_day_of_month())+". "+std::to_string(newDate.get_month())+". "+std::to_string(newDate.get_year());
    text->get_buffer()->set_text(str);

    if(onChange){
        onChange(oldDate,newDate);
    }
    oldDate = newDate;

}

Glib::DateTime rhea::PickupDT::getDate(){
    return newDate;
}

std::string rhea::PickupDT::getAsISO8601() {
//ISO8601 format je sledeči YYYY-MM-DD HH:MM:SS.SSS

    std::string day  = std::to_string(newDate.get_day_of_month());
    std::string mon  = std::to_string(newDate.get_month());
    std::string year = std::to_string(newDate.get_year());
    std::string hour = std::to_string(newDate.get_hour());
    std::string min  = std::to_string(newDate.get_minute());
    std::string sec  = std::to_string(newDate.get_second());

    day  = day.length()  < 2 ? "0" + day  : day;
    mon  = mon.length()  < 2 ? "0" + mon  : mon;
    hour = hour.length() < 2 ? "0" + hour : hour;
    min  = min.length()  < 2 ? "0" + min  : min;
    sec  = sec.length()  < 2 ? "0" + sec  : sec;

    return year + "-" + mon + "-" + day + "T" + hour + ":" + min + ":" + sec + "+00" ;
}

rhea::PickupDT::~PickupDT() {

}

void rhea::PickupDT::setSize(int width, int height) {
    this->width = width;
    this->height = height;
    text->set_size_request(width-52,height);
    btn->set_size_request(35,height);
    set_size_request(width,height);
}

void rhea::PickupDT::setValue(Glib::DateTime val){
	if (val.get_day_of_month()>0) {
		text->get_buffer()->set_text( std::to_string(val.get_day_of_month()) + ". " + std::to_string(val.get_month()) + ". " + std::to_string(val.get_year()));
	} else {
		text->get_buffer()->set_text("");
	}
	calendar->property_day().set_value(val.get_day_of_month());
	calendar->property_month().set_value(val.get_month()-1);
	calendar->property_year().set_value(val.get_year());
}

void rhea::PickupDT::setValue(std::string iso8601){
	setValue(Glib::DateTime::create_from_iso8601(iso8601));
}

void rhea::PickupDT::onChangeConnect(onGlibDateChangeCallBack_function pFunc) {
    onChange = pFunc;
}
