/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#include <iostream>
#include "DbConn.h"

rhea::DbConn::DbConn(){

}

rhea::DbConn::~DbConn() {

}

void rhea::DbConn::setFile(std::string file) {
  DB_fileName = file;
}

std::string &rhea::DbConn::getFile() {
    return DB_fileName;
}

sqlite3* rhea::DbConn::getDB(){
	return db;
}

rhea::DbConn* rhea::DbConn::getInstance() {
    ready = false;

    if (!instance) {
        instance = new DbConn;
    }
    ready = true;
    return instance;
}

sqlite3* rhea::DbConn::openDataBase() {
    int res;
    res = sqlite3_open(DB_fileName.c_str(), &db);
    if( res ) {
        fprintf(stderr, "Can't open database: %s\n", sqlite3_errmsg(db));
        return(nullptr);
    }

    if (!db) {return nullptr;}

    setSettings();

    sqlite3_stmt* stmt = {nullptr};
    std::string sql = "SELECT version FROM Settings;";
    sqlite3_prepare_v2(db,sql.c_str(),sql.size(),&stmt, nullptr);

    while ( sqlite3_step( stmt ) == SQLITE_ROW ) { // While query has result-rows.
        DB_VERSION =  std::string(reinterpret_cast<const char*>(sqlite3_column_text( stmt, 0 )));
    }

    sqlite3_finalize(stmt);

    updateDb();

    return db;
}

void rhea::DbConn::setSettings(){
    sqlite3_stmt* stmt = {nullptr};

    std::string sql = "SELECT COUNT(*) as init FROM sqlite_master WHERE type = ? AND name = ?";
    sqlite3_prepare_v2(db,sql.c_str(),sql.size(),&stmt, nullptr);

    std::string prmTable= "table";
    sqlite3_bind_text( stmt, 1, prmTable.c_str(),prmTable.size(), nullptr);

    std::string prmSettings= "Settings";
    sqlite3_bind_text( stmt, 2, prmSettings.c_str(),prmSettings.size(), nullptr);

    bool initialSetup = false;
    while ( sqlite3_step( stmt ) == SQLITE_ROW ) { // While query has result-rows.
        initialSetup = *sqlite3_column_text( stmt, 0 ) == '0';
    }

    sqlite3_finalize(stmt);

    if( initialSetup ){
        std::string str = "CREATE TABLE Settings (version TEXT);";
        if (!execQuery(str)){
            return;
        }

        str = "INSERT INTO Settings VALUES ('0');";
        if (!execQuery(str)){
            return;
        }
    }
}

void rhea::DbConn::updateDb(){

    if (onUpdateCallback){
    	onUpdateCallback();
    }
    getVersion();
}

std::string rhea::DbConn::getVersion() {
	rhea::DbResultSet* res = execQuery("SELECT version FROM Settings;");
	if (res->rowCount()>0){
		while(!res->isEof()){
	    	DB_VERSION = res->getField("version")->getAsString();
	        res->moveNext();
	    }
	}
	return DB_VERSION;
}

/*
bool rhea::DbConn::execQuery(std::string sql){
    if (db) {
        ready = false;
        int res = sqlite3_exec(db, sql.c_str(), NULL, NULL, NULL);
       if (res != SQLITE_OK) {
            sqlite3_close(db);
            return false;
        }    }
    ready = true;
    return true;
}
*/

rhea::DbResultSet* rhea::DbConn::execQuery(std::string sql, std::vector<rhea::DbAtribute>* params) {
    rhea::DbResultSet* res = new rhea::DbResultSet(db, sql,params);
    return res;
}
