/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#ifndef RHEA_DBCONN_H
#define RHEA_DBCONN_H

#include <functional>
#include <string>
#include "vector"
#include "sqlite3.h"
#include "DbResultSet.h"
#include "DbParameters.h"

namespace rhea {

    class DbConn {
    private:
    	std::function<void()> onUpdateCallback = {nullptr};
        DbConn();
        inline static bool ready;
        inline static DbConn *instance;
        std::string DB_fileName = {"something.db"};
        std::string DB_VERSION  = {"-1"};
        sqlite3* db;
        void setSettings();

    public:
        ~DbConn();
        void setFile(std::string file);
        std::string &getFile();
        sqlite3* getDB();
        sqlite3* openDataBase();
        static DbConn *getInstance();
        rhea::DbResultSet* execQuery(std::string sql, std::vector<rhea::DbAtribute>* params = nullptr);
        void updateDb();
        std::string getVersion();
        template<typename A, typename B> void setOnUpdate(A funcPtr, B objPtr) { onUpdateCallback = std::bind(funcPtr, objPtr);};
    };

}
#endif //RHEA_DBCONN_H
