/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#ifndef RHEA_DBRESULTSET_H
#define RHEA_DBRESULTSET_H

#include "vector"
#include "DbAtribute.h"
#include "sqlite3.h"

namespace rhea {
    class DbResultSet {
    private:
        std::vector<DbAtributeMetaData> metaFields;
        std::vector<std::vector<DbAtribute*>>results;
        int currentRow = {-1};

    public:
        explicit DbResultSet(sqlite3 *db, const std::string& sql,std::vector<DbAtribute>* params = nullptr);
        ~DbResultSet();
        int rowCount();
        int columnCount();
        int getRow();
        void setRow(int val);
        void movePrevious();
        void moveNext();
        void moveFirst();
        void moveLast();
        bool isBof() const;
        bool isEof() const;
        rhea::DbAtribute* getField(int row, int col);
        rhea::DbAtribute* getField(int col);
        rhea::DbAtribute* getField(std::string name);
        std::string getFieldName(int index);
        int getFieldType(int index);
        int getFieldType(std::string name);
    };

}

#endif //RHEA_DBRESULTSET_H
