/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#include "WndHeader.h"

rhea::WndHeader::WndHeader() {
	lblCaption->set_label("");
	setImgX();
	std::shared_ptr<Gtk::GestureClick> click =  Gtk::GestureClick::create();
	click->signal_released().connect(sigc::mem_fun(*this, &WndHeader::OnClick));
	clickBox->add_controller(click);
	clickBox->set_focus_on_click();
}

rhea::WndHeader::~WndHeader() {
    headerBox->unparent();
    delete(lblCaption);
    delete(imgIcon);
    delete(imgX);
    if (myStyle == WndHeaderStyle::VERT) {
        delete (vBox);
        delete (hBox);
    }else{
        delete (hBox);
        delete (vBox);
    }
    delete(headerBox);
}

void rhea::WndHeader::setIndex(const int val) {
    myIndex = val;
}

int rhea::WndHeader::getIndex() {
    return myIndex;
}


Gtk::Box* rhea::WndHeader::DrawHeader() {
    if (imgIcon->get_parent()) {
        imgIcon->unparent();
    }
    if(lblCaption->get_parent()) {
        lblCaption->unparent();
    }
    if(imgX->get_parent()) {
        imgX->unparent();
    }
    if(vBox->get_parent()) {
        vBox->unparent();
    }
    if (hBox->get_parent()) {
        hBox->unparent();
    }
    if (myStyle == WndHeaderStyle::VERT) {
        vBox->append(*imgIcon);
        vBox->append(*lblCaption);
        hBox->append(*vBox);

    }else{
        hBox->append(*imgIcon);
        hBox->append(*lblCaption);
    }

    hBox->append(*imgX);
    clickBox->append(*hBox);
    if (!clickBox->get_parent()) {
        headerBox->append(*clickBox);
    }
    return headerBox;
}

void rhea::WndHeader::setStyle(rhea::WndHeaderStyle style) {
    if(myStyle == style) {return;}
    myStyle = style;
    DrawHeader();
}

void rhea::WndHeader::setCaption(const std::string text) {
    lblCaption->set_label(text);
}

void rhea::WndHeader::setIcon(const std::string &filename) {
//    setIcon(filename,iconSize);
    imgIcon->set(filename);
    imgIcon->queue_draw();
}

//TO-DO zapakiraj v obj header mogoče? ali relativno pot
void rhea::WndHeader::setImgX() {
	imgX->add_css_class("imgX");
	imgX->set_from_icon_name("window-close"); //"edit-clear-all"
    imgX->queue_draw();
}

Gtk::Image* rhea::WndHeader::getImgX() {
	return imgX;
}

void rhea::WndHeader::setClose(boolCallBack_function pFunc) {
    myClose = pFunc;
}

void rhea::WndHeader::callClose() {

    if (myClose){
        myClose(myIndex);
    }
}

void rhea::WndHeader::setShow(voidCallBack_function pFunc) {
    myShow = pFunc;
}

void rhea::WndHeader::callShow() {
    if (myShow) {
        myShow(myIndex);
    }
}


Gtk::Box *rhea::WndHeader::getHeader() {
    return headerBox;
}

void rhea::WndHeader::OnClick(int button, double x, double y){
    if (x >= headerBox->get_width() - imgX->get_width()-5){
        callClose();
    }else{
        callShow();
    }
}
