/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#ifndef RHEA_WNDHEADER_H
#define RHEA_WNDHEADER_H

#include "gtkmm.h"

namespace rhea {

typedef void (* voidCallBack_function)(int);
typedef bool (* boolCallBack_function)(int);

enum class WndHeaderStyle { VERT, HORZ };

class WndHeader {
private:
    int                   myIndex    = {0};
    WndHeaderStyle        myStyle    = WndHeaderStyle::VERT;
    boolCallBack_function myClose    = {nullptr};
    voidCallBack_function myShow     = {nullptr};
    Gtk::Box*             headerBox  = new Gtk::Box(Gtk::Orientation::HORIZONTAL);
    Gtk::Box*             vBox       = new Gtk::Box(Gtk::Orientation::VERTICAL);
    Gtk::Box*             hBox       = new Gtk::Box(Gtk::Orientation::HORIZONTAL);
    Gtk::Label*           lblCaption = new Gtk::Label();
    Gtk::Image*           imgIcon    = new Gtk::Image();
    Gtk::Image*           imgX       = new Gtk::Image();
    Gtk::Box*             clickBox   = new Gtk::Box();
    void setImgX();
    void OnClick(int button, double x, double y);

public:
    WndHeader();
    ~WndHeader();
    Gtk::Box* DrawHeader();
    Gtk::Box* getHeader();
    void setIndex(int val);
    int getIndex();
    Gtk::Image* getImgX();
    void setStyle(WndHeaderStyle style);
    void setCaption(std::string text);
    void setIcon(const std::string &filename);
    void setClose( boolCallBack_function pFunc);
    void setShow( voidCallBack_function pFunc);
    void callClose();
    void callShow();

};

}
#endif //RHEA_WNDHEADER_H
