/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#include "iostream"
#include "WndManager.h"

int rhea::WndManager::countIndex = {0};

std::map<int, rhea::glueContainer*> rhea::WndManager::contains = {};

rhea::WndManager::WndManager() {
	cssData  = ".mngrWindowHeader {border-radius: 5px 15px 0px 0px; background: #6a88a0; } .mngrWindowHeader:hover {background-color: #D0D0EF;} ";
	cssData += ".mngrActiveFrame {background-color: #abb8c2; } .imgX {margin-left:3px; margin-right:7px;}";
}

rhea::WndManager::~WndManager() {
    delete(headerBox);
    delete(mainBox);
}

void rhea::WndManager::setHeaderStyle(rhea::WndHeaderStyle style) {
    headerStyle = style;
    for(auto item : WndManager::contains){
        item.second->header->setStyle(headerStyle);
    }

}

Gtk::Widget* rhea::WndManager::getToolBox(){
  return headerBox;
}

Gtk::Box* rhea::WndManager::getMainBox(){
    return mainBox;
}

void rhea::WndManager::addFrame(Gtk::Frame* frame, std::string caption) {
    frame->set_name(frame->get_name()+"_"+std::to_string(countIndex));
    rhea::WndHeader* hdr = new rhea::WndHeader();
//    hdr->setIcon(Gtk::Stock::HARDDISK);
    std::vector<Glib::ustring> myCSS = {"mngrWindowHeader", "mngrActiveFrame"};
    hdr->getHeader()->set_css_classes(myCSS);
    hdr->getHeader()->get_style_context()->add_provider_for_display(display,cssProvider,GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
    hdr->setStyle(headerStyle);
    hdr->setCaption(caption);
    hdr->setIndex(countIndex);
    hdr->setClose(WndManager::removeFrame);
    hdr->setShow(&WndManager::showFrame);
    hdr->getImgX()->get_style_context()->add_provider_for_display(display,cssProvider,GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
    rhea::glueContainer* cntnr = new rhea::glueContainer();
    cntnr->frmManager = this;
    cntnr->header=hdr;
    cntnr->frame =frame;
    contains.insert({countIndex,cntnr});
    headerBox->append(*hdr->DrawHeader());
    countIndex ++;
    cssProvider->load_from_data(cssData);
}

// unparent uporabljaj skozi hide. in ne direktno. Razmisli kateri frame bi prikazal ko aktivnega zbrises.
bool rhea::WndManager::removeFrame(Gtk::Frame* frame) {
    int i = -1;

    for(auto item : contains){
        if(item.second->frame == frame) {
            if (item.second->toClose && item.second->toClose()) {
                item.second->frame->unparent();
                delete (item.second->frame);
                delete (item.second->header);
                if (item.first == item.second->frmManager->currIndex){
                	item.second->frmManager->currIndex = -1;
                }
                i = item.first;
                break;
            }
        }
    }
    if (i>-1){
        contains.erase(i);
        if (!contains.empty()){
        	showFrame(contains.rbegin()->second->frame);
        }
        return true;
    }
    return false;
}

bool rhea::WndManager::removeFrame(int index) {
    auto item = contains.find(index);
    if (item != contains.end()) {
        return removeFrame(item->second->frame);
        item->second->header->getHeader()->remove_css_class("mngrActiveFrame");
    }
    return false;
}

void rhea::WndManager::showFrame(Gtk::Frame *frame) {
    for(auto item : contains){
        if(item.second->frame == frame) {
            hideFrame(item.second->frmManager);
            item.second->frmManager->mainBox->append(*item.second->frame);
            if(item.second->frmManager->currIndex !=item.second->header->getIndex()) {
                item.second->toActivate();
            }
            item.second->frmManager->currIndex = item.second->header->getIndex();
            item.second->header->getHeader()->add_css_class("mngrActiveFrame");
            item.second->frmManager->getMainBox()->show();
            break;
        }
    }
}

void rhea::WndManager::showFrame(int val) {
    WndManager* mngr = contains.find(val)->second->frmManager;
    hideFrame(mngr);
    mngr->getMainBox()->append(*(contains.find(val)->second->frame));
    if (mngr->currIndex != val){
        contains.find(val)->second->toActivate();
    }
    contains.find(val)->second->header->getHeader()->add_css_class("mngrActiveFrame");
    mngr->currIndex = val;
    mngr->getMainBox()->show();
}

void rhea::WndManager::hideFrame(WndManager* mngr) {
    if( mngr->currIndex > -1) {
    	contains.find(mngr->currIndex)->second->header->getHeader()->remove_css_class("mngrActiveFrame");
    	mngr->getMainBox()->remove(*(contains.find(mngr->currIndex)->second->frame));

        mngr->currIndex = -1;
    }
}

void rhea::WndManager::addCSS(std::string css){
	cssData +=css;
}

void rhea::WndManager::clearCSS() {
	cssData = "";
}
