/*
 * SPDX-FileCopyrightText: 2022 Institute Rhea <eduard.filipas@rhea.si>
 *
 * SPDX-License-Identifier: EUPL-1.2
 */

#ifndef RHEA_WNDMANAGER_H
#define RHEA_WNDMANAGER_H


#include "map"
#include "gtkmm.h"
#include "WndHeader.h"

namespace rhea {

typedef std::function< bool ()> callbackCloseFrame;
typedef std::function< void ()> callbackActivateFrame;

class WndManager;

struct glueContainer{
    rhea::WndManager* frmManager     = {nullptr};
    rhea::WndHeader*  header         = {nullptr};
    Gtk::Frame*       frame          = {nullptr};
    callbackCloseFrame toClose       = {nullptr};
    callbackActivateFrame toActivate = {nullptr};
};


class WndManager {
private:
	Glib::RefPtr<Gdk::Display> display = Gdk::Display::get_default();
	Glib::RefPtr<Gtk::CssProvider> cssProvider = Gtk::CssProvider::create();
	static int                            countIndex;
    static std::map<int, glueContainer *> contains;
    int                  currIndex    = {-1};
    int                  headerSize   = {3};
    std::string          cssData      = "";
    rhea::WndHeaderStyle headerStyle  = {rhea::WndHeaderStyle::VERT};
    Gtk::Box*            headerBox    = new Gtk::Box();
    Gtk::Box*            mainBox      = new Gtk::Box();

public:
    WndManager();
    ~WndManager();
    void setHeaderStyle(rhea::WndHeaderStyle  style);
    void addFrame(Gtk::Frame *frame, std::string caption);
    static void showFrame(Gtk::Frame* frame);
    static void showFrame(int index);
    static bool removeFrame(Gtk::Frame* frame);
    static bool removeFrame(int index);
    static void hideFrame(rhea::WndManager* mngr);
    Gtk::Widget *getToolBox();
    Gtk::Box* getMainBox();
    template<typename A, typename B>
    void setOnClose(A objPtr, B funcPtr);
    template<typename A, typename B>
    void setOnActivate(A objPtr, B funcPtr);
    void addCSS(std::string css);
    void clearCSS();
};

#include "WndManager.tpp"

}

#endif //RHEA_WNDMANAGER_H
